/* devicesidebar.cc
 * Copyright (C) 2001 The University of Newcastle upon Tyne
 *
 * Written by Paul Coates (Paul.Coates@ncl.ac.uk)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 */

#include <gtk/gtk.h>
#include <arpa/inet.h>
#include <unistd.h>
#include <string.h>
#include <stdio.h>
#include <netdb.h>
#include <pwd.h>
#include "devicesidebar.h"
#include "preferences.h"
#include "device.h"
#include "main.h"
#include "session.h"
#include "monitor.h"
#include "data.h"
#include "scheduler.h"
#include "topology.h"
#include "gui.h"
#include "stats.h"
#include "devicedetails.h"
#include "ipaddress.h"
#include "features.h"
#include "sbrouter.xpm"
#include "srouter.xpm"
#include "sswitch.xpm"
#include "sunknown.xpm"

static void select_cb(GtkWidget *clist, gint row, gint col,
	GdkEventButton *e, gpointer data)
{ ((DeviceSideBar*)data)->select(row); }
static void unselect_cb(GtkWidget *clist, gint row, gint col,
	GdkEventButton *e, gpointer data)
{ ((DeviceSideBar*)data)->unselect(row); }
static void graph_cb(GtkWidget *w, gpointer data)
{ ((DeviceSideBar*)data)->graph(); }
static void add_cb(GtkWidget *w, gpointer data)
{ ((DeviceSideBar*)data)->add(); }
static void edit_cb(GtkWidget *w, gpointer data)
{ ((DeviceSideBar*)data)->edit(); }
static void delete_cb(GtkWidget *w, gpointer data)
{ ((DeviceSideBar*)data)->main->topology->deviceDeleteSelected(); }
static void hide_cb(GtkWidget *w, gpointer data)
{
	((DeviceSideBar*)data)->main->session->data->hide();
	((DeviceSideBar*)data)->main->gui->update_views();
}
static void show_cb(GtkWidget *w, gpointer data)
{
	((DeviceSideBar*)data)->main->session->data->show();
	((DeviceSideBar*)data)->main->gui->update_views();
}
static void merge_cb(GtkWidget *w, gpointer data)
{ ((DeviceSideBar*)data)->main->topology->deviceMerge(); }
//static void graph_cb(GtkWidget *w, gpointer data)
//{ ((DeviceSideBar*)data)->main->gui->graph_window(); }
static void autozoom_cb(GtkWidget *w, gpointer data)
{
	DeviceSideBar *v = (DeviceSideBar*)data;
	v->autozoom=((v->autozoom=='0')?'1':'0');
	if (v->autozoom=='1') v->main->gui->update_views();
}
static void regrab_cb(GtkWidget *w, gpointer data)
{ ((DeviceSideBar*)data)->main->scheduler->regrabSelectedDevices(); }
static void community_cb(GtkWidget *w, gpointer data)
{ ((DeviceSideBar*)data)->main->gui->preferences->show_community(); }

DeviceSideBar::DeviceSideBar(Main *m)
{
  GtkWidget *label183;
  GtkWidget *hbox90;
  GtkWidget *button94;
  GtkWidget *scrolledwindow34;
  GtkWidget *hbox89;

	main = m;
	row = -1;

  vbox67 = gtk_vbox_new (FALSE, 2);
  gtk_widget_show (vbox67);
  gtk_container_set_border_width (GTK_CONTAINER (vbox67), 2);

  label183 = gtk_label_new ("Device List");
  gtk_widget_show (label183);
  gtk_box_pack_start (GTK_BOX (vbox67), label183, FALSE, FALSE, 0);
  gtk_misc_set_alignment (GTK_MISC (label183), 0, 0.5);

  hbox90 = gtk_hbox_new (FALSE, 2);
  gtk_widget_show (hbox90);
  gtk_box_pack_start (GTK_BOX (vbox67), hbox90, FALSE, FALSE, 0);

  entry42 = gtk_entry_new ();
  gtk_widget_show (entry42);
  gtk_box_pack_start (GTK_BOX (hbox90), entry42, TRUE, TRUE, 0);
  gtk_widget_set_usize (entry42, 1, -2);
	gtk_signal_connect(GTK_OBJECT(entry42), "activate",
		GTK_SIGNAL_FUNC(add_cb), this);

  button94 = gtk_button_new_with_label ("Add");
  gtk_widget_show (button94);
  gtk_box_pack_start (GTK_BOX (hbox90), button94, FALSE, FALSE, 0);
  gtk_signal_connect (GTK_OBJECT (button94), "clicked",
		GTK_SIGNAL_FUNC (add_cb), this);

  button97 = gtk_button_new ();
  gtk_widget_show (button97);
  gtk_box_pack_start (GTK_BOX (hbox90), button97, FALSE, FALSE, 0);
	gtk_signal_connect (GTK_OBJECT (button97), "clicked",
		GTK_SIGNAL_FUNC (community_cb), this);

  scrolledwindow34 = gtk_scrolled_window_new (NULL, NULL);
  gtk_widget_show (scrolledwindow34);
  gtk_box_pack_start (GTK_BOX (vbox67), scrolledwindow34, TRUE, TRUE, 0);
  gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolledwindow34),
		GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);

  clist31 = gtk_clist_new (1);
  gtk_widget_show (clist31);
  gtk_container_add (GTK_CONTAINER (scrolledwindow34), clist31);
  gtk_clist_set_column_width (GTK_CLIST (clist31), 0, 16);
  gtk_clist_column_titles_hide (GTK_CLIST (clist31));
	gtk_clist_set_selection_mode (GTK_CLIST (clist31), GTK_SELECTION_EXTENDED);
	gtk_signal_connect(GTK_OBJECT(clist31), "select_row",
		GTK_SIGNAL_FUNC(select_cb), this);
	gtk_signal_connect(GTK_OBJECT(clist31), "unselect_row",
		GTK_SIGNAL_FUNC(unselect_cb), this);

  hbox89 = gtk_hbox_new (TRUE, 2);
  gtk_widget_show (hbox89);
  gtk_box_pack_start (GTK_BOX (vbox67), hbox89, FALSE, TRUE, 0);

  button95 = gtk_button_new_with_label ("Properties");
  gtk_widget_show (button95);
  gtk_box_pack_start (GTK_BOX (hbox89), button95, FALSE, TRUE, 0);
  gtk_signal_connect (GTK_OBJECT (button95), "clicked",
		GTK_SIGNAL_FUNC (edit_cb), this);

	button96 = gtk_button_new_with_label ("Graph");
	gtk_widget_show (button96);
	gtk_box_pack_start (GTK_BOX (hbox89), button96, FALSE, TRUE, 0);
	gtk_signal_connect (GTK_OBJECT (button96), "clicked",
		GTK_SIGNAL_FUNC (graph_cb), this);

  hbox89 = gtk_hbox_new (TRUE, 2);
  gtk_widget_show (hbox89);
  gtk_box_pack_start (GTK_BOX (vbox67), hbox89, FALSE, TRUE, 0);

  button98 = gtk_button_new_with_label ("Regrab");
  gtk_widget_show (button98);
  gtk_box_pack_start (GTK_BOX (hbox89), button98, FALSE, TRUE, 0);
  gtk_signal_connect (GTK_OBJECT (button98), "clicked",
		GTK_SIGNAL_FUNC (regrab_cb), this);

  button99 = gtk_button_new_with_label ("Delete");
	gtk_widget_show (button99);
  gtk_box_pack_start (GTK_BOX (hbox89), button99, FALSE, TRUE, 0);
  gtk_signal_connect (GTK_OBJECT (button99), "clicked",
		GTK_SIGNAL_FUNC (delete_cb), this);

  hbox89 = gtk_hbox_new (TRUE, 2);
  gtk_widget_show (hbox89);
  gtk_box_pack_start (GTK_BOX (vbox67), hbox89, FALSE, TRUE, 0);

  button100 = gtk_button_new_with_label ("Merge");
  gtk_widget_show (button100);
  gtk_box_pack_start (GTK_BOX (hbox89), button100, FALSE, TRUE, 0);
  gtk_signal_connect (GTK_OBJECT (button100), "clicked",
		GTK_SIGNAL_FUNC (merge_cb), this);

  button101 = gtk_button_new_with_label ("Split");
	gtk_widget_show (button101);
  gtk_box_pack_start (GTK_BOX (hbox89), button101, FALSE, TRUE, 0);
//  gtk_signal_connect (GTK_OBJECT (button101), "clicked",
//		GTK_SIGNAL_FUNC (delete_cb), this);

  hbox89 = gtk_hbox_new (TRUE, 2);
  gtk_widget_show (hbox89);
  gtk_box_pack_start (GTK_BOX (vbox67), hbox89, FALSE, TRUE, 0);

  button102 = gtk_button_new_with_label ("Show");
  gtk_widget_show (button102);
  gtk_box_pack_start (GTK_BOX (hbox89), button102, FALSE, TRUE, 0);
  gtk_signal_connect (GTK_OBJECT (button102), "clicked",
		GTK_SIGNAL_FUNC (show_cb), this);

  button103 = gtk_button_new_with_label ("Hide");
	gtk_widget_show (button103);
  gtk_box_pack_start (GTK_BOX (hbox89), button103, FALSE, TRUE, 0);
  gtk_signal_connect (GTK_OBJECT (button103), "clicked",
		GTK_SIGNAL_FUNC (hide_cb), this);

  checkbutton14 = gtk_check_button_new_with_label ("Auto Zoom");
  gtk_widget_show (checkbutton14);
  gtk_box_pack_start (GTK_BOX (vbox67), checkbutton14, FALSE, TRUE, 0);
  gtk_signal_connect (GTK_OBJECT (checkbutton14), "clicked",
		GTK_SIGNAL_FUNC (autozoom_cb), this);

	autozoom = '0';
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(checkbutton14),
		(autozoom=='1')?TRUE:FALSE);

	gtk_widget_set_sensitive(button95, FALSE);
	gtk_widget_set_sensitive(button96, FALSE);
	gtk_widget_set_sensitive(button98, FALSE);
	gtk_widget_set_sensitive(button99, FALSE);
	gtk_widget_set_sensitive(button100, FALSE);
	gtk_widget_set_sensitive(button101, FALSE);
	gtk_widget_set_sensitive(button102, FALSE);
	gtk_widget_set_sensitive(button103, FALSE);
}

DeviceSideBar::~DeviceSideBar()
{
}

void DeviceSideBar::select(int r)
{
	Device *d;
	row = r;
	d = (Device*) gtk_clist_get_row_data(GTK_CLIST(clist31), row);
//	printf("SELECTED device %s\n", d->name);
	d->selection = TRUE;
	buttons();
	if (autozoom=='1') main->gui->monitor->viewer->fitToSelected();
	mergeButton(main->session->data->readyToMerge());
	main->gui->update_views();
}

void DeviceSideBar::unselect(int r)
{
	Device *d;

	d = (Device*) gtk_clist_get_row_data(GTK_CLIST(clist31), r);
//	printf("UNSELECTED device %s\n", d->name);
	d->selection = FALSE;
	buttons();
	if (autozoom=='1') main->gui->monitor->viewer->fitToWindow();
	mergeButton(main->session->data->readyToMerge());
	main->gui->update_views();
}

void DeviceSideBar::buttons()
{
	Link *ln;
	Device *d;
	gboolean match = FALSE;

	for (ln=main->session->data->devices.get_head(); ln!=NULL; ln=ln->next)
	{
		d = (Device*) ln->item;
		if (d->selection==TRUE) match = TRUE;
	}
	gtk_widget_set_sensitive(button95, match);
	gtk_widget_set_sensitive(button96, match);
	gtk_widget_set_sensitive(button98, match);
	gtk_widget_set_sensitive(button99, match);
	gtk_widget_set_sensitive(button100, match);
	gtk_widget_set_sensitive(button101, match);
	gtk_widget_set_sensitive(button102, match);
	gtk_widget_set_sensitive(button103, match);
}

void DeviceSideBar::add()
{
	IPAddress tip;
	gchar *buf, *buf2, *csl;
	struct hostent *h;

	buf = gtk_entry_get_text(GTK_ENTRY(entry42));
	gtk_entry_set_text(GTK_ENTRY(entry42), "");

	// convert name/ip into ip
	if ((h=gethostbyname(buf)) == NULL)
	{
		main->gui->monitor->show_message("Illegal host entry");
		return;
	}
	tip.setstring(inet_ntoa(*((struct in_addr *)h->h_addr)));
	if (tip.zero() || tip.loopback())
	{
		main->gui->monitor->show_message("Illegal host entry");
		return;
	}
	main->topology->create_placeholder(buf, tip);

	if (!main->session->devicebit)
	{
		main->session->devicebit=TRUE;
		main->setTitle();
	}

	csl = get_community_list_string(main->client);
	buf2 = g_strjoin(" ", tip.getstring(), csl, NULL);
	main->scheduler->addTask(buf2, 1, 1, 0);
	g_free(buf2);
	g_free(csl);
}

void DeviceSideBar::edit()
{
	main->gui->devicedetails->device = NULL;
	if (row!=-1)
		main->gui->devicedetails->device =
			(Device*) gtk_clist_get_row_data(GTK_CLIST(clist31), row);
	main->gui->devicedetails->show();
}

void DeviceSideBar::graph()
{
	Device *d;
	Stats *s;

	if (row==-1) return;
	d = (Device*) gtk_clist_get_row_data(GTK_CLIST(clist31), row);
	s = new Stats(main);
	gtk_entry_set_text(GTK_ENTRY(GTK_COMBO(s->combo3)->entry), d->name);
	s->device_list();
	s->show();
	main->gui->stats.tag(s);
}

void DeviceSideBar::update()
{
	Device *d;
	Link *ln;
	gchar *data[1];
	int i, row;

	row=0;
	gtk_clist_freeze(GTK_CLIST (clist31));
	gtk_clist_clear(GTK_CLIST (clist31));
	for (ln=main->session->data->devices.get_head(); ln!=NULL; ln=ln->next)
	{
		d = (Device*) ln->item;
		data[0] = strdup("");
		gtk_clist_append(GTK_CLIST (clist31), data);
		delete data[0];
		if (d->subtype==device_router)
			clist_pixtext(vbox67, GTK_CLIST(clist31), row, 0, (gchar **)srouter_xpm, d->name);
		else
		if (d->subtype==device_switch)
			clist_pixtext(vbox67, GTK_CLIST(clist31), row, 0, (gchar **)sswitch_xpm, d->name);
		else
		if (d->subtype==device_merged)
			clist_pixtext(vbox67, GTK_CLIST(clist31), row, 0, (gchar **)sbrouter_xpm, d->name);
		else
			clist_pixtext(vbox67, GTK_CLIST(clist31), row, 0, (gchar **)sunknown_xpm, d->name);
		gtk_clist_set_row_data(GTK_CLIST(clist31), row, d);
		row++;
	}
	gtk_clist_set_sort_column (GTK_CLIST (clist31), 0);
	gtk_clist_sort (GTK_CLIST (clist31));
	gtk_clist_thaw(GTK_CLIST (clist31));

	// now sorting is out of the way we can add background color
	for (i=0; i<row; i++)
	{
		gtk_clist_set_background(GTK_CLIST(clist31), i, ((i%2)?(&main->gui->list_background_color_1):(&main->gui->list_background_color_2)));
	}
}

void DeviceSideBar::mergeButton(gboolean state)
{
	gtk_widget_set_sensitive(button100, state);
}

