/* machinedetails.cc
 * Copyright (C) 2001-2 The University of Newcastle upon Tyne
 *
 * Written by Paul Coates (Paul.Coates@ncl.ac.uk)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 */

#include <glade/glade-xml.h>
#include <gtk/gtk.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <stdio.h>
#include "machinedetails.h"
#include "header.h"
#include "main.h"
#include "misc.h"
#include "machine.h"
#include "session.h"
#include "gui.h"
#include "licon.xpm"
#include "lserver.xpm"
#include "lcomputer.xpm"
#include "lprinter.xpm"

static void ok_cb(GtkWidget *w, gpointer data)
{ ((MachineDetails*)data)->save(); ((MachineDetails*)data)->hide(); }
static void cancel_cb(GtkWidget *w, gpointer data)
{ ((MachineDetails*)data)->hide(); }
static gint delete_event(GtkWidget *widget, GdkEvent *event, gpointer data)
{ ((MachineDetails*)data)->hide(); return TRUE; }
static void combo_cb(GtkWidget *w, gpointer data)
{
	int i;
	char *nm;
	MachineDetails *m = (MachineDetails*) data;
	nm = gtk_entry_get_text(GTK_ENTRY(GTK_COMBO(m->combo1)->entry));
	if (strcmp(nm, "Printer")==0) i=2; else
	if (strcmp(nm, "Server")==0) i=3; else i=1;
	gtk_notebook_set_page(GTK_NOTEBOOK (m->notebook1), i);
}

MachineDetails::MachineDetails(Main *m)
{
	GtkWidget *pixmap;
	GladeXML *gxml;

	main = m;
	machine = NULL;

	gchar *fname = user_file("nomad.glade");
	gxml = glade_xml_new (fname, "machinedetails");
	if (!gxml)
	{
		g_warning("Failed to load file nomad.glade");
		return;
	}

	window = glade_xml_get_widget (gxml, "machinedetails");
	entry8 = glade_xml_get_widget (gxml, "entry8");
	entry9 = glade_xml_get_widget (gxml, "entry9");
	entry10 = glade_xml_get_widget (gxml, "entry10");
	entry12 = glade_xml_get_widget (gxml, "entry12");
	entry13 = glade_xml_get_widget (gxml, "entry13");
	entry14 = glade_xml_get_widget (gxml, "entry14");
	combo1 = glade_xml_get_widget (gxml, "combo1");
	notebook1 = glade_xml_get_widget (gxml, "notebook1");
	label22 = glade_xml_get_widget (gxml, "label22");
	button3 = glade_xml_get_widget (gxml, "button3");
	button4 = glade_xml_get_widget (gxml, "button4");
	gtk_object_unref (GTK_OBJECT (gxml));
	g_free(fname);

//	gtk_widget_set_usize (window, 420, 280);
	gtk_widget_realize (window);
	window_icon(window, licon_xpm);
	gdk_window_set_icon_name(window->window, "Device Details");
	gtk_signal_connect (GTK_OBJECT (window), "delete_event",
		GTK_SIGNAL_FUNC(delete_event), this);
	gtk_signal_connect(GTK_OBJECT(window), "destroy",
		GTK_SIGNAL_FUNC(delete_event), this);

  pixmap = create_pixmap(window, (char**) lcomputer_xpm);
  gtk_widget_show (pixmap);
  gtk_container_add (GTK_CONTAINER (notebook1), pixmap);

  pixmap = create_pixmap(window, (char**) lprinter_xpm);
  gtk_widget_show (pixmap);
  gtk_container_add (GTK_CONTAINER (notebook1), pixmap);

  pixmap = create_pixmap(window, (char**) lserver_xpm);
  gtk_widget_show (pixmap);
  gtk_container_add (GTK_CONTAINER (notebook1), pixmap);

	gtk_signal_connect(GTK_OBJECT(GTK_COMBO(combo1)->list), "selection_changed",
		GTK_SIGNAL_FUNC (combo_cb), this);
  gtk_signal_connect (GTK_OBJECT (button3), "clicked",
		GTK_SIGNAL_FUNC (ok_cb), this);
  gtk_signal_connect (GTK_OBJECT (button4), "clicked",
		GTK_SIGNAL_FUNC (cancel_cb), this);
}

MachineDetails::~MachineDetails()
{
}

void MachineDetails::empty()
{
	gtk_entry_set_text(GTK_ENTRY(entry8), "");
	gtk_entry_set_text(GTK_ENTRY(entry9), "");
	gtk_entry_set_text(GTK_ENTRY(entry10), "");
	gtk_entry_set_text(GTK_ENTRY(entry12), "");
	gtk_entry_set_text(GTK_ENTRY(entry13), "");
	gtk_entry_set_text(GTK_ENTRY(entry14), "");
	gtk_entry_set_text(GTK_ENTRY(GTK_COMBO(combo1)->entry), "Computer");
	gtk_label_set_text(GTK_LABEL(label22), "");
}

void MachineDetails::show()
{
	char buf[16];
	int i;

	gtk_widget_show(window);
	gdk_window_raise(window->window);

	empty();

	if (machine!=NULL)
	{
		gtk_entry_set_text(GTK_ENTRY(entry8), machine->name);
		gtk_entry_set_text(GTK_ENTRY(entry9),
			(machine->ip.zero())?machine->ipinv.getstring():machine->ip.getstring());
		gtk_entry_set_text(GTK_ENTRY(entry10), machine->mac.getstring());
		gtk_entry_set_text(GTK_ENTRY(entry12), machine->location);
		gtk_entry_set_text(GTK_ENTRY(entry13), machine->contact);
		gtk_entry_set_text(GTK_ENTRY(entry14), machine->descr);
		if (machine->subtype==machine_server)
		{ strcpy(buf, "Server"); i=3; }else
		if (machine->subtype==machine_printer)
		{ strcpy(buf, "Printer"); i=2; }else
		{ strcpy(buf, "Computer"); i=1; }

		gtk_notebook_set_page(GTK_NOTEBOOK(notebook1), i);
		gtk_entry_set_text(GTK_ENTRY(GTK_COMBO(combo1)->entry), buf);
		if (machine->last_seen.zero()==0)
		{
			gtk_label_set_text(GTK_LABEL(label22),
				machine->last_seen.date_to_str(1));
		}
	}
}

void MachineDetails::hide()
{
	gtk_widget_hide(window);
}

void MachineDetails::save()
{
	char *nm;

	if (machine!=NULL)
	{
		strcpy(machine->name, gtk_entry_get_text(GTK_ENTRY(entry8)));
		machine->ip.setstring(gtk_entry_get_text(GTK_ENTRY(entry9)));
		machine->mac.setstring(gtk_entry_get_text(GTK_ENTRY(entry10)));
		strcpy(machine->location, gtk_entry_get_text(GTK_ENTRY(entry12)));
		strcpy(machine->contact, gtk_entry_get_text(GTK_ENTRY(entry13)));
		strcpy(machine->descr, gtk_entry_get_text(GTK_ENTRY(entry14)));
		nm = gtk_entry_get_text(GTK_ENTRY(GTK_COMBO(combo1)->entry));
		if (strcmp(nm, "Server")==0) machine->subtype = machine_server; else
		if (strcmp(nm, "Printer")==0) machine->subtype = machine_printer; else
			machine->subtype = machine_computer;
		main->session->devicebit=TRUE;
		main->setTitle();
	}
	main->gui->update_views();
}

