/* preferences.cc
 * Copyright (C) 2001 The University of Newcastle upon Tyne
 *
 * Written by Paul Coates (Paul.Coates@ncl.ac.uk)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 */

#include <glade/glade-xml.h>
#include <gtk/gtkinvisible.h>
#include <gnome.h>
#include "preferences.h"
#include "session.h"
#include "timeframe.h"
#include "level.h"
#include "action.h"
#include "rule.h"
#include "main.h"
#include "scheduler.h"
#include "topology.h"
#include "monitor.h"
#include "gui.h"
#include "misc.h"
#include "licon.xpm"

gchar *gconf_key[] =
{ CONFIG_SESSION_STARTUP,                CONFIG_SESSION_AUTO_SAVE_EVERY,
	CONFIG_SESSION_SAVE_SESSION_TIME,      CONFIG_SESSION_AUTO_SAVE_ON_QUIT,
	CONFIG_SESSION_PING_TIME,
	CONFIG_SESSION_TOPOLOGY_TIME,          CONFIG_SESSION_TOPOLOGY_DEFAULT,
	CONFIG_SESSION_GRAPH_DEFAULT,          CONFIG_IMPORT_AUTO_IMPORT_FILES,
	CONFIG_IMPORT_IMPORT_TIME,             CONFIG_IMPORT_FILES,
	CONFIG_SUPPORT_PING,                   CONFIG_SUPPORT_TELNET,
	CONFIG_SUPPORT_TRACEROUTE,             CONFIG_SEARCH_AUTO_ZOOM,
	CONFIG_SEARCH_AUTO_HIDE,               CONFIG_SEARCH_SHOW_HIDDEN,
	CONFIG_SEARCH_SELECT_MATCHES,          CONFIG_DIAGRAM_ANIMATE_ZOOM,
	CONFIG_DIAGRAM_SHOW_LABELS,            CONFIG_DIAGRAM_SHOW_LINE_COLORS,
	CONFIG_DIAGRAM_SHOW_LEAF,              CONFIG_DIAGRAM_SHOW_EMPTY_LEAF,
	CONFIG_DIAGRAM_GROUP_LEAF,             CONFIG_DIAGRAM_NETWORK_TYPE,
	CONFIG_DIAGRAM_NETWORK_POSITION,       CONFIG_DIAGRAM_SHOW_MACHINES,
	CONFIG_DIAGRAM_GROUP_MACHINES,         CONFIG_DIAGRAM_FONT,
	CONFIG_DIAGRAM_BACKGROUND,             CONFIG_DIAGRAM_DEVICE_UP,
	CONFIG_DIAGRAM_DEVICE_DOWN,            CONFIG_DEVICE_UP_SELECTED,
	CONFIG_DEVICE_DOWN_SELECTED,           CONFIG_DIAGRAM_MACHINE,
	CONFIG_DIAGRAM_MACHINE_SELECTED,       CONFIG_DIAGRAM_NETWORK,
	CONFIG_DIAGRAM_NETWORK_EMPTY,          CONFIG_DIAGRAM_NETWORK_SELECTED,
	CONFIG_DIAGRAM_LIST_BG_1,              CONFIG_DIAGRAM_LIST_BG_2,
	CONFIG_COMMUNITY,                      CONFIG_SAMPLE_FREQUENCY,
	CONFIG_SAMPLE_DURATION,                CONFIG_SAMPLE_ERRORS,
	CONFIG_SAMPLE_RATE,                    CONFIG_SAMPLE_REPLACE,
	CONFIG_EMAIL_SENDER_ADDRESS,           CONFIG_EMAIL_SMTP_SERVER,
	CONFIG_EMAIL_PORT,                     CONFIG_EMAIL_SUBJECT,
	CONFIG_SHOW_SIDEBAR,                   CONFIG_SHOW_STATUSBAR,
	CONFIG_EMAIL_BODY,                     NULL
};

static void config_notify_cb(GConfClient *client, guint cnxn_id, GConfEntry *entry, gpointer data)
{ ((Preferences*)data)->config_update(entry->key, 1); }
static void config_widget_cb(GtkWidget *w, gpointer data)
{ ((Preferences*)data)->config_widget_action(w); }
static void config_focus_cb(GtkWidget *w, GdkEventFocus *e, gpointer data)
{ ((Preferences*)data)->config_widget_action(w); }
static void config_color_picker_cb(GtkWidget *w, guint r, guint g, guint b, guint a, gpointer data)
{ ((Preferences*)data)->config_widget_action(w); }
static void config_font_picker_cb(GtkWidget *w, gchar *name, gpointer data)
{ ((Preferences*)data)->config_widget_action(w); }
static void config_clist_select_cb(GtkWidget *w, gint row, gint col,
	GdkEventButton *e, gpointer data)
{ ((Preferences*)data)->config_clist_select_action(w, row); }
static void config_clist_unselect_cb(GtkWidget *w, gint row, gint col,
	GdkEventButton *e, gpointer data)
{ ((Preferences*)data)->config_clist_unselect_action(w, row); }

static void sidebar_button_pressed_cb(GtkWidget *w, gpointer data)
{ ((Preferences*)data)->sidebar_button_pressed(w); }
static void sidebar_icon_pressed_cb(GtkWidget *w, gpointer data)
{ ((Preferences*)data)->sidebar_icon_pressed(w); }
static void close_cb(GtkWidget *w, gpointer data)
{ ((Preferences*)data)->hide(); }
static gint delete_event(GtkWidget *widget, GdkEvent *event, gpointer data)
{ ((Preferences*)data)->hide(); return (TRUE); }

static void notify_new_button_cb(GtkWidget *w, gpointer data)
{ ((Preferences*)data)->notify_new_button_action(); }
static void notify_delete_button_cb(GtkWidget *w, gpointer data)
{ ((Preferences*)data)->notify_delete_button_action(); }
static void notify_clist_select_cb(GtkWidget *clist, gint row, gint col,
	GdkEventButton *e, gpointer data)
{ ((Preferences*)data)->notify_clist_select_action(row); }
static void notify_clist_unselect_cb(GtkWidget *clist, gint row, gint col,
	GdkEventButton *e, gpointer data)
{ ((Preferences*)data)->notify_clist_unselect_action(); }
static void notify_event_type_combo_cb(GtkWidget *w, gpointer data)
{ ((Preferences*)data)->notify_event_type_combo_action(); }
static void notify_level_combo_cb(GtkWidget *w, gpointer data)
{ ((Preferences*)data)->notify_level_combo_action(); }
static void notify_time_frame_combo_cb(GtkWidget *w, gpointer data)
{ ((Preferences*)data)->notify_time_frame_combo_action(); }
static void notify_threshold_entry_cb(GtkWidget *w, gpointer data)
{ ((Preferences*)data)->notify_threshold_entry_action(); }
static void notify_percentage_checkbutton_cb(GtkWidget *w, gpointer data)
{ ((Preferences*)data)->notify_percentage_checkbutton_action(); }
static void notify_offset_checkbutton_cb(GtkWidget *w, gpointer data)
{ ((Preferences*)data)->notify_offset_checkbutton_action(); }
static void notify_occurance_spinbutton_cb(GtkWidget *w, gpointer data)
{ ((Preferences*)data)->notify_occurance_spinbutton_action(); }

static void level_new_button_cb(GtkWidget *w, gpointer data)
{ ((Preferences*)data)->level_new_button_action(); }
static void level_delete_button_cb(GtkWidget *w, gpointer data)
{ ((Preferences*)data)->level_delete_button_action(); }
static void level_clist_select_cb(GtkWidget *clist, gint row, gint col,
	GdkEventButton *e, gpointer data)
{ ((Preferences*)data)->level_clist_select_action(row); }
static void level_clist_unselect_cb(GtkWidget *clist, gint row, gint col,
	GdkEventButton *e, gpointer data)
{ ((Preferences*)data)->level_clist_unselect_action(); }
static void level_name_entry_cb(GtkWidget *w, gpointer data)
{ ((Preferences*)data)->level_name_entry_action(); }
static void level_color_picker_cb(GtkWidget *w, guint r, guint g, guint b, guint a, gpointer data)
{ ((Preferences*)data)->level_color_picker_action(); }

static void action_new_button_cb(GtkWidget *w, gpointer data)
{ ((Preferences*)data)->action_new_button_action(); }
static void action_delete_button_cb(GtkWidget *w, gpointer data)
{ ((Preferences*)data)->action_delete_button_action(); }
static void action_test_button_cb(GtkWidget *w, gpointer data)
{ ((Preferences*)data)->action_test_button_action(); }
static void action_type_combo_cb(GtkWidget *w, gpointer data)
{ ((Preferences*)data)->action_type_combo_action(); }
static void action_clist_select_cb(GtkWidget *clist, gint row, gint col,
	GdkEventButton *e, gpointer data)
{ ((Preferences*)data)->action_clist_select_action(row); }
static void action_clist_unselect_cb(GtkWidget *clist, gint row, gint col,
	GdkEventButton *e, gpointer data)
{ ((Preferences*)data)->action_clist_unselect_action(); }
static void action_command_line_entry_cb(GtkWidget *w, gpointer data)
{ ((Preferences*)data)->action_command_line_entry_action(); }
static void action_email_address_entry_cb(GtkWidget *w, gpointer data)
{ ((Preferences*)data)->action_email_address_entry_action(); }
static void action_sound_file_entry_cb(GtkWidget *w, gpointer data)
{ ((Preferences*)data)->action_sound_file_entry_action(); }

static void timeframe_new_button_cb(GtkWidget *w, gpointer data)
{ ((Preferences*)data)->timeframe_new_button_action(); }
static void timeframe_delete_button_cb(GtkWidget *w, gpointer data)
{ ((Preferences*)data)->timeframe_delete_button_action(); }
static void timeframe_clist_select_cb(GtkWidget *clist, gint row, gint col,
	GdkEventButton *e, gpointer data)
{ ((Preferences*)data)->timeframe_clist_select_action(row); }
static void timeframe_clist_unselect_cb(GtkWidget *clist, gint row, gint col,
	GdkEventButton *e, gpointer data)
{ ((Preferences*)data)->timeframe_clist_unselect_action(); }
static void timeframe_name_textentry_cb(GtkWidget *w, gpointer data)
{ ((Preferences*)data)->timeframe_name_textentry_action(); }
static void timeframe_invert_checkbutton_cb(GtkWidget *w, gpointer data)
{ ((Preferences*)data)->timeframe_invert_checkbutton_action(); }
static void timeframe_start_combo_cb(GtkWidget *w, gpointer data)
{ ((Preferences*)data)->timeframe_start_combo_action(); }
static void timeframe_end_combo_cb(GtkWidget *w, gpointer data)
{ ((Preferences*)data)->timeframe_end_combo_action(); }
static void timeframe_day_checkbutton_cb(GtkWidget *w, gpointer data)
{ ((Preferences*)data)->timeframe_day_action(w); }

int compare_sample_rates(Link *a, Link *b)
{
	SampleRate *s1, *s2;
	s1 = (SampleRate*) a->item;
	s2 = (SampleRate*) b->item;
	return (s1->frequency > s2->frequency);
}

Preferences::Preferences(Main *m)
{
	GtkAdjustment *adj;
	GladeXML *gxml;
	int i;

	main = m;

	monitor_button = NULL;
	community_button = NULL;

	gchar *fname = user_file("preferences.glade");
	gxml = glade_xml_new (fname, "preferences");
	if (!gxml)
	{
		g_warning("Failed to load file preferences.glade");
		return;
	}

	window                       = glade_xml_get_widget (gxml, "preferences");
	sidebar                      = glade_xml_get_widget (gxml, "sidebar");
	notebook                     = glade_xml_get_widget (gxml, "notebook");
	close_button                 = glade_xml_get_widget (gxml, "button19");

	session_name_entry           = glade_xml_get_widget (gxml, "entry13");
	session_auto_save            = glade_xml_get_widget (gxml, "checkbutton1");
	session_auto_save_time1      = glade_xml_get_widget (gxml, "entry8");
	session_auto_save_time2      = glade_xml_get_widget (gxml, "entry16");
	session_save_on_exit         = glade_xml_get_widget (gxml, "checkbutton2");
//	session_store_topology       = glade_xml_get_widget (gxml, "checkbutton3");
	session_ping_time1           = glade_xml_get_widget (gxml, "entry6");
	session_ping_time2           = glade_xml_get_widget (gxml, "entry17");
	session_regrab_time1         = glade_xml_get_widget (gxml, "entry7");
	session_regrab_time2         = glade_xml_get_widget (gxml, "entry18");
	session_topology_default     = glade_xml_get_widget (gxml, "checkbutton9");
	session_graph_default        = glade_xml_get_widget (gxml, "checkbutton7");
	import_clist                 = glade_xml_get_widget (gxml, "clist5");
	import_filename              = glade_xml_get_widget (gxml, "comboentry5");
	import_auto                  = glade_xml_get_widget (gxml, "checkbutton10");
	import_time1                 = glade_xml_get_widget (gxml, "entry14");
	import_time2                 = glade_xml_get_widget (gxml, "entry15");
	import_add                   = glade_xml_get_widget (gxml, "button10");
	import_delete                = glade_xml_get_widget (gxml, "button9");
	import_import                = glade_xml_get_widget (gxml, "button13");
	support_ping_entry           = glade_xml_get_widget (gxml, "entry9");
	support_telnet_entry         = glade_xml_get_widget (gxml, "entry10");
	support_traceroute_entry     = glade_xml_get_widget (gxml, "entry11");
	search_zoom_checkbutton      = glade_xml_get_widget (gxml, "checkbutton14");
	search_hide_checkbutton      = glade_xml_get_widget (gxml, "checkbutton15");
	search_hidden_checkbutton    = glade_xml_get_widget (gxml, "checkbutton16");
	search_all_checkbutton       = glade_xml_get_widget (gxml, "checkbutton17");

	default_animate_zoom         = glade_xml_get_widget (gxml, "checkbutton13");
	default_show_leaf            = glade_xml_get_widget (gxml, "checkbutton5");
	default_show_empty           = glade_xml_get_widget (gxml, "checkbutton4");
	default_group_leaf_type      = glade_xml_get_widget (gxml, "combo-entry6");
	default_network_type         = glade_xml_get_widget (gxml, "combo-entry12");
	default_network_position     = glade_xml_get_widget (gxml, "combo-entry9");
	default_show_machines        = glade_xml_get_widget (gxml, "checkbutton6");
	default_group_machines       = glade_xml_get_widget (gxml, "combo-entry7");

	font_default                 = glade_xml_get_widget (gxml, "fontpicker1");
	color_background             = glade_xml_get_widget (gxml, "colorpicker2");
	color_list1                  = glade_xml_get_widget (gxml, "colorpicker15");
	color_list2                  = glade_xml_get_widget (gxml, "colorpicker16");
	color_device_up              = glade_xml_get_widget (gxml, "colorpicker9");
	color_device_down            = glade_xml_get_widget (gxml, "colorpicker6");
	color_selected_device_up     = glade_xml_get_widget (gxml, "colorpicker14");
	color_selected_device_down   = glade_xml_get_widget (gxml, "colorpicker7");
	color_machine                = glade_xml_get_widget (gxml, "colorpicker13");
	color_selected_machine       = glade_xml_get_widget (gxml, "colorpicker8");
	color_network                = glade_xml_get_widget (gxml, "colorpicker12");
	color_empty_network          = glade_xml_get_widget (gxml, "colorpicker10");
	color_selected_network       = glade_xml_get_widget (gxml, "colorpicker11");

	community_clist              = glade_xml_get_widget (gxml, "clist4");
	community_entry              = glade_xml_get_widget (gxml, "entry5");
	community_add_button         = glade_xml_get_widget (gxml, "button8");
	community_delete_button      = glade_xml_get_widget (gxml, "button7");

	sample_clist                  = glade_xml_get_widget (gxml, "clist3");
	sample_label                  = glade_xml_get_widget (gxml, "label189");
	sample_error_label            = glade_xml_get_widget (gxml, "label194");
	sample_frequency1             = glade_xml_get_widget (gxml, "entry1");
	sample_frequency2             = glade_xml_get_widget (gxml, "combo-entry1");
	sample_duration1              = glade_xml_get_widget (gxml, "entry2");
	sample_duration2              = glade_xml_get_widget (gxml, "combo-entry2");
	sample_add                    = glade_xml_get_widget (gxml, "button4");
	sample_delete                 = glade_xml_get_widget (gxml, "button3");
	sample_errors1                = glade_xml_get_widget (gxml, "entry34");
	sample_errors2                = glade_xml_get_widget (gxml, "entry35");
	sample_rate1                  = glade_xml_get_widget (gxml, "entry37");
	sample_rate2                  = glade_xml_get_widget (gxml, "entry38");
	sample_replace                = glade_xml_get_widget (gxml, "checkbutton27");
	sample_apply                  = glade_xml_get_widget (gxml, "button32");
	sample_revert                 = glade_xml_get_widget (gxml, "button31");
	gtk_signal_connect(GTK_OBJECT(sample_clist), "select_row",
		GTK_SIGNAL_FUNC(config_clist_select_cb), this);
	gtk_signal_connect(GTK_OBJECT(sample_clist), "unselect_row",
		GTK_SIGNAL_FUNC(config_clist_unselect_cb), this);
	gtk_signal_connect(GTK_OBJECT(sample_errors1), "changed",
		GTK_SIGNAL_FUNC(config_widget_cb), this);
	gtk_signal_connect(GTK_OBJECT(sample_errors2), "changed",
		GTK_SIGNAL_FUNC(config_widget_cb), this);
	gtk_signal_connect(GTK_OBJECT(sample_rate1), "changed",
		GTK_SIGNAL_FUNC(config_widget_cb), this);
	gtk_signal_connect(GTK_OBJECT(sample_rate2), "changed",
		GTK_SIGNAL_FUNC(config_widget_cb), this);
	gtk_signal_connect(GTK_OBJECT(sample_replace), "toggled",
		GTK_SIGNAL_FUNC(config_widget_cb), this);
  gtk_signal_connect (GTK_OBJECT (sample_add), "clicked",
		GTK_SIGNAL_FUNC (config_widget_cb), this);
  gtk_signal_connect (GTK_OBJECT (sample_delete), "clicked",
		GTK_SIGNAL_FUNC (config_widget_cb), this);
  gtk_signal_connect (GTK_OBJECT (sample_apply), "clicked",
		GTK_SIGNAL_FUNC (config_widget_cb), this);
  gtk_signal_connect (GTK_OBJECT (sample_revert), "clicked",
		GTK_SIGNAL_FUNC (config_widget_cb), this);

	//////////////////////////////////////////////////////////
	// Events | Notifier
	//////////////////////////////////////////////////////////
	notify_clist                    = glade_xml_get_widget (gxml, "clist7");
	notify_new_button               = glade_xml_get_widget (gxml, "button23");
	notify_delete_button            = glade_xml_get_widget (gxml, "button22");
	notify_event_type_combo         = glade_xml_get_widget (gxml, "combo19");
	notify_threshold_entry          = glade_xml_get_widget (gxml, "entry27");
	notify_percentage_checkbutton   = glade_xml_get_widget (gxml, "checkbutton28");
	notify_offset_checkbutton       = glade_xml_get_widget (gxml, "checkbutton19");
	notify_occurance_spinbutton     = glade_xml_get_widget (gxml, "spinbutton6");
	notify_time_frame_combo         = glade_xml_get_widget (gxml, "combo20");
	notify_level_combo              = glade_xml_get_widget (gxml, "combo26");
	notify_table                    = glade_xml_get_widget (gxml, "table24");
	notify_hbox69                   = glade_xml_get_widget (gxml, "hbox69");
	notify_hbox75                   = glade_xml_get_widget (gxml, "hbox75");
  gtk_signal_connect (GTK_OBJECT (notify_new_button), "clicked",
		GTK_SIGNAL_FUNC (notify_new_button_cb), this);
  gtk_signal_connect (GTK_OBJECT (notify_delete_button), "clicked",
		GTK_SIGNAL_FUNC (notify_delete_button_cb), this);
 	gtk_signal_connect(GTK_OBJECT(notify_clist), "select_row",
		GTK_SIGNAL_FUNC(notify_clist_select_cb), this);
	gtk_signal_connect(GTK_OBJECT(notify_clist), "unselect_row",
		GTK_SIGNAL_FUNC(notify_clist_unselect_cb), this);
	gtk_signal_connect(GTK_OBJECT(GTK_COMBO(notify_event_type_combo)->list), "selection_changed",
		GTK_SIGNAL_FUNC(notify_event_type_combo_cb), this);
	gtk_signal_connect(GTK_OBJECT(GTK_COMBO(notify_level_combo)->list), "selection_changed",
		GTK_SIGNAL_FUNC(notify_level_combo_cb), this);
	gtk_signal_connect(GTK_OBJECT(GTK_COMBO(notify_time_frame_combo)->list), "selection_changed",
		GTK_SIGNAL_FUNC(notify_time_frame_combo_cb), this);
	gtk_signal_connect(GTK_OBJECT(notify_threshold_entry), "changed",
		GTK_SIGNAL_FUNC(notify_threshold_entry_cb), this);
	gtk_signal_connect(GTK_OBJECT(notify_percentage_checkbutton), "toggled",
		GTK_SIGNAL_FUNC(notify_percentage_checkbutton_cb), this);
	gtk_signal_connect(GTK_OBJECT(notify_offset_checkbutton), "toggled",
		GTK_SIGNAL_FUNC(notify_offset_checkbutton_cb), this);
	adj = gtk_spin_button_get_adjustment(GTK_SPIN_BUTTON(notify_occurance_spinbutton));
	gtk_signal_connect(GTK_OBJECT(adj), "value_changed",
		GTK_SIGNAL_FUNC (notify_occurance_spinbutton_cb), this);

	//////////////////////////////////////////////////////////
	// Events | Levels
	//////////////////////////////////////////////////////////
	level_clist                  = glade_xml_get_widget (gxml, "clist8");
	level_name_label             = glade_xml_get_widget (gxml, "label158");
	level_name_entry             = glade_xml_get_widget (gxml, "entry28");
	level_color_label            = glade_xml_get_widget (gxml, "label159");
	level_color_picker           = glade_xml_get_widget (gxml, "colorpicker17");
	level_new_button             = glade_xml_get_widget (gxml, "button24");
	level_delete_button          = glade_xml_get_widget (gxml, "button25");
  gtk_signal_connect (GTK_OBJECT (level_new_button), "clicked",
		GTK_SIGNAL_FUNC (level_new_button_cb), this);
  gtk_signal_connect (GTK_OBJECT (level_delete_button), "clicked",
		GTK_SIGNAL_FUNC (level_delete_button_cb), this);
	gtk_signal_connect(GTK_OBJECT(level_clist), "select_row",
		GTK_SIGNAL_FUNC(level_clist_select_cb), this);
	gtk_signal_connect(GTK_OBJECT(level_clist), "unselect_row",
		GTK_SIGNAL_FUNC(level_clist_unselect_cb), this);
	gtk_signal_connect(GTK_OBJECT(level_name_entry), "changed",
		GTK_SIGNAL_FUNC(level_name_entry_cb), this);
	gtk_signal_connect(GTK_OBJECT(level_color_picker), "color_set",
		GTK_SIGNAL_FUNC(level_color_picker_cb), this);

	action_frame                 = glade_xml_get_widget (gxml, "frame20");
	action_clist                 = glade_xml_get_widget (gxml, "clist9");
	action_type_combo            = glade_xml_get_widget (gxml, "combo21");
	action_new_button            = glade_xml_get_widget (gxml, "button26");
	action_delete_button         = glade_xml_get_widget (gxml, "button27");
	action_test_button           = glade_xml_get_widget (gxml, "button30");
	action_notebook              = glade_xml_get_widget (gxml, "notebook3");
	action_command_line_entry    = glade_xml_get_widget (gxml, "entry29");
	action_email_address_entry   = glade_xml_get_widget (gxml, "entry30");
	action_sound_file_entry      = glade_xml_get_widget (gxml, "combo-entry17");
  gtk_signal_connect (GTK_OBJECT (action_new_button), "clicked",
		GTK_SIGNAL_FUNC (action_new_button_cb), this);
  gtk_signal_connect (GTK_OBJECT (action_delete_button), "clicked",
		GTK_SIGNAL_FUNC (action_delete_button_cb), this);
  gtk_signal_connect (GTK_OBJECT (action_test_button), "clicked",
		GTK_SIGNAL_FUNC (action_test_button_cb), this);
	gtk_signal_connect(GTK_OBJECT(GTK_COMBO(action_type_combo)->list), "selection_changed",
		GTK_SIGNAL_FUNC(action_type_combo_cb), this);
	gtk_signal_connect(GTK_OBJECT(action_clist), "select_row",
		GTK_SIGNAL_FUNC(action_clist_select_cb), this);
	gtk_signal_connect(GTK_OBJECT(action_clist), "unselect_row",
		GTK_SIGNAL_FUNC(action_clist_unselect_cb), this);
	gtk_signal_connect(GTK_OBJECT(action_command_line_entry), "changed",
		GTK_SIGNAL_FUNC(action_command_line_entry_cb), this);
	gtk_signal_connect(GTK_OBJECT(action_email_address_entry), "changed",
		GTK_SIGNAL_FUNC(action_email_address_entry_cb), this);
	gtk_signal_connect(GTK_OBJECT(action_sound_file_entry), "changed",
		GTK_SIGNAL_FUNC(action_sound_file_entry_cb), this);

	timeframe_new_button         = glade_xml_get_widget (gxml, "button28");
	timeframe_delete_button      = glade_xml_get_widget (gxml, "button29");
	timeframe_name_label         = glade_xml_get_widget (gxml, "label173");
	timeframe_mon_checkbutton    = glade_xml_get_widget (gxml, "checkbutton20");
	timeframe_tue_checkbutton    = glade_xml_get_widget (gxml, "checkbutton21");
	timeframe_wed_checkbutton    = glade_xml_get_widget (gxml, "checkbutton22");
	timeframe_thu_checkbutton    = glade_xml_get_widget (gxml, "checkbutton23");
	timeframe_fri_checkbutton    = glade_xml_get_widget (gxml, "checkbutton24");
	timeframe_sat_checkbutton    = glade_xml_get_widget (gxml, "checkbutton25");
	timeframe_sun_checkbutton    = glade_xml_get_widget (gxml, "checkbutton26");
	timeframe_invert_checkbutton = glade_xml_get_widget (gxml, "checkbutton18");
	timeframe_name_textentry     = glade_xml_get_widget (gxml, "entry31");
	timeframe_start_combo        = glade_xml_get_widget (gxml, "combo22");
	timeframe_end_combo          = glade_xml_get_widget (gxml, "combo23");
	timeframe_clist              = glade_xml_get_widget (gxml, "clist10");
	timeframe_table              = glade_xml_get_widget (gxml, "table25");

	email_sender_address         = glade_xml_get_widget (gxml, "entry25");
	email_smtp_server            = glade_xml_get_widget (gxml, "entry26");
	email_port_number            = glade_xml_get_widget (gxml, "spinbutton5");
	email_message_subject        = glade_xml_get_widget (gxml, "entry36");
	email_message_body           = glade_xml_get_widget (gxml, "text1");

	gtk_object_unref (GTK_OBJECT (gxml));
	g_free(fname);

  gtk_widget_realize (window);
	window_icon(window, licon_xpm);
	gdk_window_set_icon_name(window->window, "Preferences");
	gtk_signal_connect (GTK_OBJECT (window), "delete_event",
		GTK_SIGNAL_FUNC(delete_event), this);
	gtk_signal_connect (GTK_OBJECT (window), "destroy_event",
		GTK_SIGNAL_FUNC(delete_event), this);
	gtk_notebook_set_show_tabs(GTK_NOTEBOOK(notebook), FALSE);
	build_sidebar();

	gtk_signal_connect(GTK_OBJECT(close_button), "clicked",
		GTK_SIGNAL_FUNC(close_cb), this);

	gtk_signal_connect(GTK_OBJECT(session_name_entry), "focus_out_event",
		GTK_SIGNAL_FUNC(config_focus_cb), this);
	gtk_signal_connect(GTK_OBJECT(session_name_entry), "activate",
		GTK_SIGNAL_FUNC(config_widget_cb), this);
	gtk_signal_connect(GTK_OBJECT(session_auto_save), "toggled",
		GTK_SIGNAL_FUNC(config_widget_cb), this);
	gtk_signal_connect(GTK_OBJECT(session_auto_save_time1), "focus_out_event",
		GTK_SIGNAL_FUNC(config_focus_cb), this);
	gtk_signal_connect(GTK_OBJECT(session_auto_save_time1), "activate",
		GTK_SIGNAL_FUNC(config_widget_cb), this);
	gtk_signal_connect(GTK_OBJECT(session_auto_save_time2), "changed",
		GTK_SIGNAL_FUNC(config_widget_cb), this);
	gtk_signal_connect(GTK_OBJECT(session_save_on_exit), "toggled",
		GTK_SIGNAL_FUNC(config_widget_cb), this);
	gtk_signal_connect(GTK_OBJECT(session_ping_time1), "focus_out_event",
		GTK_SIGNAL_FUNC(config_focus_cb), this);
	gtk_signal_connect(GTK_OBJECT(session_ping_time1), "activate",
		GTK_SIGNAL_FUNC(config_widget_cb), this);
	gtk_signal_connect(GTK_OBJECT(session_ping_time2), "changed",
		GTK_SIGNAL_FUNC(config_widget_cb), this);
	gtk_signal_connect(GTK_OBJECT(session_regrab_time1), "focus_out_event",
		GTK_SIGNAL_FUNC(config_focus_cb), this);
	gtk_signal_connect(GTK_OBJECT(session_regrab_time1), "activate",
		GTK_SIGNAL_FUNC(config_widget_cb), this);
	gtk_signal_connect(GTK_OBJECT(session_regrab_time2), "changed",
		GTK_SIGNAL_FUNC(config_widget_cb), this);
	gtk_signal_connect(GTK_OBJECT(session_topology_default), "toggled",
		GTK_SIGNAL_FUNC(config_widget_cb), this);
	gtk_signal_connect(GTK_OBJECT(session_graph_default), "toggled",
		GTK_SIGNAL_FUNC(config_widget_cb), this);

	gtk_signal_connect(GTK_OBJECT(import_clist), "select_row",
		GTK_SIGNAL_FUNC(config_clist_select_cb), this);
	gtk_signal_connect(GTK_OBJECT(import_clist), "unselect_row",
		GTK_SIGNAL_FUNC(config_clist_unselect_cb), this);
  gtk_signal_connect(GTK_OBJECT(import_filename), "activate",
		GTK_SIGNAL_FUNC(config_widget_cb), this);
  gtk_signal_connect(GTK_OBJECT(import_add), "clicked",
		GTK_SIGNAL_FUNC(config_widget_cb), this);
  gtk_signal_connect(GTK_OBJECT(import_delete), "clicked",
		GTK_SIGNAL_FUNC(config_widget_cb), this);
  gtk_signal_connect(GTK_OBJECT(import_import), "clicked",
		GTK_SIGNAL_FUNC(config_widget_cb), this);
	gtk_signal_connect(GTK_OBJECT(import_auto), "toggled",
		GTK_SIGNAL_FUNC(config_widget_cb), this);
	gtk_signal_connect(GTK_OBJECT(import_time1), "focus_out_event",
		GTK_SIGNAL_FUNC(config_focus_cb), this);
	gtk_signal_connect(GTK_OBJECT(import_time1), "activate",
		GTK_SIGNAL_FUNC(config_widget_cb), this);
	gtk_signal_connect(GTK_OBJECT(import_time2), "changed",
		GTK_SIGNAL_FUNC(config_widget_cb), this);

	gtk_signal_connect(GTK_OBJECT(support_ping_entry), "focus_out_event",
		GTK_SIGNAL_FUNC(config_focus_cb), this);
	gtk_signal_connect(GTK_OBJECT(support_ping_entry), "activate",
		GTK_SIGNAL_FUNC(config_widget_cb), this);
	gtk_signal_connect(GTK_OBJECT(support_telnet_entry), "focus_out_event",
		GTK_SIGNAL_FUNC(config_focus_cb), this);
	gtk_signal_connect(GTK_OBJECT(support_telnet_entry), "activate",
		GTK_SIGNAL_FUNC(config_widget_cb), this);
	gtk_signal_connect(GTK_OBJECT(support_traceroute_entry), "focus_out_event",
		GTK_SIGNAL_FUNC(config_focus_cb), this);
	gtk_signal_connect(GTK_OBJECT(support_traceroute_entry), "activate",
		GTK_SIGNAL_FUNC(config_widget_cb), this);

	gtk_signal_connect(GTK_OBJECT(search_zoom_checkbutton), "toggled",
		GTK_SIGNAL_FUNC(config_widget_cb), this);
	gtk_signal_connect(GTK_OBJECT(search_hide_checkbutton), "toggled",
		GTK_SIGNAL_FUNC(config_widget_cb), this);
	gtk_signal_connect(GTK_OBJECT(search_hidden_checkbutton), "toggled",
		GTK_SIGNAL_FUNC(config_widget_cb), this);
	gtk_signal_connect(GTK_OBJECT(search_all_checkbutton), "toggled",
		GTK_SIGNAL_FUNC(config_widget_cb), this);

	gtk_signal_connect(GTK_OBJECT(default_animate_zoom), "toggled",
		GTK_SIGNAL_FUNC(config_widget_cb), this);
	gtk_signal_connect(GTK_OBJECT(default_show_leaf), "toggled",
		GTK_SIGNAL_FUNC(config_widget_cb), this);
	gtk_signal_connect(GTK_OBJECT(default_show_empty), "toggled",
		GTK_SIGNAL_FUNC(config_widget_cb), this);
	gtk_signal_connect(GTK_OBJECT(default_group_leaf_type), "changed",
		GTK_SIGNAL_FUNC(config_widget_cb), this);
	gtk_signal_connect(GTK_OBJECT(default_network_type), "changed",
		GTK_SIGNAL_FUNC(config_widget_cb), this);
	gtk_signal_connect(GTK_OBJECT(default_network_position), "changed",
		GTK_SIGNAL_FUNC(config_widget_cb), this);
	gtk_signal_connect(GTK_OBJECT(default_show_machines), "toggled",
		GTK_SIGNAL_FUNC(config_widget_cb), this);
	gtk_signal_connect(GTK_OBJECT(default_group_machines), "changed",
		GTK_SIGNAL_FUNC(config_widget_cb), this);

	gtk_signal_connect(GTK_OBJECT(font_default), "font_set",
		GTK_SIGNAL_FUNC(config_font_picker_cb), this);
	gtk_signal_connect(GTK_OBJECT(color_background), "color_set",
		GTK_SIGNAL_FUNC(config_color_picker_cb), this);
	gtk_signal_connect(GTK_OBJECT(color_list1), "color_set",
		GTK_SIGNAL_FUNC(config_color_picker_cb), this);
	gtk_signal_connect(GTK_OBJECT(color_list2), "color_set",
		GTK_SIGNAL_FUNC(config_color_picker_cb), this);
	gtk_signal_connect(GTK_OBJECT(color_device_up), "color_set",
		GTK_SIGNAL_FUNC(config_color_picker_cb), this);
	gtk_signal_connect(GTK_OBJECT(color_device_down), "color_set",
		GTK_SIGNAL_FUNC(config_color_picker_cb), this);
	gtk_signal_connect(GTK_OBJECT(color_selected_device_up), "color_set",
		GTK_SIGNAL_FUNC(config_color_picker_cb), this);
	gtk_signal_connect(GTK_OBJECT(color_selected_device_down), "color_set",
		GTK_SIGNAL_FUNC(config_color_picker_cb), this);
	gtk_signal_connect(GTK_OBJECT(color_machine), "color_set",
		GTK_SIGNAL_FUNC(config_color_picker_cb), this);
	gtk_signal_connect(GTK_OBJECT(color_selected_machine), "color_set",
		GTK_SIGNAL_FUNC(config_color_picker_cb), this);
	gtk_signal_connect(GTK_OBJECT(color_network), "color_set",
		GTK_SIGNAL_FUNC(config_color_picker_cb), this);
	gtk_signal_connect(GTK_OBJECT(color_empty_network), "color_set",
		GTK_SIGNAL_FUNC(config_color_picker_cb), this);
	gtk_signal_connect(GTK_OBJECT(color_selected_network), "color_set",
		GTK_SIGNAL_FUNC(config_color_picker_cb), this);

	gtk_signal_connect(GTK_OBJECT(community_clist), "select_row",
		GTK_SIGNAL_FUNC(config_clist_select_cb), this);
	gtk_signal_connect(GTK_OBJECT(community_clist), "unselect_row",
		GTK_SIGNAL_FUNC(config_clist_unselect_cb), this);
  gtk_signal_connect(GTK_OBJECT(community_entry), "activate",
		GTK_SIGNAL_FUNC(config_widget_cb), this);
  gtk_signal_connect(GTK_OBJECT(community_add_button), "clicked",
		GTK_SIGNAL_FUNC(config_widget_cb), this);
  gtk_signal_connect(GTK_OBJECT(community_delete_button), "clicked",
		GTK_SIGNAL_FUNC(config_widget_cb), this);

  gtk_signal_connect (GTK_OBJECT (timeframe_new_button), "clicked",
		GTK_SIGNAL_FUNC (timeframe_new_button_cb), this);
  gtk_signal_connect (GTK_OBJECT (timeframe_delete_button), "clicked",
		GTK_SIGNAL_FUNC (timeframe_delete_button_cb), this);
	gtk_signal_connect(GTK_OBJECT(timeframe_clist), "select_row",
		GTK_SIGNAL_FUNC(timeframe_clist_select_cb), this);
	gtk_signal_connect(GTK_OBJECT(timeframe_clist), "unselect_row",
		GTK_SIGNAL_FUNC(timeframe_clist_unselect_cb), this);
	gtk_signal_connect(GTK_OBJECT(timeframe_name_textentry), "changed",
		GTK_SIGNAL_FUNC(timeframe_name_textentry_cb), this);
	gtk_signal_connect(GTK_OBJECT(timeframe_invert_checkbutton), "toggled",
		GTK_SIGNAL_FUNC(timeframe_invert_checkbutton_cb), this);
	gtk_signal_connect(GTK_OBJECT(GTK_COMBO(timeframe_start_combo)->list), "selection_changed",
		GTK_SIGNAL_FUNC(timeframe_start_combo_cb), this);
	gtk_signal_connect(GTK_OBJECT(GTK_COMBO(timeframe_end_combo)->list), "selection_changed",
		GTK_SIGNAL_FUNC(timeframe_end_combo_cb), this);
	gtk_signal_connect(GTK_OBJECT(timeframe_mon_checkbutton), "toggled",
		GTK_SIGNAL_FUNC(timeframe_day_checkbutton_cb), this);
	gtk_signal_connect(GTK_OBJECT(timeframe_tue_checkbutton), "toggled",
		GTK_SIGNAL_FUNC(timeframe_day_checkbutton_cb), this);
	gtk_signal_connect(GTK_OBJECT(timeframe_wed_checkbutton), "toggled",
		GTK_SIGNAL_FUNC(timeframe_day_checkbutton_cb), this);
	gtk_signal_connect(GTK_OBJECT(timeframe_thu_checkbutton), "toggled",
		GTK_SIGNAL_FUNC(timeframe_day_checkbutton_cb), this);
	gtk_signal_connect(GTK_OBJECT(timeframe_fri_checkbutton), "toggled",
		GTK_SIGNAL_FUNC(timeframe_day_checkbutton_cb), this);
	gtk_signal_connect(GTK_OBJECT(timeframe_sat_checkbutton), "toggled",
		GTK_SIGNAL_FUNC(timeframe_day_checkbutton_cb), this);
	gtk_signal_connect(GTK_OBJECT(timeframe_sun_checkbutton), "toggled",
		GTK_SIGNAL_FUNC(timeframe_day_checkbutton_cb), this);

	gtk_signal_connect(GTK_OBJECT(email_sender_address), "focus_out_event",
		GTK_SIGNAL_FUNC(config_focus_cb), this);
	gtk_signal_connect(GTK_OBJECT(email_sender_address), "activate",
		GTK_SIGNAL_FUNC(config_widget_cb), this);
	gtk_signal_connect(GTK_OBJECT(email_smtp_server), "focus_out_event",
		GTK_SIGNAL_FUNC(config_focus_cb), this);
	gtk_signal_connect(GTK_OBJECT(email_smtp_server), "activate",
		GTK_SIGNAL_FUNC(config_widget_cb), this);
	gtk_signal_connect(GTK_OBJECT(email_port_number), "focus_out_event",
		GTK_SIGNAL_FUNC(config_focus_cb), this);
	gtk_signal_connect(GTK_OBJECT(email_port_number), "activate",
		GTK_SIGNAL_FUNC(config_widget_cb), this);
	gtk_signal_connect(GTK_OBJECT(email_message_subject), "focus_out_event",
		GTK_SIGNAL_FUNC(config_focus_cb), this);
	gtk_signal_connect(GTK_OBJECT(email_message_subject), "activate",
		GTK_SIGNAL_FUNC(config_widget_cb), this);
	gtk_signal_connect(GTK_OBJECT(email_message_body), "focus_out_event",
		GTK_SIGNAL_FUNC(config_focus_cb), this);
	gtk_signal_connect(GTK_OBJECT(email_message_body), "activate",
		GTK_SIGNAL_FUNC(config_widget_cb), this);

	// create notifiers
	for (i=0; gconf_key[i] != NULL; i++)
	{
		gconf_client_notify_add(main->client, gconf_key[i],
			config_notify_cb, this, NULL, NULL);
	}
}

Preferences::~Preferences()
{
	empty();
}

void Preferences::show()
{
	gtk_widget_show(window);
	gdk_window_raise(window->window);
}

void Preferences::show_community()
{
	if (current_panel != NULL) gtk_widget_hide (current_panel);
	current_panel = NULL;
	if (monitor_button != NULL)
		sidebar_button_pressed(monitor_button);
	if (community_button!=NULL)
		gtk_signal_emit_by_name (GTK_OBJECT (community_button), "clicked");
	gtk_widget_show(window);
	gdk_window_raise(window->window);
}

void Preferences::hide()
{
	//FIXME if save_xml stuff flag set
	save_xml();
	gtk_widget_hide(window);
}

void Preferences::empty()
{
	TimeFrame *t;
	Level *l;
	Rule *r;
	Link *ln;
	SampleRate *s;
	for (ln=timeframes.get_head(); ln!=NULL; ln=ln->next)
	{ t = (TimeFrame*) ln->item; if (t!=NULL) delete t; } timeframes.empty();
	for (ln=levels.get_head(); ln!=NULL; ln=ln->next)
	{ l = (Level*) ln->item; if (l!=NULL) delete l; } levels.empty();
	for (ln=rules.get_head(); ln!=NULL; ln=ln->next)
	{ r = (Rule*) ln->item; if (r!=NULL) delete r; } rules.empty();
	for (ln=samples.get_head(); ln!=NULL; ln=ln->next)
	{ s = (SampleRate*) ln->item; if (s!=NULL) delete s; } samples.empty();
}

//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
////////////                                                      ////////////
////////////                       Sidebar                        ////////////
////////////                                                      ////////////
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////

void Preferences::build_sidebar()
{
	GtkWidget *panel;

	panel = sidebar_button("Application");
	sidebar_icon(panel, "Session", "gnome-session.png", 3);
	sidebar_icon(panel, "Import", "grip-48.png", 5);
	sidebar_icon(panel, "Support", "default-applications.png", 4);
#ifdef BLEEDING_EDGE
	sidebar_icon(panel, "Search", "gnome-searchtool.png", 8);
#endif
	sidebar_pad(panel);

	panel = sidebar_button("Diagram");
	sidebar_icon(panel, "Defaults", "faq-48.png", 6);
	sidebar_icon(panel, "Fonts & Colors", "gnome-graphics.png", 7);
	sidebar_pad(panel);

	panel = sidebar_button("Monitor");
	sidebar_icon(panel, "Community", "helix-setup-users3.png", 2);
	sidebar_icon(panel, "Graph Server", "gnome-clock.png", 1);
	sidebar_pad(panel);

#ifdef BLEEDING_EDGE
	panel = sidebar_button("Events");
	sidebar_icon(panel, "Notifier", "gnome-ticker.png", 9);
	sidebar_icon(panel, "Levels", "volume-48.png", 10);
	sidebar_icon(panel, "Time Frame", "myevo-appointments.png", 11);
	sidebar_icon(panel, "Email", "emblem-mail.png", 12);
	sidebar_pad(panel);
#endif

	current_panel = NULL;
}

GtkWidget *Preferences::sidebar_button(char *label)
{
	GtkWidget *button, *button2, *panel;

	button = gtk_button_new_with_label (label);
	if (strcmp(label, "Monitor")==0) monitor_button = button;
	gtk_widget_show (button);
	panel = gtk_vbox_new (FALSE, 0);
	gtk_box_pack_start(GTK_BOX(sidebar), button, FALSE, TRUE, 0);
	gtk_box_pack_start(GTK_BOX(sidebar), panel, TRUE, TRUE, 2);
	gtk_object_set_data (GTK_OBJECT (button), "panel", panel);
	gtk_signal_connect (GTK_OBJECT (button), "clicked",
		GTK_SIGNAL_FUNC(sidebar_button_pressed_cb), this);

	// icon buttons are in a radio group, we need to create a hidden
	// button to make it look like there are no buttons selected at start
	button_group = NULL;
	button2 = gtk_radio_button_new(button_group);
	button_group = gtk_radio_button_group(GTK_RADIO_BUTTON (button2));
	gtk_box_pack_start (GTK_BOX (panel), button2, FALSE, TRUE, 0);
	gtk_button_clicked (GTK_BUTTON (button2));

	return panel;
}

void Preferences::sidebar_icon(GtkWidget *w, char *label, char *fname, int pg)
{
	GtkWidget *button, *vbox, *pixmap, *widget;
	gchar *str;

	button = gtk_radio_button_new (button_group);
	if (strcmp(label, "Community")==0) community_button = button;
	gtk_button_set_relief (GTK_BUTTON (button), GTK_RELIEF_NONE);
	gtk_toggle_button_set_mode (GTK_TOGGLE_BUTTON (button), FALSE);
	button_group = gtk_radio_button_group(GTK_RADIO_BUTTON (button));
	gtk_object_set_data (GTK_OBJECT (button), "page", GINT_TO_POINTER(pg));
	gtk_signal_connect (GTK_OBJECT (button), "clicked",
		GTK_SIGNAL_FUNC(sidebar_icon_pressed_cb), this);
	gtk_box_pack_start (GTK_BOX (w), button, FALSE, TRUE, 0);
	gtk_widget_show (button);

	vbox = gtk_vbox_new (FALSE, 0);
	gtk_container_add (GTK_CONTAINER (button), vbox);
	gtk_widget_show (vbox);

	str = user_file(fname);
	pixmap = gnome_pixmap_new_from_file(str);
	gtk_box_pack_start (GTK_BOX (vbox), pixmap, FALSE, FALSE, 0);
	gtk_widget_show (pixmap);
	g_free(str);

	widget = gtk_label_new (label);
	gtk_misc_set_alignment (GTK_MISC (widget), 0.5, 0.5);
	gtk_label_set_line_wrap (GTK_LABEL (widget), FALSE);
	gtk_box_pack_start (GTK_BOX (vbox), widget, FALSE, TRUE, 0);
	gtk_widget_show (widget);
}

void Preferences::sidebar_pad(GtkWidget *w)
{
	GtkWidget *widget;

	widget = gtk_invisible_new();
	gtk_box_pack_end(GTK_BOX (w), widget, TRUE, TRUE, 0);
	gtk_widget_show(widget);
	gtk_object_set_data (GTK_OBJECT (w), "group", button_group);
}

void Preferences::sidebar_button_pressed(GtkWidget *w)
{
	GSList *grp;
	GtkWidget *panel;
	int page = 0;

	if (w==NULL) return;

	panel = GTK_WIDGET(gtk_object_get_data(GTK_OBJECT (w), "panel"));
	if (current_panel != NULL)
	{
		gtk_widget_hide (current_panel);
	}
	if (current_panel != panel)
	{
		current_panel = panel;
		gtk_widget_show (panel);
		// set notebook page based on radio button currently set
		grp = (GSList*)(gtk_object_get_data(GTK_OBJECT (panel), "group"));
		while (grp!=NULL)
		{
			if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON (grp->data)))
			{
				page = GPOINTER_TO_INT(gtk_object_get_data(GTK_OBJECT(grp->data), "page"));
			}
			grp = g_slist_next(grp);
		}
	}else
	{
		current_panel = NULL;
	}
	gtk_notebook_set_page(GTK_NOTEBOOK(notebook), page);
}

void Preferences::sidebar_icon_pressed(GtkWidget *w)
{
	int page = GPOINTER_TO_INT(gtk_object_get_data(GTK_OBJECT(w), "page"));
	gtk_notebook_set_page(GTK_NOTEBOOK(notebook), page);
	// FIXME the next line we should check the pages have
	// changed before saving
//	save_rules = TRUE;
//	if (((page==9) || (page==10) || (page==11)) && (save_rules==TRUE))
		save_xml();
}

//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
////////////                                                      ////////////
////////////                    Gconf Calls                       ////////////
////////////                                                      ////////////
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////

void Preferences::set_gconf_from_spinbutton(GtkWidget *w, gchar *config)
{
	GError *error;
	gint val;

	val = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(w));
	gconf_client_set_int (main->client, config, val, &error);
}

void Preferences::set_spinbutton_from_gconf(GtkWidget *w, gchar *config)
{
	GError *error;
	gint val;

	val = gconf_client_get_int (main->client, config, &error);
	gtk_spin_button_set_value(GTK_SPIN_BUTTON(w), val);
}

void Preferences::set_gconf_from_editable(GtkWidget *w, gchar *config)
{
	GError *error;
	gchar *str;
	str = gtk_editable_get_chars(GTK_EDITABLE(w), 0, -1);
	gconf_client_set_string (main->client, config, str, &error);
	g_free(str);
}

void Preferences::set_editable_from_gconf(GtkWidget *w, gchar *config)
{
	GError *error;
	gchar *str, *str2;
	gint pos;

	str = gconf_client_get_string (main->client, config, &error);
	str2 = gtk_editable_get_chars(GTK_EDITABLE(w), 0, -1);
	if ((str!=NULL) && (str2!=NULL) && (strcmp(str, str2)!=0))
	{
		if (GTK_IS_ENTRY(w)) gtk_entry_set_text(GTK_ENTRY(w), str); else
		if (GTK_IS_TEXT(w))
		{
			gtk_editable_delete_text(GTK_EDITABLE(w), 0, -1); pos=0;
			gtk_editable_insert_text(GTK_EDITABLE(w), str, strlen(str), &pos);
		}
	}
	g_free(str);
}

void Preferences::set_gconf_from_togglebutton(GtkWidget *w, gchar *config)
{
	GError *error;
	gboolean value;

	value = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(w));
	gconf_client_set_bool(main->client, config, value, &error);
}

void Preferences::set_togglebutton_from_gconf(GtkWidget *w, gchar *config)
{
	GError *error;
	gboolean value;

	value = gconf_client_get_bool(main->client, config, &error);
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(w), value);
}

void Preferences::set_gconf_from_color(GtkWidget *w, gchar *config)
{
	GError *error;
	guint8 r, g, b, a;
	gchar str[9];

	gnome_color_picker_get_i8 (GNOME_COLOR_PICKER(w), &r, &g, &b, &a);
	snprintf (str, 9, "#%02X%02X%02X", r, g, b);
	gconf_client_set_string (main->client, config, str, &error);
}

void Preferences::set_color_from_gconf(GtkWidget *w, gchar *config)
{
	GError *error;
	gchar *str;
	guint r, g, b;

	str = gconf_client_get_string (main->client, config, &error);
	if (str!=NULL)
	{
		sscanf(str, "#%2X%2X%2X", &r, &g, &b);
		gnome_color_picker_set_i8(GNOME_COLOR_PICKER(w), r, g, b, 0);
		g_free(str);
	}
}

void Preferences::set_gconf_from_font(GtkWidget *w, gchar *config)
{
	GError *error;
	gchar *str;

	str = gnome_font_picker_get_font_name (GNOME_FONT_PICKER(w));
	gconf_client_set_string (main->client, config, str, &error);
}

void Preferences::set_font_from_gconf(GtkWidget *w, gchar *config)
{
	GError *error;
	gchar *str;

	str = gconf_client_get_string (main->client, config, &error);
	if (str!=NULL)
	{
		gnome_font_picker_set_font_name(GNOME_FONT_PICKER(w), str);
		g_free(str);
	}
}

void Preferences::set_time_from_gconf(GtkWidget *w, GtkWidget *w2, gchar *key)
{
	GError *error;
	gchar *str, *str2;
	gchar **sarray;

	str = gconf_client_get_string (main->client, key, &error);

	if (str!=NULL)
	{
		sarray = g_strsplit(str, " ", 2);
		str2 = gtk_editable_get_chars(GTK_EDITABLE(w), 0, -1);
		if ((sarray[0]!=NULL) && (strcmp(str2, sarray[0])!=0))
			gtk_entry_set_text(GTK_ENTRY(w), sarray[0]);
		g_free(str2);
		str2 = gtk_editable_get_chars(GTK_EDITABLE(w2), 0, -1);
		if ((sarray[1]!=NULL) && (strcmp(str2, sarray[1])!=0))
			gtk_entry_set_text(GTK_ENTRY(w2), sarray[1]);
		g_free(str2);
		g_strfreev (sarray);
		g_free(str);
	}
}

void Preferences::set_gconf_from_time(GtkWidget *w, GtkWidget *w2, gchar *key)
{
	GError *error;
	gchar *str, *str2, *str3;

	str = gtk_editable_get_chars(GTK_EDITABLE(w), 0, -1);
	str2 = gtk_editable_get_chars(GTK_EDITABLE(w2), 0, -1);
	str3 = g_strjoin(" ", str, str2, NULL);
	gconf_client_set_string (main->client, key, str3, &error);
	g_free(str3);
	g_free(str2);
	g_free(str);
}

void Preferences::set_simple_clist_from_gconf(GtkWidget *clist, gchar *key)
{
	GSList *strings, *tmp;
	gchar *data[1];
	gchar *str;

	strings = gconf_client_get_list(main->client, key,
		GCONF_VALUE_STRING, NULL);
 	if (strings==NULL) return;
	tmp = strings;

	gtk_clist_freeze(GTK_CLIST(clist));
	gtk_clist_clear(GTK_CLIST(clist));

	while(tmp!=NULL)
	{
		str = (gchar*) tmp->data;
		if (str!=NULL)
 		{
			data[0] = strdup(str);
			gtk_clist_append(GTK_CLIST(clist), data);
			delete data[0];
		}
		tmp = g_slist_next(tmp);
	}

	gtk_clist_set_sort_column (GTK_CLIST(clist), 0);
	gtk_clist_sort (GTK_CLIST (clist));
	gtk_clist_thaw(GTK_CLIST(clist));
 	g_slist_foreach (strings, (GFunc)g_free, NULL);
	g_slist_free (strings);
}

void Preferences::set_gconf_from_simple_clist(GtkWidget *clist, gchar *key)
{
	GError *error;
	GSList *slist = NULL;
	gint i, num;
	gchar *data;

	num = GTK_CLIST(clist)->rows;
	for (i=0; i<num; i++)
	{
		gtk_clist_get_text(GTK_CLIST(clist), i, 0, &data);
		slist = g_slist_append(slist, data);
	}
	gconf_client_set_list (main->client,
		key, GCONF_VALUE_STRING, (GSList *) slist, &error);
	g_slist_free (slist);
}

//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
////////////                                                      ////////////
////////////                   Notify Actions                     ////////////
////////////                                                      ////////////
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////

void Preferences::config_update(gchar *key, gint refresh)
{
	gchar *str;
	gint val;

	// if any local variables are used they should be set here (only here)

	if (strcmp(key, CONFIG_SESSION_STARTUP)==0)
	{
		set_editable_from_gconf(session_name_entry, key);
	}else
	if (strcmp(key, CONFIG_SESSION_AUTO_SAVE_EVERY)==0)
	{
		set_togglebutton_from_gconf(session_auto_save, key);
		autosave_flag = gconf_client_get_bool(main->client, key, NULL);
	}else
	if (strcmp(key, CONFIG_SESSION_SAVE_SESSION_TIME)==0)
	{
		set_time_from_gconf(session_auto_save_time1,
			session_auto_save_time2, key);
		str = gconf_client_get_string(main->client, key, NULL);
		val = timestr_to_int(str);
		if (val<MIN_SAVE_SESSION_TIME) val=MIN_SAVE_SESSION_TIME;
		main->scheduler->autosave_interval = val;
	}else
	if (strcmp(key, CONFIG_SESSION_AUTO_SAVE_ON_QUIT)==0)
	{
		set_togglebutton_from_gconf(session_save_on_exit, key);
		save_on_quit_flag = gconf_client_get_bool(main->client, key, NULL);
	}else
	if (strcmp(key, CONFIG_SESSION_PING_TIME)==0)
	{
		set_time_from_gconf(session_ping_time1, session_ping_time2, key);
		str = gconf_client_get_string(main->client, key, NULL);
		val = timestr_to_int(str);
		if (val<MIN_PING_DEVICES_TIME) val=MIN_PING_DEVICES_TIME;
		main->scheduler->ping_interval = val;
	}else
	if (strcmp(key, CONFIG_SESSION_TOPOLOGY_TIME)==0)
	{
		set_time_from_gconf(session_regrab_time1, session_regrab_time2, key);
		str = gconf_client_get_string(main->client, key, NULL);
		val = timestr_to_int(str);
		if (val<MIN_REGRAB_TOPOLOGY_TIME) val=MIN_REGRAB_TOPOLOGY_TIME;
		main->scheduler->topology_interval = val;
	}else
	if (strcmp(key, CONFIG_SESSION_TOPOLOGY_DEFAULT)==0)
		set_togglebutton_from_gconf(session_topology_default, key);
	else
	if (strcmp(key, CONFIG_SESSION_GRAPH_DEFAULT)==0)
		set_togglebutton_from_gconf(session_graph_default, key);
	else
	if (strcmp(key, CONFIG_IMPORT_AUTO_IMPORT_FILES)==0)
	{
		set_togglebutton_from_gconf(import_auto, key);
		import_auto_flag = gconf_client_get_bool(main->client, key, NULL);
	}else
	if (strcmp(key, CONFIG_IMPORT_IMPORT_TIME)==0)
	{
		set_time_from_gconf(import_time1, import_time2, key);
		str = gconf_client_get_string(main->client, key, NULL);
		val = timestr_to_int(str);
		if (val<MIN_IMPORT_TIME) val=MIN_IMPORT_TIME;
		main->scheduler->import_interval = val;
	}else
	if (strcmp(key, CONFIG_IMPORT_FILES)==0)
	{
		set_simple_clist_from_gconf(import_clist, key);
		import_row = -1;
	}else
	if (strcmp(key, CONFIG_SUPPORT_PING)==0)
		set_editable_from_gconf(support_ping_entry, key);
	else
	if (strcmp(key, CONFIG_SUPPORT_TELNET)==0)
		set_editable_from_gconf(support_telnet_entry, key);
	else
	if (strcmp(key, CONFIG_SUPPORT_TRACEROUTE)==0)
		set_editable_from_gconf(support_traceroute_entry, key);
	else
	if (strcmp(key, CONFIG_SEARCH_AUTO_ZOOM)==0)
		set_togglebutton_from_gconf(search_zoom_checkbutton, key);
	else
	if (strcmp(key, CONFIG_SEARCH_AUTO_HIDE)==0)
		set_togglebutton_from_gconf(search_hide_checkbutton, key);
	else
	if (strcmp(key, CONFIG_SEARCH_SHOW_HIDDEN)==0)
		set_togglebutton_from_gconf(search_hidden_checkbutton, key);
	else
	if (strcmp(key, CONFIG_SEARCH_SELECT_MATCHES)==0)
		set_togglebutton_from_gconf(search_all_checkbutton, key);
	else
	if (strcmp(key, CONFIG_DIAGRAM_ANIMATE_ZOOM)==0)
	{
		set_togglebutton_from_gconf(default_animate_zoom, key);
		main->gui->monitor->viewer->animate_version =
			gconf_client_get_bool(main->client, key, NULL);
	}else
	if (strcmp(key, CONFIG_DIAGRAM_SHOW_LABELS)==0)
		main->gui->monitor->labels();
	else
	if (strcmp(key, CONFIG_DIAGRAM_SHOW_LINE_COLORS)==0)
		main->gui->monitor->colour();
	else
	if (strcmp(key, CONFIG_DIAGRAM_SHOW_LEAF)==0)
	{
		set_togglebutton_from_gconf(default_show_leaf, key);
		gtk_widget_set_sensitive(default_show_empty, gconf_client_get_bool(main->client, CONFIG_DIAGRAM_SHOW_LEAF, NULL));
	}else
	if (strcmp(key, CONFIG_DIAGRAM_SHOW_EMPTY_LEAF)==0)
		set_togglebutton_from_gconf(default_show_empty, key);
	else
	if (strcmp(key, CONFIG_DIAGRAM_GROUP_LEAF)==0)
		set_editable_from_gconf(default_group_leaf_type, key);
	else
	if (strcmp(key, CONFIG_DIAGRAM_NETWORK_TYPE)==0)
	{
		set_editable_from_gconf(default_network_type, key);
		str = gconf_client_get_string(main->client, key, NULL);
		main->topology->default_network_type =
			(strcmp(str, "Cloud")==0)?network_cloud:network_auto;
	}else
	if (strcmp(key, CONFIG_DIAGRAM_NETWORK_POSITION)==0)
		set_editable_from_gconf(default_network_position, key);
	else
	if (strcmp(key, CONFIG_DIAGRAM_SHOW_MACHINES)==0)
		set_togglebutton_from_gconf(default_show_machines, key);
	else
	if (strcmp(key, CONFIG_DIAGRAM_GROUP_MACHINES)==0)
		set_editable_from_gconf(default_group_machines, key);
	else
	if (strcmp(key, CONFIG_DIAGRAM_FONT)==0)
	{
		set_font_from_gconf(font_default, key);
		if (refresh==1) main->gui->update_views();
	}else
	if (strcmp(key, CONFIG_DIAGRAM_BACKGROUND)==0)
	{
		set_color_from_gconf(color_background, key);
		main->gui->assign_color(key);
		if (refresh==1) main->gui->update_views();
	}else
	if (strcmp(key, CONFIG_DIAGRAM_LIST_BG_1)==0)
	{
		set_color_from_gconf(color_list1, key);
		main->gui->assign_color(key);
		if (refresh==1) main->gui->update_lists();
	}else
	if (strcmp(key, CONFIG_DIAGRAM_LIST_BG_2)==0)
	{
		set_color_from_gconf(color_list2, key);
		main->gui->assign_color(key);
		if (refresh==1) main->gui->update_lists();
	}else
	if (strcmp(key, CONFIG_DIAGRAM_DEVICE_UP)==0)
	{
		set_color_from_gconf(color_device_up, key);
		main->gui->assign_color(key);
		if (refresh==1) main->gui->update_views();
	}else
	if (strcmp(key, CONFIG_DIAGRAM_DEVICE_DOWN)==0)
	{
		set_color_from_gconf(color_device_down, key);
		main->gui->assign_color(key);
		if (refresh==1) main->gui->update_views();
	}else
	if (strcmp(key, CONFIG_DEVICE_UP_SELECTED)==0)
	{
		set_color_from_gconf(color_selected_device_up, key);
		main->gui->assign_color(key);
		if (refresh==1) main->gui->update_views();
	}else
	if (strcmp(key, CONFIG_DEVICE_DOWN_SELECTED)==0)
	{
		set_color_from_gconf(color_selected_device_down, key);
		main->gui->assign_color(key);
		if (refresh==1) main->gui->update_views();
	}else
	if (strcmp(key, CONFIG_DIAGRAM_MACHINE)==0)
	{
		set_color_from_gconf(color_machine, key);
		main->gui->assign_color(key);
		if (refresh==1) main->gui->update_views();
	}else
	if (strcmp(key, CONFIG_DIAGRAM_MACHINE_SELECTED)==0)
	{
		set_color_from_gconf(color_selected_machine, key);
		main->gui->assign_color(key);
		if (refresh==1) main->gui->update_views();
	}else
	if (strcmp(key, CONFIG_DIAGRAM_NETWORK)==0)
	{
		set_color_from_gconf(color_network, key);
		main->gui->assign_color(key);
		if (refresh==1) main->gui->update_views();
	}else
	if (strcmp(key, CONFIG_DIAGRAM_NETWORK_EMPTY)==0)
	{
		set_color_from_gconf(color_empty_network, key);
		main->gui->assign_color(key);
		if (refresh==1) main->gui->update_views();
	}else
	if (strcmp(key, CONFIG_DIAGRAM_NETWORK_SELECTED)==0)
	{
		set_color_from_gconf(color_selected_network, key);
		main->gui->assign_color(key);
		if (refresh==1) main->gui->update_views();
	}else
	if (strcmp(key, CONFIG_COMMUNITY)==0)
	{
		set_simple_clist_from_gconf(community_clist, key);
		community_row = -1;
	}else

// we should ignore all notifiers for sample rate as not instant apply
/*
	if ((strcmp(key, CONFIG_SAMPLE_FREQUENCY)==0) ||
			(strcmp(key, CONFIG_SAMPLE_DURATION)==0))
	{
	}else
	if (strcmp(key, CONFIG_SAMPLE_ERRORS)==0)
	{
	}else
	if (strcmp(key, CONFIG_SAMPLE_REPLACE)==0)
	{
	}else
*/

	if (strcmp(key, CONFIG_EMAIL_SENDER_ADDRESS)==0)
		set_editable_from_gconf(email_sender_address, key);
	else
	if (strcmp(key, CONFIG_EMAIL_SMTP_SERVER)==0)
		set_editable_from_gconf(email_smtp_server, key);
	else
	if (strcmp(key, CONFIG_EMAIL_PORT)==0)
		set_spinbutton_from_gconf(email_port_number, key);
	else
	if (strcmp(key, CONFIG_EMAIL_SUBJECT)==0)
		set_editable_from_gconf(email_message_subject, key);
	else
	if (strcmp(key, CONFIG_EMAIL_BODY)==0)
		set_editable_from_gconf(email_message_body, key);
	else
	if (strcmp(key, CONFIG_SHOW_SIDEBAR)==0)
		main->gui->monitor->sidepanel();
	else
	if (strcmp(key, CONFIG_SHOW_STATUSBAR)==0)
		main->gui->monitor->statbar();
}

//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
////////////                                                      ////////////
////////////         Callbacks that need to update gconf          ////////////
////////////                                                      ////////////
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////

void Preferences::config_widget_action(GtkWidget *w)
{
	if (w == session_name_entry)
		set_gconf_from_editable(w, CONFIG_SESSION_STARTUP); else
	if (w == session_auto_save)
		set_gconf_from_togglebutton(w, CONFIG_SESSION_AUTO_SAVE_EVERY); else
	if ((w == session_auto_save_time1) || (w == session_auto_save_time2))
		set_gconf_from_time(session_auto_save_time1, session_auto_save_time2,
			CONFIG_SESSION_SAVE_SESSION_TIME); else
	if (w == session_save_on_exit)
		set_gconf_from_togglebutton(w, CONFIG_SESSION_AUTO_SAVE_ON_QUIT); else
	if ((w == session_ping_time1) || (w == session_ping_time2))
		set_gconf_from_time(session_ping_time1, session_ping_time2,
			CONFIG_SESSION_PING_TIME); else
	if ((w == session_regrab_time1) || (w == session_regrab_time2))
		set_gconf_from_time(session_regrab_time1, session_regrab_time2,
			CONFIG_SESSION_TOPOLOGY_TIME); else
	if (w == session_topology_default)
		set_gconf_from_togglebutton(w, CONFIG_SESSION_TOPOLOGY_DEFAULT); else
	if (w == session_graph_default)
		set_gconf_from_togglebutton(w, CONFIG_SESSION_GRAPH_DEFAULT); else
	if (w == import_auto)
		set_gconf_from_togglebutton(w, CONFIG_IMPORT_AUTO_IMPORT_FILES); else
	if ((w == import_time1) || (w == import_time2))
		set_gconf_from_time(import_time1, import_time2, CONFIG_IMPORT_IMPORT_TIME);
	else
	if ((w == import_add) || (w == import_filename))
		import_add_action(); else
	if (w == import_delete)
		import_del_action(); else
	if (w == import_import)
		import_import_action(TRUE); else
	if (w == support_ping_entry)
		set_gconf_from_editable(w, CONFIG_SUPPORT_PING); else
	if (w == support_telnet_entry)
		set_gconf_from_editable(w, CONFIG_SUPPORT_TELNET); else
	if (w == support_traceroute_entry)
		set_gconf_from_editable(w, CONFIG_SUPPORT_TRACEROUTE); else
	if (w == search_zoom_checkbutton)
		set_gconf_from_togglebutton(w, CONFIG_SEARCH_AUTO_ZOOM); else
	if (w == search_hide_checkbutton)
		set_gconf_from_togglebutton(w, CONFIG_SEARCH_AUTO_HIDE); else
	if (w == search_hidden_checkbutton)
		set_gconf_from_togglebutton(w, CONFIG_SEARCH_SHOW_HIDDEN); else
	if (w == search_all_checkbutton)
		set_gconf_from_togglebutton(w, CONFIG_SEARCH_SELECT_MATCHES); else
	if (w == default_animate_zoom)
		set_gconf_from_togglebutton(w, CONFIG_DIAGRAM_ANIMATE_ZOOM); else
	if (w == default_show_leaf)
		set_gconf_from_togglebutton(w, CONFIG_DIAGRAM_SHOW_LEAF); else
	if (w == default_show_empty)
		set_gconf_from_togglebutton(w, CONFIG_DIAGRAM_SHOW_EMPTY_LEAF); else
	if (w == default_group_leaf_type)
		set_gconf_from_editable(w, CONFIG_DIAGRAM_GROUP_LEAF); else
	if (w == default_network_type)
		set_gconf_from_editable(w, CONFIG_DIAGRAM_NETWORK_TYPE); else
	if (w == default_network_position)
		set_gconf_from_editable(w, CONFIG_DIAGRAM_NETWORK_POSITION); else
	if (w == default_show_machines)
		set_gconf_from_togglebutton(w, CONFIG_DIAGRAM_SHOW_MACHINES); else
	if (w == default_group_machines)
		set_gconf_from_editable(w, CONFIG_DIAGRAM_GROUP_MACHINES); else
	if (w == font_default)
		set_gconf_from_font(w, CONFIG_DIAGRAM_FONT); else
	if (w == color_background)
		set_gconf_from_color(w, CONFIG_DIAGRAM_BACKGROUND); else
	if (w == color_list1)
		set_gconf_from_color(w, CONFIG_DIAGRAM_LIST_BG_1); else
	if (w == color_list2)
		set_gconf_from_color(w, CONFIG_DIAGRAM_LIST_BG_2); else
	if (w == color_device_up)
		set_gconf_from_color(w, CONFIG_DIAGRAM_DEVICE_UP); else
	if (w == color_device_down)
		set_gconf_from_color(w, CONFIG_DIAGRAM_DEVICE_DOWN); else
	if (w == color_selected_device_up)
		set_gconf_from_color(w, CONFIG_DEVICE_UP_SELECTED); else
	if (w == color_selected_device_down)
		set_gconf_from_color(w, CONFIG_DEVICE_DOWN_SELECTED); else
	if (w == color_machine)
		set_gconf_from_color(w, CONFIG_DIAGRAM_MACHINE); else
	if (w == color_selected_machine)
		set_gconf_from_color(w, CONFIG_DIAGRAM_MACHINE_SELECTED); else
	if (w == color_network)
		set_gconf_from_color(w, CONFIG_DIAGRAM_NETWORK); else
	if (w == color_empty_network)
		set_gconf_from_color(w, CONFIG_DIAGRAM_NETWORK_EMPTY); else
	if (w == color_selected_network)
		set_gconf_from_color(w, CONFIG_DIAGRAM_NETWORK_SELECTED); else

	if ((w == community_add_button) || (w == community_entry))
		community_add_action(); else
	if (w == community_delete_button)
		community_del_action(); else

	if ((w == sample_errors1) || (w == sample_errors2) ||
			(w == sample_rate1) || (w == sample_rate2) || (w == sample_replace))
	{
		gtk_widget_set_sensitive(sample_apply, TRUE);
		gtk_widget_set_sensitive(sample_revert, TRUE);
		if (w != sample_replace) sample_refresh2();
	}else
	if (w == sample_add) sample_add_action(); else
	if (w == sample_delete) sample_delete_action(); else
	if (w == sample_apply) sample_apply_action(); else
	if (w == sample_revert) sample_revert_action(); else

	if (w == email_sender_address)
		set_gconf_from_editable(w, CONFIG_EMAIL_SENDER_ADDRESS); else
	if (w == email_smtp_server)
		set_gconf_from_editable(w, CONFIG_EMAIL_SMTP_SERVER); else
	if (w == email_port_number)
		set_gconf_from_spinbutton(w, CONFIG_EMAIL_PORT); else
	if (w == email_message_subject)
		set_gconf_from_editable(w, CONFIG_EMAIL_SUBJECT); else
	if (w == email_message_body)
		set_gconf_from_editable(w, CONFIG_EMAIL_BODY);
}

void Preferences::config_clist_select_action(GtkWidget *w, int row)
{
	if (w == import_clist)
	{
		import_row = row;
		gtk_widget_set_sensitive(import_delete, TRUE);
	}else
	if (w == community_clist)
	{
		community_row = row;
		gtk_widget_set_sensitive(community_delete_button, TRUE);
	}else
	if (w == sample_clist)
	{
		sample_row = row;
		gtk_widget_set_sensitive(sample_delete, TRUE);
	}
}

void Preferences::config_clist_unselect_action(GtkWidget *w, int row)
{
	if (w == import_clist)
	{
		gtk_widget_set_sensitive(import_delete, FALSE);
		import_row = -1;
	}else
	if (w == community_clist)
	{
		gtk_widget_set_sensitive(community_delete_button, FALSE);
		community_row = -1;
	}else
	if (w == sample_clist)
	{
		gtk_widget_set_sensitive(sample_delete, FALSE);
		sample_row = -1;
	}
}

//////////////////////////////////////////////////////////////////////////////
////////////                       Global                         ////////////
//////////////////////////////////////////////////////////////////////////////

void Preferences::global_init()
{
	int i;

	for (i=0; gconf_key[i] != NULL; i++)
		config_update(gconf_key[i], 0);

	// load xml file for notifier, levels, actions and timeframes
	load_xml();

	gtk_widget_set_sensitive(import_delete, FALSE);
	import_row = -1;

	gtk_widget_set_sensitive(community_delete_button, FALSE);
	community_row = -1;

	gtk_widget_set_sensitive(sample_delete, FALSE);
	gtk_widget_set_sensitive(sample_apply, FALSE);
	gtk_widget_set_sensitive(sample_revert, FALSE);
	sample_row = -1;
	sample_load();

	gtk_widget_set_sensitive(timeframe_table, FALSE);
	gtk_widget_set_sensitive(timeframe_delete_button, FALSE);
	gtk_widget_set_sensitive(timeframe_name_textentry, FALSE);
	gtk_widget_set_sensitive(timeframe_name_label, FALSE);
	current_timeframe = -1;
	timeframes.empty();

	gtk_widget_set_sensitive(action_frame, FALSE);
	gtk_widget_set_sensitive(level_color_picker, FALSE);
	gtk_widget_set_sensitive(level_color_label, FALSE);
	gtk_widget_set_sensitive(level_delete_button, FALSE);
	gtk_widget_set_sensitive(level_name_entry, FALSE);
	gtk_widget_set_sensitive(level_name_label, FALSE);
	gtk_entry_set_text(GTK_ENTRY(level_name_entry), "");
	gnome_color_picker_set_i8(GNOME_COLOR_PICKER(level_color_picker), 0, 0, 0, 0);
	gtk_clist_clear(GTK_CLIST(action_clist));
	gtk_entry_set_text(GTK_ENTRY(GTK_COMBO(action_type_combo)->entry), "");
	gtk_notebook_set_page(GTK_NOTEBOOK(action_notebook), 0);
	current_level = -1;

	gtk_widget_set_sensitive(notify_table, FALSE);
	gtk_widget_set_sensitive(notify_hbox69, FALSE);
	gtk_widget_set_sensitive(notify_hbox75, FALSE);
	gtk_widget_set_sensitive(notify_delete_button, FALSE);
	gtk_entry_set_text(GTK_ENTRY(GTK_COMBO(notify_event_type_combo)->entry), "");
	gtk_entry_set_text(GTK_ENTRY(GTK_COMBO(notify_time_frame_combo)->entry), "");
	gtk_entry_set_text(GTK_ENTRY(GTK_COMBO(notify_level_combo)->entry), "");
	gtk_entry_set_text(GTK_ENTRY(notify_threshold_entry), "");
	notify_event_type_list();
	current_rule = -1;

	main->gui->update_views();
}

//////////////////////////////////////////////////////////////////////////////
////////////                      Import                          ////////////
//////////////////////////////////////////////////////////////////////////////

void Preferences::import_add_action()
{
	gchar *data[1];

	gtk_clist_freeze(GTK_CLIST(import_clist));
	data[0] = strdup(gtk_entry_get_text(GTK_ENTRY(import_filename)));
	if (strcmp(data[0], "")!=0)
		gtk_clist_append(GTK_CLIST(import_clist), data);
	delete data[0];
	gtk_clist_thaw(GTK_CLIST(import_clist));
	gtk_entry_set_text(GTK_ENTRY(import_filename), "");
	set_gconf_from_simple_clist(import_clist, CONFIG_IMPORT_FILES);
}

void Preferences::import_del_action()
{
	gtk_clist_remove(GTK_CLIST(import_clist), import_row);
	set_gconf_from_simple_clist(import_clist, CONFIG_IMPORT_FILES);
}

void Preferences::import_import_action(gboolean show_progress)
{
	GSList *strings, *tmp;
	gchar *str;

	strings = gconf_client_get_list(main->client, CONFIG_IMPORT_FILES,
		GCONF_VALUE_STRING, NULL);
 	if (strings==NULL) return;
	tmp = strings;

	while(tmp!=NULL)
	{
		str = g_strdup((gchar*) tmp->data);
		main->session->import_init(str);
		tmp = g_slist_next(tmp);
	}
 	g_slist_foreach (strings, (GFunc)g_free, NULL);
	g_slist_free (strings);
	main->session->import(show_progress);
}

//////////////////////////////////////////////////////////////////////////////
////////////                     Community                        ////////////
//////////////////////////////////////////////////////////////////////////////

void Preferences::community_add_action()
{
	gchar *data[1];

	gtk_clist_freeze(GTK_CLIST(community_clist));
	data[0] = strdup(gtk_entry_get_text(GTK_ENTRY(community_entry)));
	if (strcmp(data[0], "")!=0)
		gtk_clist_append(GTK_CLIST(community_clist), data);
	delete data[0];
	gtk_clist_thaw(GTK_CLIST(community_clist));
	gtk_entry_set_text(GTK_ENTRY(community_entry), "");
	set_gconf_from_simple_clist(community_clist, CONFIG_COMMUNITY);
}

void Preferences::community_del_action()
{
	gtk_clist_remove(GTK_CLIST(community_clist), community_row);
	set_gconf_from_simple_clist(community_clist, CONFIG_COMMUNITY);
}

gchar *get_community_list_string(GConfClient* client)
{
	GSList *strings, *tmp;
	gchar *str, *val, *key;
	gint flag = 0;

	strings = gconf_client_get_list(client, CONFIG_COMMUNITY,
		GCONF_VALUE_STRING, NULL);
 	if (strings==NULL) { str = g_strdup("public"); return str; }
	tmp = strings;

	while(tmp!=NULL)
	{
		val = (gchar*) tmp->data;
		if (val!=NULL)
		{
			if (flag==0)
				str = g_strdup(val);
			else
			{
				key = g_strjoin(" ", str, val, NULL);
				g_free(str);
				str = key;
			}
			flag = 1;
		}
		tmp = g_slist_next(tmp);
	}

 	g_slist_foreach (strings, (GFunc)g_free, NULL);
	g_slist_free (strings);

	if (flag == 0) { str = g_strdup("public"); }
	return str;
}

//////////////////////////////////////////////////////////////////////////////
////////////                    Sample Rate                       ////////////
//////////////////////////////////////////////////////////////////////////////

// load on startup or on revert
void Preferences::sample_load()
{
	GSList *slist, *tmp;
	SampleRate *s;
	Link *ln;
	gint num;

	for (ln=samples.get_head(); ln!=NULL; ln=ln->next)
	{ s = (SampleRate*) ln->item; if (s!=NULL) delete s; } samples.empty();

	slist = gconf_client_get_list(main->client, CONFIG_SAMPLE_FREQUENCY,
		GCONF_VALUE_INT, NULL);
	tmp = slist;
	while (tmp!=NULL)
	{
		num = GPOINTER_TO_INT(tmp->data);
		s = new SampleRate();
		s->frequency = num;
		samples.tag(s);
		tmp = g_slist_next(tmp);
	}
	g_slist_free(slist);

	slist = gconf_client_get_list(main->client, CONFIG_SAMPLE_DURATION,
		GCONF_VALUE_INT, NULL);
	tmp = slist;
	ln = samples.get_head();
	while ((tmp!=NULL) && (ln!=NULL))
	{
		s = (SampleRate*) ln->item;
		num = GPOINTER_TO_INT(tmp->data);
		s->duration = num;
		ln = ln->next;
		tmp = g_slist_next(tmp);
	}
	g_slist_free(slist);

	samples.sort(compare_sample_rates, BUBBLE_SORT);

	set_time_from_gconf(sample_errors1, sample_errors2, CONFIG_SAMPLE_ERRORS);
	set_time_from_gconf(sample_rate1, sample_rate2, CONFIG_SAMPLE_RATE);
	set_togglebutton_from_gconf(sample_replace, CONFIG_SAMPLE_REPLACE);

	sample_refresh();
	sample_refresh2();
	gtk_widget_set_sensitive(sample_apply, FALSE);
	gtk_widget_set_sensitive(sample_revert, FALSE);
}

// save on accept
void Preferences::sample_save()
{
	SampleRate *s;
	GSList *slist;
	Link *ln;

	slist = NULL;
	for (ln=samples.get_head(); ln!=NULL; ln=ln->next)
	{
		s = (SampleRate*) ln->item;
		slist = g_slist_append(slist, GINT_TO_POINTER(s->frequency));
	}
	gconf_client_set_list(main->client, CONFIG_SAMPLE_FREQUENCY,
		GCONF_VALUE_INT, (GSList *) slist, NULL);
	g_slist_free(slist);

	slist = NULL;
	for (ln=samples.get_head(); ln!=NULL; ln=ln->next)
	{
		s = (SampleRate*) ln->item;
		slist = g_slist_append(slist, GINT_TO_POINTER(s->duration));
	}
	gconf_client_set_list(main->client, CONFIG_SAMPLE_DURATION,
		GCONF_VALUE_INT, (GSList *) slist, NULL);
	g_slist_free(slist);

	set_gconf_from_time(sample_errors1, sample_errors2, CONFIG_SAMPLE_ERRORS);
	set_gconf_from_time(sample_rate1, sample_rate2, CONFIG_SAMPLE_RATE);
	set_gconf_from_togglebutton(sample_replace, CONFIG_SAMPLE_REPLACE);

	gtk_widget_set_sensitive(sample_apply, FALSE);
	gtk_widget_set_sensitive(sample_revert, FALSE);
}

void Preferences::sample_refresh()
{
	gchar *data[1];
	gchar *str, *str2, *str3;
	char spd[128];
	SampleRate *s;
	Link *ln;
	gint row, num;

	row = 0;
	gtk_clist_freeze(GTK_CLIST(sample_clist));
	gtk_clist_clear(GTK_CLIST(sample_clist));
	for (ln=samples.get_head(); ln!=NULL; ln=ln->next)
	{
		s = (SampleRate*) ln->item;
		str2 = strdup(int_to_timestr(s->frequency));
		str3 = strdup(int_to_timestr(s->duration));
		str = g_strjoin(" ", "Every", str2, "for", str3, NULL);
		data[0] = strdup(str);
		if (strcmp(data[0], "")!=0)
		{
			gtk_clist_append(GTK_CLIST(sample_clist), data);
			gtk_clist_set_row_data(GTK_CLIST(sample_clist), row, s); row++;
		}
		delete data[0];
		g_free(str3); g_free(str2); g_free(str);
	}
	gtk_clist_thaw(GTK_CLIST(sample_clist));

	num = 0;
	for (ln=samples.get_head(); ln!=NULL; ln=ln->next)
	{
		s = (SampleRate*) ln->item;
		if (s->frequency > 0)
		num += (s->duration/s->frequency);
	}
	num *= 89;							// SAMPLE_SIZE
	num += 8 + (33*row);		// header

	if (num > 8192)
		sprintf(spd, "Current disk space requirements %d Kbytes / interface",
			(int)num/1024);
	else
		sprintf(spd, "Current disk space requirements %d bytes / interface", num);
	gtk_label_set_text(GTK_LABEL(sample_label), spd);
}

void Preferences::sample_refresh2()
{
	gchar *str, *str2, *str3;
	char spd[128];
	gint32 val, err;

	str = gtk_editable_get_chars(GTK_EDITABLE(sample_errors1), 0, -1);
	str2 = gtk_editable_get_chars(GTK_EDITABLE(sample_errors2), 0, -1);
	str3 = g_strjoin(" ", str, str2, NULL);
	val = timestr_to_int(str3);
	g_free(str3); g_free(str2); g_free(str);

	str = gtk_editable_get_chars(GTK_EDITABLE(sample_rate1), 0, -1);
	str2 = gtk_editable_get_chars(GTK_EDITABLE(sample_rate2), 0, -1);
	str3 = g_strjoin(" ", str, str2, NULL);
	err = timestr_to_int(str3);
	g_free(str3); g_free(str2); g_free(str);

	if (err<120) err=120;
	val /= err;
	val *= 65;			// size of sample
	val += 25;			// header

	if (val > 8192)
		sprintf(spd, "Current disk space requirements %d Kbytes / interface",
			(int)val/1024);
	else
		sprintf(spd, "Current disk space requirements %d bytes / interface", val);
	gtk_label_set_text(GTK_LABEL(sample_error_label), spd);
}

void Preferences::sample_add_action()
{
	GtkWidget *dialog;
	gchar *str, *str2, *str3;
	gint freq, dur;
	SampleRate *s;

	str = gtk_editable_get_chars(GTK_EDITABLE(sample_frequency1), 0, -1);
	str2 = gtk_editable_get_chars(GTK_EDITABLE(sample_frequency2), 0, -1);
	str3 = g_strjoin(" ", str, str2, NULL);
	freq = timestr_to_int(str3);
	g_free(str3); g_free(str2); g_free(str);

	str = gtk_editable_get_chars(GTK_EDITABLE(sample_duration1), 0, -1);
	str2 = gtk_editable_get_chars(GTK_EDITABLE(sample_duration2), 0, -1);
	str3 = g_strjoin(" ", str, str2, NULL);
	dur = timestr_to_int(str3);
	g_free(str3); g_free(str2); g_free(str);

	if (freq<=dur)
	{
		s = new SampleRate();
		s->frequency = freq;
		s->duration = dur;
		samples.tag(s);
		samples.sort(compare_sample_rates, BUBBLE_SORT);
		sample_refresh();
		gtk_widget_set_sensitive(sample_apply, TRUE);
		gtk_widget_set_sensitive(sample_revert, TRUE);
	}else
	{
		dialog = gnome_error_dialog(
			"Duration can't be shorter than the sample frequency");
		gnome_dialog_run_and_close (GNOME_DIALOG (dialog));
	}
}

void Preferences::sample_delete_action()
{
	SampleRate *s;

	s = (SampleRate*)gtk_clist_get_row_data(GTK_CLIST(sample_clist), sample_row);
	samples.del(s);
	delete s;
	samples.sort(compare_sample_rates, BUBBLE_SORT);
	sample_refresh();
	gtk_widget_set_sensitive(sample_delete, FALSE);
	gtk_widget_set_sensitive(sample_apply, TRUE);
	gtk_widget_set_sensitive(sample_revert, TRUE);
	sample_row = -1;
}

void Preferences::sample_apply_action()
{
	GtkWidget *dialog;
	gint choice;

	dialog = gnome_question_dialog_modal("Making changes to these settings may\nremove data collected by the graph server.\nAre you sure you want to apply changes?", NULL, NULL);
	choice = gnome_dialog_run_and_close (GNOME_DIALOG (dialog));
	if (choice==0)
	{
		sample_save();
	}
}

void Preferences::sample_revert_action()
{
	sample_load();
}

//////////////////////////////////////////////////////////////////////////////
////////////                      Notify                          ////////////
//////////////////////////////////////////////////////////////////////////////

void Preferences::notify_event_type_list()
{
	GList *items = NULL;
	int i;

	for (i=0; i<12; i++)
		items = g_list_append(items, rule_names[i]);
	gtk_combo_set_popdown_strings(GTK_COMBO(notify_event_type_combo), items);
}

gint Preferences::rules_add(Rule *r)
{
	gchar *data[5];
	gint row;

	data[0] = strdup(rule_names[0]);
	data[1] = strdup("");
	data[2] = strdup("");
	data[3] = strdup("");
	data[4] = strdup("");
	row = gtk_clist_append(GTK_CLIST(notify_clist), data);
	delete data[0]; delete data[1]; delete data[2];
	delete data[3]; delete data[4];
	gtk_clist_set_row_data(GTK_CLIST(notify_clist), row, r);
	return row;
}

void Preferences::notify_new_button_action()
{
	Rule *r;
	gint row;

	r = new Rule();
	rules.tag(r);
	row = rules_add(r);
	gtk_clist_select_row(GTK_CLIST(notify_clist), row, 0);
}

void Preferences::notify_delete_button_action()
{
}

void Preferences::notify_clist_select_action(int row)
{
	Rule *r;
//	char buf[64];

	current_rule = row;
	r = (Rule*) gtk_clist_get_row_data(GTK_CLIST(notify_clist), row);

	gtk_widget_set_sensitive(notify_table, TRUE);
	gtk_widget_set_sensitive(notify_hbox69, TRUE);
	gtk_widget_set_sensitive(notify_hbox75, TRUE);
	gtk_widget_set_sensitive(notify_delete_button, TRUE);


/*
	if (a->type==0) strcpy(buf, "Send Email"); else
	if (a->type==1) strcpy(buf, "Play Sound"); else strcpy(buf, "Run Script");
	gtk_entry_set_text(GTK_ENTRY(action_command_line_entry), (a->type==2)?a->param:"");
	gtk_entry_set_text(GTK_ENTRY(action_email_address_entry), (a->type==0)?a->param:"");
	gtk_entry_set_text(GTK_ENTRY(action_sound_file_entry), (a->type==1)?a->param:"");
	gtk_entry_set_text(GTK_ENTRY(GTK_COMBO(action_type_combo)->entry), buf);
	gtk_notebook_set_page(GTK_NOTEBOOK(action_notebook), a->type+1);
*/
}

void Preferences::notify_clist_unselect_action()
{
	gtk_widget_set_sensitive(notify_table, FALSE);
	gtk_widget_set_sensitive(notify_hbox69, FALSE);
	gtk_widget_set_sensitive(notify_hbox75, FALSE);
	gtk_widget_set_sensitive(notify_delete_button, FALSE);
	current_rule = -1;
	gtk_entry_set_text(GTK_ENTRY(GTK_COMBO(notify_event_type_combo)->entry), "");
	gtk_entry_set_text(GTK_ENTRY(GTK_COMBO(notify_time_frame_combo)->entry), "");
	gtk_entry_set_text(GTK_ENTRY(GTK_COMBO(notify_level_combo)->entry), "");
	gtk_entry_set_text(GTK_ENTRY(notify_threshold_entry), "");
}

void Preferences::notify_event_type_combo_action()
{
}

void Preferences::notify_level_combo_action()
{
}

void Preferences::notify_time_frame_combo_action()
{
}

void Preferences::notify_threshold_entry_action()
{
}

void Preferences::notify_percentage_checkbutton_action()
{
}

void Preferences::notify_offset_checkbutton_action()
{
}

void Preferences::notify_occurance_spinbutton_action()
{
}







//////////////////////////////////////////////////////////////////////////////
////////////                       Level                          ////////////
//////////////////////////////////////////////////////////////////////////////

gint Preferences::level_add(Level *l)
{
	gchar *data[1];
	gint row;

	data[0] = strdup(l->name);
	row = gtk_clist_append(GTK_CLIST(level_clist), data);
	delete data[0];
	gtk_clist_set_row_data(GTK_CLIST(level_clist), row, l);
	return row;
}

void Preferences::level_new_button_action()
{
	Level *l;
	gint row;

	l = new Level();
	levels.tag(l);
	row = level_add(l);
	gtk_clist_select_row(GTK_CLIST(level_clist), row, 0);
}

void Preferences::level_delete_button_action()
{
	Level *l;

	l = (Level*) gtk_clist_get_row_data(GTK_CLIST(level_clist), current_level);
	if (l==NULL) return;
	levels.del(l);
	delete l;
	gtk_clist_remove(GTK_CLIST(level_clist), current_level);
	level_clist_unselect_action();
}

void Preferences::level_clist_select_action(gint row)
{
	Level *l;
	guint r, g, b;

	current_level = row;
	l = (Level*) gtk_clist_get_row_data(GTK_CLIST(level_clist), row);
	gtk_widget_set_sensitive(action_frame, TRUE);
	gtk_widget_set_sensitive(level_color_picker, TRUE);
	gtk_widget_set_sensitive(level_color_label, TRUE);
	gtk_widget_set_sensitive(level_delete_button, TRUE);
	gtk_widget_set_sensitive(level_name_entry, TRUE);
	gtk_widget_set_sensitive(level_name_label, TRUE);

	gtk_entry_set_text(GTK_ENTRY(level_name_entry), l->name);

	// set color in picker
	sscanf(l->color, "#%2X%2X%2X", &r, &g, &b);
	gnome_color_picker_set_i8(GNOME_COLOR_PICKER(level_color_picker), r, g, b, 0);

	// init action list
	action_init(l);
}

void Preferences::level_clist_unselect_action()
{
	gtk_widget_set_sensitive(action_frame, FALSE);
	gtk_widget_set_sensitive(level_color_picker, FALSE);
	gtk_widget_set_sensitive(level_color_label, FALSE);
	gtk_widget_set_sensitive(level_delete_button, FALSE);
	gtk_widget_set_sensitive(level_name_entry, FALSE);
	gtk_widget_set_sensitive(level_name_label, FALSE);
	current_level = -1;
	gtk_entry_set_text(GTK_ENTRY(level_name_entry), "");	
	gnome_color_picker_set_i8(GNOME_COLOR_PICKER(level_color_picker), 0, 0, 0, 0);
	gtk_clist_clear(GTK_CLIST(action_clist));
	gtk_entry_set_text(GTK_ENTRY(GTK_COMBO(action_type_combo)->entry), "");
	gtk_notebook_set_page(GTK_NOTEBOOK(action_notebook), 0);
}

void Preferences::level_name_entry_action()
{
	Level *l;
	char buf[256];

	l = (Level*) gtk_clist_get_row_data(GTK_CLIST(level_clist), current_level);
	if (l==NULL) return;
	strcpy(buf, gtk_entry_get_text(GTK_ENTRY(level_name_entry)));
	gtk_clist_set_text (GTK_CLIST(level_clist), current_level, 0, buf);
	strcpy(l->name, buf);
}

void Preferences::level_color_picker_action()
{
	Level *l;
	guint8 r, g, b, a;
	gchar str[9];

	l = (Level*) gtk_clist_get_row_data(GTK_CLIST(level_clist), current_level);
	if (l==NULL) return;
	gnome_color_picker_get_i8 (GNOME_COLOR_PICKER(level_color_picker), &r, &g, &b, &a);
	snprintf(str, 9, "#%02X%02X%02X", r, g, b);
	strcpy(l->color, str);
}

//////////////////////////////////////////////////////////////////////////////
////////////                      Action                          ////////////
//////////////////////////////////////////////////////////////////////////////

void Preferences::action_init(Level *l)
{
	Link *ln;
	Action *a;
	gchar *data[2];
	char buf[65];
	gint row;

	current_action = -1;
	gtk_widget_set_sensitive(action_type_combo, FALSE);
	gtk_widget_set_sensitive(action_delete_button, FALSE);
	gtk_widget_set_sensitive(action_test_button, FALSE);
	gtk_widget_set_sensitive(action_notebook, FALSE);
	gtk_entry_set_text(GTK_ENTRY(GTK_COMBO(action_type_combo)->entry), "");

	// load action list into clist
	gtk_clist_clear(GTK_CLIST(action_clist));
	for (ln=l->actions.get_head(); ln!=NULL; ln=ln->next)
	{
		a = (Action*) ln->item;
		if (a->type==0) strcpy(buf, "Send Email"); else
		if (a->type==1) strcpy(buf, "Play Sound"); else strcpy(buf, "Run Script");
		data[0] = strdup(buf);
		data[1] = strdup(a->param);
		row = gtk_clist_append(GTK_CLIST(action_clist), data);
		delete data[0]; delete data[1];
		gtk_clist_set_row_data(GTK_CLIST(action_clist), row, a);
	}
}

void Preferences::action_new_button_action()
{
	gchar *data[2];
	Action *a;
	Level *l;
	gint row;

	l = (Level*) gtk_clist_get_row_data(GTK_CLIST(level_clist), current_level);
	if (l==NULL) return;

	a = new Action();
	l->actions.tag(a);

	data[0] = strdup("Send Email");
	data[1] = strdup("");
	row = gtk_clist_append(GTK_CLIST(action_clist), data);
	delete data[0]; delete data[1];
	gtk_clist_set_row_data(GTK_CLIST(action_clist), row, a);
	gtk_clist_select_row(GTK_CLIST(action_clist), row, 0);
}

void Preferences::action_delete_button_action()
{
	Action *a;
	Level *l;

	l = (Level*) gtk_clist_get_row_data(GTK_CLIST(level_clist), current_level);
	if (l==NULL) return;
	a = (Action*) gtk_clist_get_row_data(GTK_CLIST(action_clist), current_action);
	if (a==NULL) return;
	l->actions.del(a);
	delete a;
	gtk_clist_remove(GTK_CLIST(action_clist), current_action);
	action_clist_unselect_action();
}

void Preferences::action_test_button_action()
{
}

void Preferences::action_type_combo_action()
{
	Action *a;
	char buf[64];
	int i;

	a = (Action*) gtk_clist_get_row_data(GTK_CLIST(action_clist), current_action);
	if (a==NULL) return;
	strcpy(buf, gtk_entry_get_text(GTK_ENTRY(GTK_COMBO(action_type_combo)->entry)));
	gtk_clist_set_text(GTK_CLIST(action_clist), current_action, 0, buf);
	gtk_clist_set_text(GTK_CLIST(action_clist), current_action, 1, a->param);
	if (strcmp(buf, "Send Email")==0) { a->type = action_send_email; i=0; } else
	if (strcmp(buf, "Play Sound")==0) { a->type = action_play_sound; i=1; } else
	{ a->type = action_run_script; i=2; }
	if (a->type==action_send_email)
		strcpy(a->param, gtk_entry_get_text(GTK_ENTRY(action_email_address_entry)));
	else
	if (a->type==action_play_sound)
		strcpy(a->param, gtk_entry_get_text(GTK_ENTRY(action_sound_file_entry)));
	else
		strcpy(a->param, gtk_entry_get_text(GTK_ENTRY(action_command_line_entry)));
	gtk_notebook_set_page(GTK_NOTEBOOK(action_notebook), i+1);
}

void Preferences::action_clist_select_action(int row)
{
	Action *a;
	char buf[64];
	int i;

	current_action = row;
	a = (Action*) gtk_clist_get_row_data(GTK_CLIST(action_clist), row);
	gtk_widget_set_sensitive(action_type_combo, TRUE);
	gtk_widget_set_sensitive(action_delete_button, TRUE);
	gtk_widget_set_sensitive(action_test_button, TRUE);
	gtk_widget_set_sensitive(action_notebook, TRUE);
	if (a->type==0) strcpy(buf, "Send Email"); else
	if (a->type==1) strcpy(buf, "Play Sound"); else strcpy(buf, "Run Script");
	gtk_entry_set_text(GTK_ENTRY(action_command_line_entry), (a->type==action_run_script)?a->param:"");
	gtk_entry_set_text(GTK_ENTRY(action_email_address_entry), (a->type==action_send_email)?a->param:"");
	gtk_entry_set_text(GTK_ENTRY(action_sound_file_entry), (a->type==action_play_sound)?a->param:"");
	gtk_entry_set_text(GTK_ENTRY(GTK_COMBO(action_type_combo)->entry), buf);
	gtk_notebook_set_page(GTK_NOTEBOOK(action_notebook), i+1);
}

void Preferences::action_clist_unselect_action()
{
	gtk_widget_set_sensitive(action_type_combo, FALSE);
	gtk_widget_set_sensitive(action_delete_button, FALSE);
	gtk_widget_set_sensitive(action_test_button, FALSE);
	gtk_widget_set_sensitive(action_notebook, FALSE);
	current_action = -1;
	gtk_entry_set_text(GTK_ENTRY(GTK_COMBO(action_type_combo)->entry), "");
	gtk_notebook_set_page(GTK_NOTEBOOK(action_notebook), 0);
}

void Preferences::action_command_line_entry_action()
{
	Action *a;
	char buf[256];

	a = (Action*) gtk_clist_get_row_data(GTK_CLIST(action_clist), current_action);
	if (a==NULL) return;
	strcpy(buf, gtk_entry_get_text(GTK_ENTRY(action_command_line_entry)));
	gtk_clist_set_text (GTK_CLIST(action_clist), current_action, 1, buf);
	if (a->type==2) strcpy(a->param, buf);
}

void Preferences::action_email_address_entry_action()
{
	Action *a;
	char buf[256];

	a = (Action*) gtk_clist_get_row_data(GTK_CLIST(action_clist), current_action);
	if (a==NULL) return;
	strcpy(buf, gtk_entry_get_text(GTK_ENTRY(action_email_address_entry)));
	gtk_clist_set_text (GTK_CLIST(action_clist), current_action, 1, buf);
	if (a->type==0) strcpy(a->param, buf);
}

void Preferences::action_sound_file_entry_action()
{
	Action *a;
	char buf[256];

	a = (Action*) gtk_clist_get_row_data(GTK_CLIST(action_clist), current_action);
	if (a==NULL) return;
	strcpy(buf, gtk_entry_get_text(GTK_ENTRY(action_sound_file_entry)));
	gtk_clist_set_text (GTK_CLIST(action_clist), current_action, 1, buf);
	if (a->type==1) strcpy(a->param, buf);
}

//////////////////////////////////////////////////////////////////////////////
////////////                     Time Frame                       ////////////
//////////////////////////////////////////////////////////////////////////////

gint Preferences::timeframe_add(TimeFrame *t)
{
	gchar *data[3];
	char buf[64];
	gint row;

	data[0] = strdup(t->descr);
	sprintf(buf, "%d:%02d", t->start/60, t->start%60);
	data[1] = strdup(buf);
	sprintf(buf, "%d:%02d", t->end/60, t->end%60);
	data[2] = strdup(buf);
	data[3] = strdup("");
	data[4] = strdup((t->inverse_flag=='0')?"No":"Yes");
	row = gtk_clist_append(GTK_CLIST(timeframe_clist), data);
	delete data[0]; delete data[1]; delete data[2];
	delete data[3]; delete data[4];
	gtk_clist_set_row_data(GTK_CLIST(timeframe_clist), row, t);
	return row;
}

void Preferences::timeframe_new_button_action()
{
	TimeFrame *t;
	gint row;

	t = new TimeFrame();
	timeframes.tag(t);
	row = timeframe_add(t);
	gtk_clist_select_row(GTK_CLIST(timeframe_clist), row, 0);
}

void Preferences::timeframe_delete_button_action()
{
	TimeFrame *t;

	t = (TimeFrame*) gtk_clist_get_row_data(GTK_CLIST(timeframe_clist),
		current_timeframe);
	if (t==NULL) return;
	timeframes.del(t);
	delete t;
	gtk_clist_remove(GTK_CLIST(timeframe_clist), current_timeframe);
	timeframe_clist_unselect_action();
}

void Preferences::timeframe_clist_select_action(gint row)
{
	TimeFrame *t;
	char buf[64];

	current_timeframe = row;
	t = (TimeFrame*) gtk_clist_get_row_data(GTK_CLIST(timeframe_clist), row);
	gtk_widget_set_sensitive(timeframe_table, TRUE);
	gtk_widget_set_sensitive(timeframe_delete_button, TRUE);
	gtk_widget_set_sensitive(timeframe_name_textentry, TRUE);
	gtk_widget_set_sensitive(timeframe_name_label, TRUE);
	gtk_entry_set_text(GTK_ENTRY(timeframe_name_textentry), t->descr);
	sprintf(buf, "%d:%02d", t->start/60, t->start%60);
	gtk_entry_set_text(GTK_ENTRY(GTK_COMBO(timeframe_start_combo)->entry), buf);
	sprintf(buf, "%d:%02d", t->end/60, t->end%60);
	gtk_entry_set_text(GTK_ENTRY(GTK_COMBO(timeframe_end_combo)->entry), buf);
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(timeframe_invert_checkbutton),
		(t->inverse_flag=='0')?FALSE:TRUE);
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(timeframe_mon_checkbutton),
		((t->daymask&1)==1)?TRUE:FALSE);
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(timeframe_tue_checkbutton),
		((t->daymask&2)==2)?TRUE:FALSE);
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(timeframe_wed_checkbutton),
		((t->daymask&4)==4)?TRUE:FALSE);
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(timeframe_thu_checkbutton),
		((t->daymask&8)==8)?TRUE:FALSE);
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(timeframe_fri_checkbutton),
		((t->daymask&16)==16)?TRUE:FALSE);
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(timeframe_sat_checkbutton),
		((t->daymask&32)==32)?TRUE:FALSE);
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(timeframe_sun_checkbutton),
		((t->daymask&64)==64)?TRUE:FALSE);

	timeframe_day_string(current_timeframe);
}

void Preferences::timeframe_clist_unselect_action()
{
	gtk_widget_set_sensitive(timeframe_table, FALSE);
	gtk_widget_set_sensitive(timeframe_delete_button, FALSE);
	gtk_widget_set_sensitive(timeframe_name_textentry, FALSE);
	gtk_widget_set_sensitive(timeframe_name_label, FALSE);
	current_timeframe = -1;
}

void Preferences::timeframe_name_textentry_action()
{
	TimeFrame *t;
	char buf[256];

	t = (TimeFrame*) gtk_clist_get_row_data(GTK_CLIST(timeframe_clist),
		current_timeframe);
	if (t==NULL) return;
	strcpy(buf, gtk_entry_get_text(GTK_ENTRY(timeframe_name_textentry)));
	gtk_clist_set_text (GTK_CLIST(timeframe_clist), current_timeframe, 0, buf);
	strcpy(t->descr, buf);
}

void Preferences::timeframe_invert_checkbutton_action()
{
	TimeFrame *t;
	t = (TimeFrame*) gtk_clist_get_row_data(GTK_CLIST(timeframe_clist),
		current_timeframe);
	if (t==NULL) return;
	t->inverse_flag = (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(timeframe_invert_checkbutton))==TRUE)?'1':'0';
	gtk_clist_set_text(GTK_CLIST(timeframe_clist), current_timeframe, 4, (t->inverse_flag=='0')?"No":"Yes");
}

void Preferences::timeframe_start_combo_action()
{
	TimeFrame *t;
	char buf[64];
	int a, b;

	t = (TimeFrame*) gtk_clist_get_row_data(GTK_CLIST(timeframe_clist),
		current_timeframe);
	if (t==NULL) return;
	strcpy(buf, gtk_entry_get_text(GTK_ENTRY(GTK_COMBO(timeframe_start_combo)->entry)));
	gtk_clist_set_text(GTK_CLIST(timeframe_clist), current_timeframe, 1, buf);
	sscanf(buf, "%d:%02d", &a, &b);
	t->start = (a*60)+b;
}

void Preferences::timeframe_end_combo_action()
{
	TimeFrame *t;
	char buf[64];
	int a, b;

	t = (TimeFrame*) gtk_clist_get_row_data(GTK_CLIST(timeframe_clist),
		current_timeframe);
	if (t==NULL) return;
	strcpy(buf, gtk_entry_get_text(GTK_ENTRY(GTK_COMBO(timeframe_end_combo)->entry)));
	gtk_clist_set_text(GTK_CLIST(timeframe_clist), current_timeframe, 2, buf);
	sscanf(buf, "%d:%02d", &a, &b);
	t->end = (a*60)+b;
}

void Preferences::timeframe_day_action(GtkWidget *w)
{
	TimeFrame *t;
	int size;

	if (w==timeframe_mon_checkbutton) size = 1; else
	if (w==timeframe_tue_checkbutton) size = 2; else
	if (w==timeframe_wed_checkbutton) size = 4; else
	if (w==timeframe_thu_checkbutton) size = 8; else
	if (w==timeframe_fri_checkbutton) size = 16; else
	if (w==timeframe_sat_checkbutton) size = 32; else
	if (w==timeframe_sun_checkbutton) size = 64; else return;

	t = (TimeFrame*) gtk_clist_get_row_data(GTK_CLIST(timeframe_clist),
		current_timeframe);
	if (t==NULL) return;
	t->daymask = (t->daymask&(255-size));
	if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(w)) == TRUE) t->daymask += size;
	timeframe_day_string(current_timeframe);
}

void Preferences::timeframe_day_string(int row)
{
	TimeFrame *t;
	char buf[16];

	t = (TimeFrame*) gtk_clist_get_row_data(GTK_CLIST(timeframe_clist),
		current_timeframe);
	if (t==NULL) return;
	sprintf(buf, "%s%s%s%s%s%s%s", (((t->daymask&1)==1)?"Mo":""),
		(((t->daymask&2)==2)?"Tu":""), (((t->daymask&4)==4)?"We":""),
		(((t->daymask&8)==8)?"Th":""), (((t->daymask&16)==16)?"Fr":""),
		(((t->daymask&32)==32)?"Sa":""), (((t->daymask&64)==64)?"Su":""));
	gtk_clist_set_text(GTK_CLIST(timeframe_clist), current_timeframe, 3, buf);
}

void Preferences::save_xml()
{
/*
	xmlDocPtr doc;
	Level *l;
	TimeFrame *t;
	Rule *r;
	Link *ln;
	char fname[256];

	sprintf(fname, "%s/rules", main->session->datadir);

	doc = xmlNewDoc((xmlChar*)"1.0");
	doc->root = xmlNewDocNode(doc, NULL, (xmlChar*)"Nomad_Rules", NULL);

	for (ln=levels.get_head(); ln!=NULL; ln=ln->next)
	{
		l = (Level*) ln->item;
		l->save(doc->root);
	}
	for (ln=timeframes.get_head(); ln!=NULL; ln=ln->next)
	{
		t = (TimeFrame*) ln->item;
		t->save(doc->root);
	}

	for (ln=rules.get_head(); ln!=NULL; ln=ln->next)
	{
		r = (Rule*) ln->item;
		r->save(doc->root);
	}

	xmlSaveFile(fname, doc);
	xmlFreeDoc(doc);
*/
}

void Preferences::load_xml()
{
/*
	xmlDocPtr doc;
	xmlNodePtr xmlnode;
	Level *l;
	TimeFrame *t;
	Rule *r;
	char fname[256];

	sprintf(fname, "%s/rules", main->session->datadir);
	doc = xmlParseFile(fname);

	if (!doc || !doc->root || !doc->root->name ||
			strcmp((char*)doc->root->name, "Nomad_Rules")!=0)
	{
		xmlFreeDoc(doc);
		return;
	}

	empty();
	xmlnode = doc->root->childs;
	while (xmlnode != NULL)
	{
		if (xmlnode->name!=NULL)
		{
			if (strcmp((char*)xmlnode->name, "Level")==0)
			{
				l = new Level();
				l->load(xmlnode);
				levels.tag(l);
				level_add(l);
				while(gtk_events_pending()) gtk_main_iteration();
			}else
			if (strcmp((char*)xmlnode->name, "TimeFrame")==0)
			{
				t = new TimeFrame();
				t->load(xmlnode);
				timeframes.tag(t);
				timeframe_add(t);
				while(gtk_events_pending()) gtk_main_iteration();
			}else
			if (strcmp((char*)xmlnode->name, "Network")==0)
			{
				r = new Rule();
				r->load(xmlnode);
				rules.tag(r);
				rules_add(r);
				while(gtk_events_pending()) gtk_main_iteration();
			}
		}
		xmlnode = xmlnode->next;
	}

	xmlFreeDoc(doc);
*/
}

