/* print.cc
 * Copyright (C) 2001 The University of Newcastle upon Tyne
 *
 * Written by Paul Coates (Paul.Coates@ncl.ac.uk)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 */

#include <gtk/gtk.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <stdio.h>
#include "main.h"
#include "gui.h"
#include "data.h"
#include "diagram.h"
#include "print.h"
#include "licon.xpm"
#include "session.h"

static struct _paper_sizes {
	char *name;
	int width;
	int height;
	int lmargin;
	int bmargin;
} paper_sizes[] = {
{"A4", 595, 842, 32, 32},
{"A3", 842, 1191, 32, 32},
{"A5", 420, 595, 32, 32}, 
{"B4", 729, 1032, 32, 32},
{"B5", 516, 729, 32, 32},
{"Letter", 612, 792, 32, 32}
};

static void width_changed_cb(GtkWidget *w, gpointer data)
{
	GtkAdjustment *adj = (GtkAdjustment*)w;
((Print*)data)->pages_wide = (gint) adj->value; ((Print*)data)->draw(); }

static void height_changed_cb(GtkWidget *w, gpointer data)
{
	GtkAdjustment *adj = (GtkAdjustment*)w;
((Print*)data)->pages_high = (gint) adj->value; ((Print*)data)->draw(); }

static void portrait_cb(GtkWidget *w, gpointer data)
{ ((Print*)data)->orientation = 0; ((Print*)data)->draw(); }
static void landscape_cb(GtkWidget *w, gpointer data)
{ ((Print*)data)->orientation = 1; ((Print*)data)->draw(); }
static void printer_cb(GtkWidget *w, gpointer data)
{
	Print *p = (Print*)data;
	gtk_notebook_set_page(GTK_NOTEBOOK(p->notebook1), 0);
	gtk_label_set_text(GTK_LABEL(p->label16), "Command");
	p->send_to_file = 0;
}

static void file_cb(GtkWidget *w, gpointer data)
{
	Print *p = (Print*)data;
	gtk_notebook_set_page(GTK_NOTEBOOK(p->notebook1), 1);
	gtk_label_set_text(GTK_LABEL(p->label16), "Filename");
	p->send_to_file = 1;
}

static void on_Close_activate(GtkWidget *w, gpointer data)
{
	((Print*)data)->hide();
}

static void on_OK_activate(GtkWidget *w, gpointer data)
{
	((Print*)data)->ok();
}

static void on_File_activate(GtkWidget *w, gpointer data)
{
//	((Print*)data)->main->gui->filesel->print();
}

static void paper_cb(GtkWidget *w, gpointer data)
{
	((Print*)data)->draw();
}

static gint delete_event(GtkWidget *widget, GdkEvent *event, gpointer data)
{
	((Print*)data)->hide();
	return (TRUE); 
}

static gint expose_event(GtkWidget *widget, GdkEventExpose *e, gpointer d)
{
	Print *m = (Print*)d;

	if (m->canvas!=NULL)
	{
		gdk_draw_pixmap(widget->window,
			widget->style->fg_gc[GTK_WIDGET_STATE (widget)],
			m->canvas, e->area.x, e->area.y, e->area.x, e->area.y,
			e->area.width, e->area.height);
	}
	return FALSE;
}

static gint configure_event(GtkWidget *w, GdkEventConfigure *e, gpointer d)
{
	((Print*)d)->draw();
	return TRUE;
}

Print::Print(Main *m)
{
	main = m;
	canvas = NULL;
	width = height = 0;
	print_group = NULL;
	file_group = NULL;
	strcpy(filename, "");
	page_types = (sizeof(paper_sizes) / sizeof(struct _paper_sizes));
	paper_width = paper_sizes[0].width - (2*paper_sizes[0].lmargin);
	paper_height = paper_sizes[0].height - (2*paper_sizes[0].bmargin);
	pages_wide = pages_high = 1;
	orientation = 0;
	send_to_file = 0;
	create();
	diagram = new Diagram(window3, main);

	gc = gdk_gc_new(window3->window);
  gdk_gc_set_line_attributes(gc, 1, GDK_LINE_SOLID, GDK_CAP_BUTT,
		GDK_JOIN_MITER);
}

Print::~Print()
{
	gdk_gc_unref(gc);
}

void Print::create()
{
	int i;

  window3 = gtk_window_new (GTK_WINDOW_TOPLEVEL);
  gtk_window_set_title (GTK_WINDOW (window3), "Network Monitor - Print");
  gtk_window_set_modal (GTK_WINDOW (window3), TRUE);
  gtk_window_set_policy (GTK_WINDOW (window3), FALSE, FALSE, FALSE);
  gtk_widget_realize (window3);
	window_icon(window3, licon_xpm);
	gdk_window_set_icon_name(window3->window, "Print");
	gtk_signal_connect(GTK_OBJECT(window3), "delete_event",
		GTK_SIGNAL_FUNC(delete_event), this);
	gtk_signal_connect(GTK_OBJECT(window3), "destroy",
		GTK_SIGNAL_FUNC(delete_event), this);

  hbox16 = gtk_hbox_new (FALSE, 0);
  gtk_widget_show (hbox16);
  gtk_container_add (GTK_CONTAINER (window3), hbox16);

  table4 = gtk_table_new (6, 3, FALSE);
  gtk_widget_show (table4);
  gtk_box_pack_start (GTK_BOX (hbox16), table4, TRUE, TRUE, 0);
  gtk_container_set_border_width (GTK_CONTAINER (table4), 4);
  gtk_table_set_row_spacings (GTK_TABLE (table4), 4);
  gtk_table_set_col_spacings (GTK_TABLE (table4), 4);

  label17 = gtk_label_new ("Print To");
  gtk_widget_show (label17);
  gtk_table_attach (GTK_TABLE (table4), label17, 0, 1, 0, 1,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);
  gtk_misc_set_alignment (GTK_MISC (label17), 1, 0.5);
  gtk_misc_set_padding (GTK_MISC (label17), 8, 0);

  label19 = gtk_label_new ("Orientation");
  gtk_widget_show (label19);
  gtk_table_attach (GTK_TABLE (table4), label19, 0, 1, 3, 4,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);
  gtk_misc_set_alignment (GTK_MISC (label19), 1, 0.5);
  gtk_misc_set_padding (GTK_MISC (label19), 8, 0);

  label20 = gtk_label_new ("Pages");
  gtk_widget_show (label20);
  gtk_table_attach (GTK_TABLE (table4), label20, 0, 1, 4, 5,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);
  gtk_misc_set_alignment (GTK_MISC (label20), 1, 0.5);
  gtk_misc_set_padding (GTK_MISC (label20), 8, 0);

  hbox19 = gtk_hbox_new (FALSE, 0);
  gtk_widget_show (hbox19);
  gtk_table_attach (GTK_TABLE (table4), hbox19, 1, 2, 4, 5,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (GTK_FILL), 0, 0);

  label25 = gtk_label_new ("Width");
  gtk_widget_show (label25);
  gtk_box_pack_start (GTK_BOX (hbox19), label25, TRUE, TRUE, 0);

  spinbutton2_adj = gtk_adjustment_new (1, 1, 5, 1, 1, 1);
  spinbutton2 = gtk_spin_button_new (GTK_ADJUSTMENT (spinbutton2_adj), 1, 0);
  gtk_widget_show (spinbutton2);
  gtk_box_pack_start (GTK_BOX (hbox19), spinbutton2, FALSE, FALSE, 0);
	gtk_signal_connect (GTK_OBJECT (spinbutton2_adj), "value_changed",
		GTK_SIGNAL_FUNC (width_changed_cb), this);

  hbox20 = gtk_hbox_new (FALSE, 0);
  gtk_widget_show (hbox20);
  gtk_table_attach (GTK_TABLE (table4), hbox20, 2, 3, 4, 5,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (GTK_FILL), 0, 0);

  label26 = gtk_label_new ("Height");
  gtk_widget_show (label26);
  gtk_box_pack_start (GTK_BOX (hbox20), label26, TRUE, TRUE, 0);

  spinbutton3_adj = gtk_adjustment_new (1, 1, 5, 1, 1, 1);
  spinbutton3 = gtk_spin_button_new (GTK_ADJUSTMENT (spinbutton3_adj), 1, 0);
  gtk_widget_show (spinbutton3);
  gtk_box_pack_start (GTK_BOX (hbox20), spinbutton3, FALSE, FALSE, 0);
	gtk_signal_connect (GTK_OBJECT (spinbutton3_adj), "value_changed",
		GTK_SIGNAL_FUNC (height_changed_cb), this);


  radiobutton19 = gtk_radio_button_new_with_label (print_group, "Portrait");
  print_group = gtk_radio_button_group (GTK_RADIO_BUTTON (radiobutton19));
  gtk_widget_show (radiobutton19);
  gtk_table_attach (GTK_TABLE (table4), radiobutton19, 1, 2, 3, 4,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);
	gtk_signal_connect (GTK_OBJECT(radiobutton19), "clicked",
		GTK_SIGNAL_FUNC(portrait_cb), this);

  radiobutton20 = gtk_radio_button_new_with_label (print_group, "Landscape");
  print_group = gtk_radio_button_group (GTK_RADIO_BUTTON (radiobutton20));
  gtk_widget_show (radiobutton20);
  gtk_table_attach (GTK_TABLE (table4), radiobutton20, 2, 3, 3, 4,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);
	gtk_signal_connect (GTK_OBJECT(radiobutton20), "clicked",
		GTK_SIGNAL_FUNC(landscape_cb), this);

  radiobutton17 = gtk_radio_button_new_with_label (file_group, "Printer");
  file_group = gtk_radio_button_group (GTK_RADIO_BUTTON (radiobutton17));
  gtk_widget_show (radiobutton17);
  gtk_table_attach (GTK_TABLE (table4), radiobutton17, 1, 2, 0, 1,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);
	gtk_signal_connect (GTK_OBJECT(radiobutton17), "clicked",
		GTK_SIGNAL_FUNC(printer_cb), this);

  radiobutton18 = gtk_radio_button_new_with_label (file_group, "File");
  file_group = gtk_radio_button_group (GTK_RADIO_BUTTON (radiobutton18));
  gtk_widget_show (radiobutton18);
  gtk_table_attach (GTK_TABLE (table4), radiobutton18, 2, 3, 0, 1,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);
	gtk_signal_connect (GTK_OBJECT(radiobutton18), "clicked",
		GTK_SIGNAL_FUNC(file_cb), this);

  hbox17 = gtk_hbox_new (FALSE, 8);
  gtk_widget_show (hbox17);
  gtk_table_attach (GTK_TABLE (table4), hbox17, 0, 3, 5, 6,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (GTK_FILL), 0, 0);
  gtk_container_set_border_width (GTK_CONTAINER (hbox17), 4);

  button9 = gtk_button_new_with_label ("Print");
  gtk_widget_show (button9);
  gtk_box_pack_start (GTK_BOX (hbox17), button9, TRUE, TRUE, 0);
  gtk_widget_set_usize (button9, 60, -2);
  gtk_signal_connect (GTK_OBJECT (button9), "clicked", GTK_SIGNAL_FUNC (on_OK_activate), this);

  button10 = gtk_button_new_with_label ("Close");
  gtk_widget_show (button10);
  gtk_box_pack_start (GTK_BOX (hbox17), button10, TRUE, TRUE, 0);
  gtk_widget_set_usize (button10, 60, -2);
  gtk_signal_connect (GTK_OBJECT (button10), "clicked", GTK_SIGNAL_FUNC (on_Close_activate), this);

  label16 = gtk_label_new ("Command");
  gtk_widget_show (label16);
  gtk_table_attach (GTK_TABLE (table4), label16, 0, 1, 1, 2,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);
  gtk_misc_set_alignment (GTK_MISC (label16), 1, 0.5);
  gtk_misc_set_padding (GTK_MISC (label16), 8, 0);

  notebook1 = gtk_notebook_new ();
  gtk_widget_show (notebook1);
  gtk_table_attach (GTK_TABLE (table4), notebook1, 1, 3, 1, 2,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (GTK_FILL), 0, 0);
  GTK_WIDGET_UNSET_FLAGS (notebook1, GTK_CAN_FOCUS);
  gtk_notebook_set_show_tabs (GTK_NOTEBOOK (notebook1), FALSE);
  gtk_notebook_set_show_border (GTK_NOTEBOOK (notebook1), FALSE);

  entry5 = gtk_entry_new ();
  gtk_widget_show (entry5);
  gtk_container_add (GTK_CONTAINER (notebook1), entry5);
  gtk_entry_set_text (GTK_ENTRY (entry5), "lpr");

  hbox18 = gtk_hbox_new (FALSE, 2);
  gtk_widget_show (hbox18);
  gtk_container_add (GTK_CONTAINER (notebook1), hbox18);

  entry6 = gtk_entry_new ();
  gtk_widget_show (entry6);
  gtk_box_pack_start (GTK_BOX (hbox18), entry6, TRUE, TRUE, 0);

  button8 = gtk_button_new_with_label ("...");
  gtk_widget_show (button8);
  gtk_box_pack_start (GTK_BOX (hbox18), button8, FALSE, FALSE, 0);
  gtk_signal_connect (GTK_OBJECT (button8), "clicked", GTK_SIGNAL_FUNC (on_File_activate), this);

  label22 = gtk_label_new ("Paper Size");
  gtk_widget_show (label22);
  gtk_table_attach (GTK_TABLE (table4), label22, 0, 1, 2, 3,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);
  gtk_misc_set_alignment (GTK_MISC (label22), 1, 0.5);
  gtk_misc_set_padding (GTK_MISC (label22), 8, 0);

  combo3 = gtk_combo_new ();
  gtk_widget_show (combo3);
  gtk_table_attach (GTK_TABLE (table4), combo3, 1, 3, 2, 3,
                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);
	gtk_entry_set_editable(GTK_ENTRY(GTK_COMBO(combo3)->entry), FALSE);

  viewport4 = gtk_viewport_new (NULL, NULL);
  gtk_widget_show (viewport4);
  gtk_box_pack_start (GTK_BOX (hbox16), viewport4, FALSE, TRUE, 0);
  gtk_container_set_border_width (GTK_CONTAINER (viewport4), 4);

  drawingarea4 = gtk_drawing_area_new ();
  gtk_widget_show (drawingarea4);
  gtk_container_add (GTK_CONTAINER (viewport4), drawingarea4);
  gtk_widget_set_usize (drawingarea4, 160, -2);

	combo1_items = NULL;
	for (i=0; i<page_types; i++)
	{
		combo1_items = g_list_append(combo1_items, paper_sizes[i].name);
	}
  gtk_combo_set_popdown_strings(GTK_COMBO(combo3), combo1_items);
  g_list_free(combo1_items);

	gtk_signal_connect (GTK_OBJECT (drawingarea4), "expose_event",
		(GtkSignalFunc) expose_event, this);
	gtk_signal_connect (GTK_OBJECT(drawingarea4),"configure_event",
		(GtkSignalFunc) configure_event, this);
	gtk_signal_connect(GTK_OBJECT(GTK_COMBO(combo3)->list), "selection_changed",
		GTK_SIGNAL_FUNC (paper_cb), this);
}

void Print::show()
{
	gtk_widget_show(window3);
	gdk_window_raise(window3->window);
}

void Print::hide()
{
	gtk_widget_hide(window3);
}

void Print::draw()
{
	char *string;
	gint ow, oh, tx, ty, nw, nh, i;
	float pw, ph, ratio;
//	float t1, t2;
//	float ratio2;

	oh = height;
	ow = width;

//	main->files->width_height();

	height = drawingarea4->allocation.height;
	width = drawingarea4->allocation.width;

	if ((canvas == NULL) || (oh != height) || (ow != width))
	{
		if (canvas) gdk_pixmap_unref(canvas);
		canvas = gdk_pixmap_new(drawingarea4->window, width, height, -1);
	}

// work out paper size
	string = gtk_entry_get_text(GTK_ENTRY(GTK_COMBO(combo3)->entry));
	for (i=0; i<page_types; i++)
	{
		if (strcmp(string, paper_sizes[i].name)==0)
		{
			paper_width = paper_sizes[i].width - (2*paper_sizes[i].lmargin);
			paper_height = paper_sizes[i].height - (2*paper_sizes[i].bmargin);
		}
	}

// adjust for different orientation
	if (orientation == 1)
	{
		i = paper_width;
		paper_width = paper_height;
		paper_height = i;
	}

	nw = paper_width*pages_wide;
	nh = paper_height*pages_high;

	pw = (width-20)/(float)nw;
	ph = (height-20)/(float)nh;
	if (pw>ph) ratio = ph; else ratio = pw;
	ow = (gint)(ratio*nw);
	oh = (gint)(ratio*nh);
	tx = (width-ow)/2;
	ty = (height-oh)/2;

// clear window
	int x, y, w, h;
	GtkStyle *style;

	x = drawingarea4->allocation.width;
	y = drawingarea4->allocation.height;
	w = x/128;
	h = y/128;
	style = gtk_widget_get_style(window3);
	for (x=0; x<=w; x++)
		for (y=0; y<=h; y++)
			gtk_draw_flat_box(style, canvas, GTK_STATE_NORMAL,
				GTK_SHADOW_NONE, x*128, y*128, 128, 128);

// draw shadow
/*
	gdk_gc_set_foreground(gc, &main->gui->darkgray);
	gdk_draw_rectangle(canvas, gc, FALSE, tx+1, ty+1, ow, oh);

// draw paper
	gdk_gc_set_foreground(gc, &main->gui->white);
	gdk_draw_rectangle(canvas, gc, TRUE, tx, ty, ow, oh);
	gdk_gc_set_foreground(gc, &main->gui->black);
	gdk_draw_rectangle(canvas, gc, FALSE, tx, ty, ow, oh);
*/
// draw multipage lines
	for (i=1; i<pages_wide; i++)
	{
		gdk_draw_line(canvas, gc, tx+(i*ow/pages_wide), ty, tx+(i*ow/pages_wide), ty+oh);
	}
	for (i=1; i<pages_high; i++)
	{
		gdk_draw_line(canvas, gc, tx, ty+(i*oh/pages_high), tx+ow, ty+(i*oh/pages_high));
	}

// draw diagram

/*
	t1 = float((diagram->data->width)/(float)(ow-8));
	t2 = float((diagram->data->height)/(float)(oh-8));
	diagram->draw(canvas, ((t1<t2)?t2:t1), diagram->data->width/2,
		diagram->data->height/2, width, height, 1);
*/

// force an expose event
	gdk_draw_pixmap(drawingarea4->window,
		drawingarea4->style->fg_gc[GTK_WIDGET_STATE(drawingarea4)],
		canvas, 0, 0, 0, 0, drawingarea4->allocation.width,
		drawingarea4->allocation.height);
}

void Print::ok()
{
//	char buf[256];
	char *string;
	int i, mw, mh;

	mw = mh = 0;

// work out paper size
	string = gtk_entry_get_text(GTK_ENTRY(GTK_COMBO(combo3)->entry));
	for (i=0; i<page_types; i++)
	{
		if (strcmp(string, paper_sizes[i].name)==0)
		{
			mw = paper_sizes[i].lmargin;
			mh = paper_sizes[i].bmargin;
			paper_width = paper_sizes[i].width - (2*mw);
			paper_height = paper_sizes[i].height - (2*mh);
		}
	}

// adjust for different orientation
	if (orientation == 1)
	{
		i = paper_width;
		paper_width = paper_height;
		paper_height = i;
	}
/*
	if (send_to_file == 1)
	{
		main->files->print_view(filename, paper_width, paper_height, pages_wide, pages_high, mw, mh);
	}else
	{
		sprintf(buf, "/tmp/network.%d", getpid());
		main->files->print_view(buf, paper_width, paper_height, pages_wide, pages_high, mw, mh);
		sprintf(buf, "%s /tmp/network.%d", gtk_entry_get_text(GTK_ENTRY(entry5)),
			getpid());
		system(buf);
		sprintf(buf, "rm -f /tmp/network.%d", getpid());
		system(buf);
	}
*/
	hide();
}

