/* stats.cc
 * Copyright (C) 2001 The University of Newcastle upon Tyne
 *
 * Written by Paul Coates (Paul.Coates@ncl.ac.uk)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 */

#include <glade/glade-xml.h>
#include <gtk/gtkinvisible.h>
#include <gtk/gtk.h>
#include <sys/time.h>
#include <sys/stat.h>
#include <string.h>
#include <stdlib.h>
#include <unistd.h>
#include <stdio.h>
#include <math.h>
#include <pwd.h>
#include "main.h"
#include "misc.h"
#include "session.h"
#include "data.h"
#include "gui.h"
#include "device.h"
#include "interface.h"
#include "stats.h"
#include "dataset.h"
#include "graphio.h"
#include "preferences.h"
#include "licon.xpm"
#include "intup.xpm"
#include "intdown.xpm"
#include "sleft.xpm"
#include "sright.xpm"
#include "stoday.xpm"
#include "sall.xpm"

static void on_Close_activate(GtkWidget *w, gpointer data)
{ ((Stats*)data)->hide(); }
static void nav_cb(GtkWidget *w, gpointer data)
{ ((Stats*)data)->nav_action(w); }
static void start_left_cb(GtkWidget *w, gpointer data)
{ ((Stats*)data)->start_left(); }
static void start_right_cb(GtkWidget *w, gpointer data)
{ ((Stats*)data)->start_right(); }
static void start_today_cb(GtkWidget *w, gpointer data)
{ ((Stats*)data)->start_today(); }
static void start_all_cb(GtkWidget *w, gpointer data)
{ ((Stats*)data)->start_all(); }
static void middle_left_cb(GtkWidget *w, gpointer data)
{ ((Stats*)data)->middle_left(); }
static void middle_right_cb(GtkWidget *w, gpointer data)
{ ((Stats*)data)->middle_right(); }
static void end_left_cb(GtkWidget *w, gpointer data)
{ ((Stats*)data)->end_left(); }
static void end_right_cb(GtkWidget *w, gpointer data)
{ ((Stats*)data)->end_right(); }
static void device_list_cb(GtkWidget *w, gpointer data)
{ ((Stats*)data)->device_list(); }
//static void interface_list_cb(GtkWidget *w, gpointer data)
//{ ((Stats*)data)->interface_list(); }
static void interface_clist_cb(GtkWidget *clist, gint row, gint column, GdkEventButton *event, gpointer data)
{
	Stats *m = (Stats*) data;
	Interface *in;

	in = (Interface*) gtk_clist_get_row_data(GTK_CLIST(clist), row);
	m->current_interface = in;
	m->load();

	gtk_clist_freeze(GTK_CLIST(clist));
	if (row==0)
		gtk_clist_moveto(GTK_CLIST(clist), row, 0, 0.0, 0.0);
	else
	if (row==GTK_CLIST(clist)->rows - 1)
		gtk_clist_moveto(GTK_CLIST(clist), row, 0, 1.0, 0.0);
	else
	if (gtk_clist_row_is_visible(GTK_CLIST(clist), row-1)!=GTK_VISIBILITY_FULL)
		gtk_clist_moveto(GTK_CLIST(clist), row-1, 0, 0.0, 0.0);
	else
	if (gtk_clist_row_is_visible(GTK_CLIST(clist), row+1)!=GTK_VISIBILITY_FULL)
		gtk_clist_moveto(GTK_CLIST(clist), row+1, 0, 1.0, 0.0);
	gtk_clist_thaw(GTK_CLIST(clist));
}

static void interface_clist_cb2(GtkWidget *clist, gint row, gint column, GdkEventButton *event, gpointer data)
{
	Stats *m = (Stats*) data;
	m->current_interface = NULL;
	m->draw();
}

static void radio1_cb(GtkWidget *w, gpointer data)
{
	Stats *s = (Stats*) data;
	s->packets->scale = (w == s->radio1)?0:1;
	s->draw();
}

static void radio2_cb(GtkWidget *w, gpointer data)
{
	Stats *s = (Stats*) data;
	s->errors->scale = (w == s->radio3)?0:1;
	s->draw();
}

// static void check_cb(GtkWidget *w, gpointer data)
// { ((Stats*)data)->set_check(w); }

static void scroll_cb(GtkWidget *w, gpointer data)
{
	Stats *s = (Stats*)data;
	GtkAdjustment *adj = GTK_ADJUSTMENT(s->scrollbar_adj);
	// work out new start position from adj->value
printf("SCROLLBAR VALUES val=%d, min=%d, max=%d, pagesize=%d\n", (int)adj->value, (int)adj->lower, (int)adj->upper, (int)adj->page_size);
	s->draw();
}

static gint delete_event(GtkWidget *widget, GdkEvent *event, gpointer data)
{
	((Stats*)data)->hide();
	return (TRUE);
}

static gint expose_event(GtkWidget *widget, GdkEventExpose *event, gpointer data)
{
	GdkPixmap *canvas = NULL;
	Stats *m = (Stats*)data;

	if (widget == m->drawingarea1) canvas = m->packets->canvas;
	if (widget == m->drawingarea2) canvas = m->errors->canvas;
	if (canvas != NULL)
	{
		gdk_draw_pixmap(widget->window,
			widget->style->fg_gc[GTK_WIDGET_STATE (widget)],
			canvas, event->area.x, event->area.y, event->area.x, event->area.y,
			event->area.width, event->area.height);
	}
	return FALSE;
}

static gint configure_event(GtkWidget *w, GdkEventConfigure *event, gpointer data)
{ ((Stats*)data)->draw(); return TRUE; }

int compare_router_names(Link *a, Link *b)
{
	Device *r1, *r2;

	r1 = (Device*) a->item;
	r2 = (Device*) b->item;

	return (strcmp(r1->name, r2->name));
}

Stats::Stats(Main *m)
{
	GList *glist = NULL;
	Link *ln;
	Device *d;
	GladeXML *gxml;
	GdkColor col;
	int i;

	main = m;
	current_interface = NULL;
	current_router = NULL;
	oldest_time.now();
	newest_time.now();
	day_range = 0;
	packets = new Graph(m);
	errors = new Graph(m);
	packets_ok = FALSE;
	errors_ok = FALSE;
//	draw_lock = FALSE;

//	canvas1 = NULL;
//	canvas2 = NULL;
//	height1 = width1 = 0;
//	height2 = width2 = 0;
//	scale1 = 0;
//	scale2 = 0;

//	iface = NULL;
//	strcpy(current_file, "");

	gchar *fname = user_file("graph.glade");
	gxml = glade_xml_new (fname, "graph");
	if (!gxml)
	{
		g_warning("Failed to load file graph.glade");
		return;
	}

	window                = glade_xml_get_widget (gxml, "graph");
	navbox                = glade_xml_get_widget (gxml, "scrolledwindow9");
	navbox2               = glade_xml_get_widget (gxml, "vbox21");
	nbutton1              = glade_xml_get_widget (gxml, "button17");
	npanel1               = glade_xml_get_widget (gxml, "hbox35");
	nbutton2              = glade_xml_get_widget (gxml, "button18");
	npanel2               = glade_xml_get_widget (gxml, "hbox37");
	clist6                = glade_xml_get_widget (gxml, "clist6");
	combo3                = glade_xml_get_widget (gxml, "combo3");
	button_utilization    = glade_xml_get_widget (gxml, "checkbutton3");
	button_packets_in     = glade_xml_get_widget (gxml, "checkbutton4");
	button_packets_out    = glade_xml_get_widget (gxml, "checkbutton7");
	button_broadcast_in   = glade_xml_get_widget (gxml, "checkbutton5");
	button_broadcast_out  = glade_xml_get_widget (gxml, "checkbutton8");
	button_multicast_in   = glade_xml_get_widget (gxml, "checkbutton6");
	button_multicast_out  = glade_xml_get_widget (gxml, "checkbutton9");
	button_errors_in      = glade_xml_get_widget (gxml, "checkbutton19");
	button_errors_out     = glade_xml_get_widget (gxml, "checkbutton20");
	label_utilization     = glade_xml_get_widget (gxml, "label54");
	label_packets_in      = glade_xml_get_widget (gxml, "label50");
	label_packets_out     = glade_xml_get_widget (gxml, "label55");
	label_broadcast_in    = glade_xml_get_widget (gxml, "label51");
	label_broadcast_out   = glade_xml_get_widget (gxml, "label56");
	label_multicast_in    = glade_xml_get_widget (gxml, "label52");
	label_multicast_out   = glade_xml_get_widget (gxml, "label57");
	label_errors_in       = glade_xml_get_widget (gxml, "label53");
	label_errors_out      = glade_xml_get_widget (gxml, "label58");
	button_dropped        = glade_xml_get_widget (gxml, "checkbutton12");
	button_crc            = glade_xml_get_widget (gxml, "checkbutton13");
	button_runts          = glade_xml_get_widget (gxml, "checkbutton14");
	button_giant          = glade_xml_get_widget (gxml, "checkbutton15");
	button_fragments      = glade_xml_get_widget (gxml, "checkbutton16");
	button_jabbers        = glade_xml_get_widget (gxml, "checkbutton17");
	button_collisions     = glade_xml_get_widget (gxml, "checkbutton18");
	label_dropped         = glade_xml_get_widget (gxml, "label59");
	label_crc             = glade_xml_get_widget (gxml, "label60");
	label_runts           = glade_xml_get_widget (gxml, "label61");
	label_giant           = glade_xml_get_widget (gxml, "label62");
	label_fragments       = glade_xml_get_widget (gxml, "label63");
	label_jabbers         = glade_xml_get_widget (gxml, "label64");
	label_collisions      = glade_xml_get_widget (gxml, "label65");
	button1               = glade_xml_get_widget (gxml, "button1");
	button2               = glade_xml_get_widget (gxml, "button2");
	button3               = glade_xml_get_widget (gxml, "button3");
	button4               = glade_xml_get_widget (gxml, "button4");
	button5               = glade_xml_get_widget (gxml, "button5");
	button6               = glade_xml_get_widget (gxml, "button6");
	button13              = glade_xml_get_widget (gxml, "button13");
	button15              = glade_xml_get_widget (gxml, "button15");
	button16              = glade_xml_get_widget (gxml, "button16");
	radio1                = glade_xml_get_widget (gxml, "radio1");
	radio2                = glade_xml_get_widget (gxml, "radio2");
	radio3                = glade_xml_get_widget (gxml, "radio3");
	radio4                = glade_xml_get_widget (gxml, "radio4");
	drawingarea1          = glade_xml_get_widget (gxml, "drawingarea1");
	drawingarea2          = glade_xml_get_widget (gxml, "drawingarea2");
	scrollbar             = glade_xml_get_widget (gxml, "hscrollbar1");
	scrollbar_adj         = GTK_RANGE(scrollbar)->adjustment;
	gtk_object_unref (GTK_OBJECT (gxml));
	g_free(fname);

	gtk_widget_set_usize (window, 560, 420);
	gtk_widget_realize (window);
	window_icon(window, licon_xpm);
	gdk_window_set_icon_name(window->window, "Preferences");
	gtk_signal_connect (GTK_OBJECT (window), "delete_event",
		GTK_SIGNAL_FUNC(delete_event), this);
	gtk_signal_connect(GTK_OBJECT(window), "destroy",
		GTK_SIGNAL_FUNC(delete_event), this);
	gtk_widget_realize(drawingarea1);
	gtk_widget_realize(drawingarea2);

	gtk_widget_show(npanel1);
	gtk_widget_show(npanel2);

	pix[0] = create_pixmap(window, sleft_xpm);
	pix[1] = create_pixmap(window, sright_xpm);
	pix[2] = create_pixmap(window, sall_xpm);
	pix[3] = create_pixmap(window, stoday_xpm);
	pix[4] = create_pixmap(window, sleft_xpm);
	pix[5] = create_pixmap(window, sright_xpm);
	pix[6] = create_pixmap(window, sleft_xpm);
	pix[7] = create_pixmap(window, sright_xpm);
	gtk_container_add (GTK_CONTAINER (button1), pix[0]);
	gtk_container_add (GTK_CONTAINER (button2), pix[1]);
	gtk_container_add (GTK_CONTAINER (button15), pix[2]);
	gtk_container_add (GTK_CONTAINER (button16), pix[3]);
	gtk_container_add (GTK_CONTAINER (button5), pix[4]);
	gtk_container_add (GTK_CONTAINER (button6), pix[5]);
	gtk_container_add (GTK_CONTAINER (button3), pix[6]);
	gtk_container_add (GTK_CONTAINER (button4), pix[7]);
	gtk_widget_set_sensitive(button1, FALSE);
	gtk_widget_set_sensitive(button2, FALSE);
	gtk_widget_set_sensitive(button3, FALSE);
	gtk_widget_set_sensitive(button4, FALSE);
	gtk_widget_set_sensitive(button5, FALSE);
	gtk_widget_set_sensitive(button6, FALSE);
	gtk_widget_set_sensitive(button15, FALSE);
	gtk_widget_set_sensitive(button16, FALSE);

	gtk_signal_connect(GTK_OBJECT(GTK_COMBO(combo3)->list), "selection_changed",
		GTK_SIGNAL_FUNC (device_list_cb), this);

	gtk_signal_connect (GTK_OBJECT (nbutton1), "pressed",
		GTK_SIGNAL_FUNC (nav_cb), this);
	gtk_signal_connect (GTK_OBJECT (nbutton2), "pressed",
		GTK_SIGNAL_FUNC (nav_cb), this);
	gtk_signal_connect (GTK_OBJECT (button1), "pressed",
		GTK_SIGNAL_FUNC (start_left_cb), this);
	gtk_signal_connect (GTK_OBJECT (button2), "pressed",
		GTK_SIGNAL_FUNC (start_right_cb), this);
	gtk_signal_connect (GTK_OBJECT (button16), "pressed",
		GTK_SIGNAL_FUNC (start_today_cb), this);
	gtk_signal_connect (GTK_OBJECT (button15), "pressed",
		GTK_SIGNAL_FUNC (start_all_cb), this);
	gtk_signal_connect (GTK_OBJECT (button3), "pressed",
		GTK_SIGNAL_FUNC (middle_left_cb), this);
	gtk_signal_connect (GTK_OBJECT (button4), "pressed",
		GTK_SIGNAL_FUNC (middle_right_cb), this);
	gtk_signal_connect (GTK_OBJECT (button5), "pressed",
		GTK_SIGNAL_FUNC (end_left_cb), this);
	gtk_signal_connect (GTK_OBJECT (button6), "pressed",
		GTK_SIGNAL_FUNC (end_right_cb), this);
	gtk_signal_connect (GTK_OBJECT (radio1), "pressed",
		GTK_SIGNAL_FUNC (radio1_cb), this);
	gtk_signal_connect (GTK_OBJECT (radio2), "pressed",
		GTK_SIGNAL_FUNC (radio1_cb), this);
	gtk_signal_connect (GTK_OBJECT (radio3), "pressed",
		GTK_SIGNAL_FUNC (radio2_cb), this);
	gtk_signal_connect (GTK_OBJECT (radio4), "pressed",
		GTK_SIGNAL_FUNC (radio2_cb), this);

	gtk_signal_connect(GTK_OBJECT(clist6), "select_row",
		GTK_SIGNAL_FUNC(interface_clist_cb), this);
	gtk_signal_connect(GTK_OBJECT(clist6), "unselect_row",
		GTK_SIGNAL_FUNC(interface_clist_cb2), this);
	gtk_signal_connect (GTK_OBJECT (scrollbar_adj), "value_changed",
		GTK_SIGNAL_FUNC (scroll_cb), this);

	gtk_signal_connect (GTK_OBJECT (button13), "pressed",
		GTK_SIGNAL_FUNC (on_Close_activate), this);
	gtk_signal_connect (GTK_OBJECT (drawingarea1), "expose_event",
		(GtkSignalFunc) expose_event, this);
	gtk_signal_connect (GTK_OBJECT(drawingarea1),"configure_event",
		(GtkSignalFunc) configure_event, this);
	gtk_signal_connect (GTK_OBJECT (drawingarea2), "expose_event",
		(GtkSignalFunc) expose_event, this);
	gtk_signal_connect (GTK_OBJECT(drawingarea2),"configure_event",
		(GtkSignalFunc) configure_event, this);

	alloc_color(lightgray, 49151, 49151, 49151);
	alloc_color(gray, 32767, 32767, 32767);
	alloc_color(darkgray, 16383, 16383, 16383);
	alloc_color(black, 0, 0, 0);

	gc      = gdk_gc_new(window->window);
	gridgc  = gdk_gc_new(window->window);
	gdk_gc_set_line_attributes(gc, 1, GDK_LINE_SOLID, GDK_CAP_BUTT,
		GDK_JOIN_MITER);

	main->session->data->devices.sort(compare_router_names, BUBBLE_SORT);

	for (ln=main->session->data->devices.get_head(); ln!=NULL; ln=ln->next)
	{
		d = (Device*) ln->item;
		glist = g_list_append(glist, d->name);
	}
	gtk_combo_set_popdown_strings(GTK_COMBO(combo3), glist);
	g_list_free(glist);

// FIXME colors should come from gconf
	key_color(label_utilization, "#ff0000");
	key_color(label_packets_in, "#ff7f00");
	key_color(label_broadcast_in, "#ffff00");
	key_color(label_multicast_in, "#7f7f00");
	key_color(label_errors_in, "#007f00");
	key_color(label_packets_out, "#0000ff");
	key_color(label_broadcast_out, "#ff00ff");
	key_color(label_multicast_out, "#7f007f");
	key_color(label_errors_out, "#7f0000");
	key_color(label_dropped, "#ff0000");
	key_color(label_crc, "#ff7f00");
	key_color(label_runts, "#ffff00");
	key_color(label_giant, "#007f00");
	key_color(label_fragments, "#0000ff");
	key_color(label_jabbers, "#ff00ff");
	key_color(label_collisions, "#7f007f");
}

Stats::~Stats()
{
	if (gc) gdk_gc_unref(gc);
	if (packets != NULL) delete packets;
	if (errors != NULL) delete errors;
	if (gridgc) gdk_gc_unref(gridgc);
//	delete check_states;
//	if (iface != NULL)
//		delete [] iface;
}

void Stats::show()
{
	load();
	gtk_widget_show(window);
	gdk_window_raise(window->window);
}

void Stats::hide()
{
	gtk_widget_hide(window);
}

void Stats::nav_action(GtkWidget *w)
{
	if (w == nbutton1)
	{
		if (GTK_WIDGET_VISIBLE(npanel1))
			gtk_widget_hide(npanel1); else gtk_widget_show(npanel1);
	}else
	if (w == nbutton2)
	{
		if (GTK_WIDGET_VISIBLE(npanel2))
			gtk_widget_hide(npanel2); else gtk_widget_show(npanel2);
	}
	// force the scrollbar to update
	gtk_widget_set_usize(navbox, 0, 0);
}

void Stats::device_list()
{
	Device *d;
	Link *ln;

	char *string = gtk_entry_get_text(GTK_ENTRY(GTK_COMBO(combo3)->entry));

	for (ln=main->session->data->devices.get_head(); ln!=NULL; ln=ln->next)
	{
		d = (Device*) ln->item;
		if (strcmp(d->name, string)==0)
		{
			current_router = d;
			current_interface = NULL;
			set_interface_list();
		}
	}
	load();
}

void Stats::set_interface_list()
{
	gchar *data[2];
	Link *ln;
	Interface *in;
	int row = 0;

	if (current_router == NULL) return;
	if (current_router->interfaces.size()<1) return;

	gtk_clist_freeze(GTK_CLIST(clist6));
	gtk_clist_clear(GTK_CLIST(clist6));
	for (ln=current_router->interfaces.get_head(); ln!=NULL; ln=ln->next)
	{
		in = (Interface*) ln->item;
		data[0] = strdup("");
		data[1] = strdup(in->comment);
		gtk_clist_append(GTK_CLIST(clist6), data);
		delete data[0]; delete data[1];
		gtk_clist_set_row_data(GTK_CLIST(clist6), row, in);
		if (in->status=='1')
			clist_pixtext(window, GTK_CLIST(clist6), row, 0, (gchar **)intup_xpm, in->descr);
		else
			clist_pixtext(window, GTK_CLIST(clist6), row, 0, (gchar **)intdown_xpm, in->descr);
		row++;
	}
	gtk_clist_thaw(GTK_CLIST(clist6));
}

void Stats::start_left()
{
	if (gstart.ts > oldest_time.ts)
	{
		gstart.shift_days(-1);
		day_range++;
	}
	draw();
}

void Stats::start_right()
{
	if ((gstart.ts < gend.ts) && (day_range>1))
	{
		gstart.shift_days(1);
		day_range--;
	}
	draw();
}

void Stats::start_today()
{
	gstart = newest_time;
	gend = newest_time;
	gend.shift_days(1);
	day_range = gend.diff_days(gstart);
	draw();
}

void Stats::start_all()
{
	gstart = oldest_time;
	gend = newest_time;
	gend.shift_days(1);
	day_range = gend.diff_days(gstart);
	draw();
}

void Stats::middle_left()
{
	if (gstart.ts > oldest_time.ts)
	{
		gstart.shift_days(-1);
		gend.shift_days(-1);
	}
	draw();
}

void Stats::middle_right()
{
	if (gend.ts <= newest_time.ts)
	{
		gstart.shift_days(1);
		gend.shift_days(1);
	}
	draw();
}

void Stats::end_left()
{
	if ((gstart.ts < gend.ts) && (day_range>1))
	{
		gend.shift_days(-1);
		day_range--;
	}
	draw();
}

void Stats::end_right()
{
	if (gend.ts <= newest_time.ts)
	{
		gend.shift_days(1);
		day_range++;
	}
	draw();
}

// void Stats::set_check(GtkWidget *w)
// {
// 	int page = GPOINTER_TO_INT(gtk_object_get_data(GTK_OBJECT(w), "value"));
// 	check_states[page] = (GTK_TOGGLE_BUTTON (w)->active)?'y':'n';
// 	draw();
// }

void setStyle(GtkWidget *widget, gpointer data)
{
	GtkStyle *style;

	style = (GtkStyle *) data;
	gtk_widget_set_style(widget, style);
	if (GTK_IS_CONTAINER (widget))
	{
		gtk_container_foreach (GTK_CONTAINER (widget), setStyle, style);
	}
}

void Stats::key_color(GtkWidget *w, gchar *str)
{
	GtkStyle *defstyle;
	GtkStyle *style;
	GdkColor col;
	gint r, g, b;

	sscanf(str, "#%2X%2X%2X", &r, &g, &b);
	r*=256; g*=256; b*=256;
	alloc_color(col, r, g, b);

	defstyle = gtk_widget_get_style(window);
	style = gtk_style_copy(defstyle);
	style->fg[GTK_STATE_NORMAL] = col;
//	style->bg[GTK_STATE_NORMAL] = col;
//	style->light[GTK_STATE_NORMAL] = col;
//	style->dark[GTK_STATE_NORMAL] = col;
//	style->mid[GTK_STATE_NORMAL] = col;
	style->text[GTK_STATE_NORMAL] = col;
//	style->base[GTK_STATE_NORMAL] = col;
	setStyle(w, style);
}

void Stats::load()
{
	char fname[256];

	if ((current_router!=NULL) && (current_interface!=NULL))
	{
		sprintf(fname, "%s/graph/%s/v%03d", main->session->datadir,
			current_router->pingname.getstring(), current_interface->index);
		gio.record_size = GRAPH_PACKET_RECORD_SIZE;
		gio.num_values = GRAPH_NUM_PACKETS;
		packets->data->empty();
//printf("LOADING PACKETS FOR A GRAPH\n");
		packets_ok = gio.get(fname, packets->data);
//		packets->data->print();
//printf("DONE (LOADING PACKETS FOR A GRAPH)\n");
		sprintf(fname, "%s/graph/%s/e%03d", main->session->datadir,
			current_router->pingname.getstring(), current_interface->index);
		gio.record_size = GRAPH_ERROR_RECORD_SIZE;
		gio.num_values = GRAPH_NUM_ERRORS;
		errors->data->empty();
//printf("LOADING ERRORS FOR A GRAPH\n");
		errors_ok = gio.get(fname, errors->data);
//printf("DONE (LOADING ERRORS FOR A GRAPH)\n");
	}

	draw();
}

void Stats::draw()
{
	if (packets_ok)
		draw_graph(drawingarea1, packets);
	else
		draw_failed(drawingarea1, packets);

	if (errors_ok)
		draw_graph(drawingarea2, errors);
	else
		draw_failed(drawingarea2, errors);
}

void Stats::draw_init_canvas(GtkWidget *drawingarea, Graph *g)
{
	gint ow, oh;

	if (drawingarea->window == NULL) return;
	if (g == NULL) return;

	oh = g->height;
	ow = g->width;

	g->height = drawingarea->allocation.height;
	g->width = drawingarea->allocation.width;

	if ((g->canvas == NULL) || (oh != g->height) || (ow != g->width))
	{
		if (g->canvas) gdk_pixmap_unref(g->canvas);
		g->canvas = gdk_pixmap_new(drawingarea->window, g->width, g->height, -1);
	}

	gdk_gc_set_foreground(gc, &lightgray);
	gdk_draw_rectangle(g->canvas, gc, TRUE, 0, 0, -1, -1);
}

void Stats::draw_failed(GtkWidget *drawingarea, Graph *g)
{
	GdkFont *fnt;
	gint txtw;
	gchar buf[64];

	draw_init_canvas(drawingarea, g);

	fnt = gdk_font_load("-*-helvetica-medium-r-*-*-10-*-*-*-*-*-*-*");
	if ((current_router!=NULL) && (current_interface!=NULL))
		sprintf(buf, "Failed to load data for graph");
	else
		sprintf(buf, "Summary here");
	txtw = gdk_string_width(fnt, buf);
	gdk_gc_set_foreground(gc, &black);
	gdk_draw_string(g->canvas, fnt, gc, (g->width-txtw)/2, (g->height/2), buf);
}

// do graph drawing in this class but pass a pointer to graph object here
void Stats::draw_graph(GtkWidget *drawingarea, Graph *g)
{
	GtkStyle *style;
	gint ow, oh;
	int i, j, k, type, num_speeds, num_times, txtw, day_len, first_day;
	int font_height, step, bound;
	char buf[256], buf2[256], buf3[256];
	GdkFont *fnt;
	gint8 dash_list[2];
	TimeStamp gtmp;
	double s0, s1, s2, t1, t2;

//	if (draw_lock==TRUE) return;
//	draw_lock = TRUE;

	if (g->data->num_entries == 0) return;
	if (g->data->num_variables == 0) return;

//	g->data->print();

	draw_init_canvas(drawingarea, g);



	fnt = gdk_font_load("-*-helvetica-medium-r-*-*-10-*-*-*-*-*-*-*");
//	font_height = fnt->ascent + fnt->descent;
	font_height = fnt->ascent;

	// work out the time details for the sample data
	oldest_time = g->data->timestamps[0];
	newest_time = g->data->timestamps[g->data->num_entries-1];
	oldest_time.midnight();
	newest_time.midnight();
	length = newest_time.diff_days(oldest_time);
	if (day_range == 0)
	{
		gstart = newest_time;
		gend = newest_time;
		gend.shift_days(1);
	}
	day_range = gend.diff_days(gstart);

	gtmp = gend; gtmp.shift_days(-1);
	first_day = gstart.get_day();
	strcpy(buf2, gstart.date_to_str(2));
	strcpy(buf3, gtmp.date_to_str(2));

//	scrollbar_adj->lower = 0;
//	scrollbar_adj->upper = length;
//	scrollbar_adj->value = 0;
//	scrollbar_adj->step_increment = 1;
//	scrollbar_adj->page_increment = 1;
//	scrollbar_adj->page_size = day_range;
//	gtk_signal_emit_by_name (GTK_OBJECT(scrollbar_adj), "changed");
	// Note: the change in the scrollbar settings here causes a redraw
	// jumping out from the middle of this draw routinue, this should only
	// effect the BX, BY, BW, BH values storing the sizes for the canvas
	// (which is why they have been moved to after this statement) which will
	// have changed when return back here

	// gray out buttons out of range
	gtk_widget_set_sensitive(button1, FALSE);
	gtk_widget_set_sensitive(button2, FALSE);
	gtk_widget_set_sensitive(button3, FALSE);
	gtk_widget_set_sensitive(button4, FALSE);
	gtk_widget_set_sensitive(button5, FALSE);
	gtk_widget_set_sensitive(button6, FALSE);
	gtk_widget_set_sensitive(button15, FALSE);
	gtk_widget_set_sensitive(button16, FALSE);

	// show valid buttons
	gtk_widget_set_sensitive(button15, TRUE);
	gtk_widget_set_sensitive(button16, TRUE);
	if (gstart.ts > oldest_time.ts)
	{
		gtk_widget_set_sensitive(button1, TRUE);
		gtk_widget_set_sensitive(button3, TRUE);
	}
	if ((gstart.ts < gend.ts) && (day_range>1))
	{
		gtk_widget_set_sensitive(button2, TRUE);
		gtk_widget_set_sensitive(button5, TRUE);
	}
	if (gend.ts <= newest_time.ts)
	{
		gtk_widget_set_sensitive(button6, TRUE);
		gtk_widget_set_sensitive(button4, TRUE);
	}

	// set window title with full dates
	if (day_range>1)
		sprintf(buf, "Network Graph - %s -> %s", buf2, buf3);
	else
		sprintf(buf, "Network Graph - %s", buf2);
	gtk_window_set_title (GTK_WINDOW (window), buf);

// clear window
	gdk_gc_set_foreground(gc, &black);
	style = gtk_widget_get_style(window);
	gtk_draw_flat_box(style, g->canvas, GTK_STATE_NORMAL,
		GTK_SHADOW_NONE, 0, 0, g->width, g->height);

	if ((current_router!=NULL) && (current_interface!=NULL))
	{
		BX = LEFT_MGN;
		BY = TOP_MGN;
		BW = g->width-LEFT_MGN-RIGHT_MGN;
		BH = g->height-TOP_MGN-BOTTOM_MGN;

		// draw times
		num_times = ((BW/day_range)/14)-1;		// 18 is room for text + a gap

		// 1,2,3,4,6,8,12,24
		if (num_times<=0) num_times=1; else
		if (num_times==5) num_times=4; else
		if (num_times==7) num_times=6; else
		if ((num_times>=9) && (num_times<=11)) num_times=8; else
		if ((num_times>=13) && (num_times<=23)) num_times=12; else
		if (num_times>=25) num_times=24;

		for (i=0; i<(num_times*day_range)+1; i++)
		{
			sprintf(buf, "%d", ((24/num_times)*i)%24);
			txtw = gdk_string_width(fnt, buf);
			gdk_draw_string(g->canvas, fnt, gc,
				BX-(txtw/2)+(BW*i/(num_times*day_range)), g->height+12-BOTTOM_MGN, buf);
			gdk_draw_line(g->canvas, gc, BX+(BW*i/(num_times*day_range)),
				g->height-BOTTOM_MGN, BX+(BW*i/(num_times*day_range)), g->height-BOTTOM_MGN+2);
		}

		// work out which format to use from the string lengths
		gtmp.setstring("20000524000000");
		type = 6;
		day_len = (BW/day_range);
		for (i=5; i>1; i--)
		{
			strcpy(buf, gtmp.date_to_str(i));
			if (gdk_string_width(fnt, buf)+4<day_len) type=i;
		}

		// print the days
		gdk_gc_set_foreground(gc, &black);
		for (i=0; i<day_range; i++)
		{
			gtmp = gstart;
			gtmp.shift_days(i);
			strcpy(buf, gtmp.date_to_str(type));
			txtw = gdk_string_width(fnt, buf);
			gdk_draw_string(g->canvas, fnt, gc,
				BX+((day_len-txtw)/2)+(BW*i/day_range), g->height+24-BOTTOM_MGN, buf);
		}

		gdk_gc_set_foreground(gridgc, &lightgray);
		for (i=0; i<(num_times*day_range); i++)
		{
			if (i%num_times==0)
			{
				j = (first_day+1+(i/num_times));
				while (j<0) j+=777;
				if ((j%7==0) || (j%7==1))
				{
					gdk_draw_rectangle(g->canvas, gridgc, TRUE,
						BX+(BW*i/(num_times*day_range)), BY, (BW/day_range)+1, BH);
//					gdk_draw_rectangle(g->canvas, gridgc, TRUE,
//						SX+(SW*i/(num_times*day_range)), SY, (SW/day_range)+1, SH);
				}
			}
		}

		// draw grid + speeds
		gdk_gc_set_foreground(gc, &black);
		gdk_gc_set_foreground(gridgc, &gray);
		gdk_gc_set_line_attributes (gridgc, 1, GDK_LINE_ON_OFF_DASH,
			GDK_CAP_NOT_LAST, GDK_JOIN_MITER);
		dash_list[0] = 2; dash_list[1] = 2;
		gdk_gc_set_dashes(gridgc, 0, dash_list, sizeof(dash_list));
//printf("BH=%d onfly=%d\n", BH, g->height-TOP_MGN-BOTTOM_MGN);
		for (i=1; i<(num_times*day_range); i++)
		{
			gdk_draw_line(g->canvas, gridgc, BX+(BW*i/(num_times*day_range)), BY+BH,
				BX+(BW*i/(num_times*day_range)), BY);
//		gdk_draw_line(g->canvas, gridgc, SX+(SW*i/(num_times*day_range)), SY+SH,
//			SX+(SW*i/(num_times*day_range)), SY);
		}
		if (g->scale==0)
		{
			num_speeds = (BH/(font_height+2))-1;
			// 1,2,4,5,10,20,25
			if (num_speeds==3) num_speeds=2; else
			if ((num_speeds>=6) && (num_speeds<=9)) num_speeds=5; else
			if ((num_speeds>=11) && (num_speeds<=19)) num_speeds=10; else
			if ((num_speeds>=21) && (num_speeds<=24)) num_speeds=20; else
			if (num_speeds>=26) num_speeds=25;
			for (i=num_speeds; i>=0; i--)
			{
				speed_to_str((((current_interface->speed)/num_speeds)*i), buf);
				txtw = gdk_string_width(fnt, buf);
				gdk_draw_string(g->canvas, fnt, gc,
					BX-2-txtw, BY+BH-(BH*i/num_speeds)+5, buf);
				gdk_draw_line(g->canvas, gc, BX-2, BY+BH-(BH*i/num_speeds),
					BX, BY+BH-(BH*i/num_speeds));
				gdk_draw_line(g->canvas, gridgc, BX, BY+BH-(BH*i/num_speeds), BX+BW,
					BY+BH-(BH*i/num_speeds));
			}
		}else
		{
			// We split the scale into five and workout a linear progression
			// based on gap at top of section after log(10) the value, then we
			// write scale using log(10)

			s0 = (current_interface->speed) / 100;
			for (k=1; k<=5; k++)
			{
				t1 = ((double)current_interface->speed);
				s1 = (log(t1+s0)-log(s0));

				bound=k*20; num_speeds=(k-1)*20; step=20;
				for (i=0; i<20; i++)
				{
					t2 = (((double)current_interface->speed)/((i+2)*k))*(((i+2)*k)-1);
					s2 = (log(t2+s0)-log(s0));
					j = gint(((double)BH*(s1-s2))/s1);
					if (j > font_height)
					{
						if (i<=1) step=10; else
						if (i==2) step=5; else
						if ((i>=3) && (i<=7)) step=4; else
						if ((i>=8) && (i<=17)) step=2; else
						if (i>=18) step=1;
					}
				}
				i=num_speeds;
				while(i<bound)
				{
					i+=step;
					speed_to_str((((current_interface->speed)/100)*i), buf);
					txtw = gdk_string_width(fnt, buf);
					t2 = ((double)current_interface->speed)/100*i;
					s2 = (log(t2+s0)-log(s0));
					j = gint(((double)BH*s2)/s1);
					gdk_draw_string(g->canvas, fnt, gc, BX-2-txtw, BY+BH-j+5, buf);
					gdk_draw_line(g->canvas, gc, BX-2, BY+BH-j, BX, BY+BH-j);
					gdk_draw_line(g->canvas, gridgc, BX, BY+BH-j, BX+BW, BY+BH-j);
				}
			}
			speed_to_str(0, buf);
			txtw = gdk_string_width(fnt, buf);
			gdk_draw_string(g->canvas, fnt, gc, BX-2-txtw, BY+BH+5, buf);
			gdk_draw_line(g->canvas, gc, BX-2, BY+BH, BX, BY+BH);
		}

		gdk_gc_set_foreground(gridgc, &darkgray);
		dash_list[0] = 4; dash_list[1] = 4;
		gdk_gc_set_dashes(gridgc, 0, dash_list, sizeof(dash_list));
		for (i=0; i<(num_times*day_range); i++)
		{
			if (i%num_times==0)
			{
				gdk_draw_line(g->canvas, gridgc, BX+(BW*i/(num_times*day_range)),
					BY+BH, BX+(BW*i/(num_times*day_range)), BY);
//				gdk_draw_line(g->canvas, gridgc, SX+(SW*i/(num_times*day_range)),
//					SY+SH, SX+(SW*i/(num_times*day_range)), SY);
			}
		}

		// draw lines
		for (i=0; i<g->data->num_variables; i++)
		{
//			if (check_states[i]=='y')
//				draw_line(g, i);
			if (i==1) //octets in
			{
				printf("Octets In Line\n");
				unsigned long val1;
				TimeStamp tx1;
				for (j=0; j<g->data->num_entries; j++)
				{
					val1 = g->data->variables[i].values[j];
					tx1 = g->data->timestamps[j];
					if (tx1.ts != 0)
						printf("ts1=%s val1=%lu\n", tx1.date_to_str(1), val1);
				}
			}
		}

		// draw axis
		gdk_gc_set_foreground(gc, &black);
		gdk_draw_rectangle(g->canvas, gc, FALSE, BX, BY, BW, BH);
	}

	gdk_draw_pixmap(drawingarea->window,
		drawingarea->style->fg_gc[GTK_WIDGET_STATE(drawingarea)],
		g->canvas, 0, 0, 0, 0, drawingarea->allocation.width,
		drawingarea->allocation.height);
//	draw_lock = FALSE;
}

void Stats::draw_line(Graph *g, int var)
{
	TimeStamp tx1, tx2;
	gint i, x1, y1, x2, y2;
	int x, y, w, h;
	GdkRectangle rect;
	GdkColor col;
	double s0, s1, s2, s3, t1, t2, t3;
	unsigned long val1, val2;

	if (current_interface->speed==0) return;

	x=BX; y=BY; w=BW; h=BH;

// FIXME drawing graphs broken until new format fixed
	// set line colour
//	alloc_color(col, gio.global_types[var].r,
//		gio.global_types[var].g, gio.global_types[var].b);
	alloc_color(col, 0xffff, 0, 0);
	gdk_gc_set_foreground(gc, &col);

	s0 = (current_interface->speed) / 100;
	t1 = ((double)current_interface->speed);
	s1 = (log(t1+s0)-log(s0));
// if (var==1) printf("GRAPH - num_entries=%d\n", g->data->num_entries);
	for (i=1; i<g->data->num_entries; i++)
	{
		// work out coordinates in the graph
		val1 = g->data->variables[var].values[i-1];
		val2 = g->data->variables[var].values[i];
		tx1 = g->data->timestamps[i-1];
		tx2 = g->data->timestamps[i];

//		if (var==1) printf("GRAPH - Timestamp=%s value=%lu\n", tx1.date_to_str(1), val1);

		if ((tx1.ts>gstart.ts) && (tx1.ts<gend.ts))
		{
//			printf(" IN RANGE");
			x1 = ((tx1.ts-gstart.ts)*w/(gend.ts-gstart.ts))+x;
			x2 = ((tx2.ts-gstart.ts)*w/(gend.ts-gstart.ts))+x;
//			if (gio.global_types[var].rate == 1)
//			{
/*
				if (g->scale == 0)
				{
					y1 = gint(y+h-((((double)val1)/current_interface->speed)*h*8));
					y2 = gint(y+h-((((double)val2)/current_interface->speed)*h*8));
				}else
				{
					t2 = (((double)val1)*8);
					t3 = (((double)val2)*8);
					s2 = (log(t2+s0)-log(s0));
					s3 = (log(t3+s0)-log(s0));
					y1 = gint(y+h-(((double)h/s1)*s2));
					y2 = gint(y+h-(((double)h/s1)*s3));
				}
				rect.x = x; rect.y = y;
				rect.width = w; rect.height = h;
				gdk_gc_set_clip_rectangle(gc, &rect);
				gdk_draw_line(g->canvas, gc, x1, y1, x2, y2);
				gdk_gc_set_clip_rectangle(gc, NULL);
*/
/*
			}else
			if (gio.global_types[var].rate == 2)
			{
				// draw in top box
				y1 = ((val1*SH)/100);
				y2 = ((val2*SH)/100);
				rect.x = SX; rect.y = SY;
				rect.width = SW; rect.height = SH;
				gdk_gc_set_clip_rectangle(gc, &rect);
				gdk_draw_line(canvas, gc, x1, SY+SH-y1, x2, SY+SH-y2);
				gdk_gc_set_clip_rectangle(gc, NULL);
			}
*/
		}
//		printf("\n");
	}
}

