/* topology.h
 * Copyright (C) 2001 The University of Newcastle upon Tyne
 *
 * Written by Paul Coates (Paul.Coates@ncl.ac.uk)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 */

#ifndef TOPOLOGY_H
#define TOPOLOGY_H 1

#include "header.h"
#include "ipaddress.h"
#include "list.h"

#define BUFSIZE 256

class Main;
class Subnet;
class Device;
class Network;
class Parser;
class Interface;
class MACAddress;
class Machine;
class Topology
{
	public:
		Topology(Main *m);
		~Topology();

		void create_placeholder(char *nmstr, IPAddress tip);
		void deviceAdd(char *fname, int hop);
		void deviceDeleteSelected();
		void deviceDelete(Device *d);
		void deviceMerge();
		void deviceSplit(Device *d);

		NetworkType default_network_type;

//		void buildRestForLoad();

	private:
		void setPreferences();
		void copyDeviceHeader();
		void clearDeviceTags();
		void checkForDuplicateDevice();
		void checkForDuplicateDeviceWithIP(IPAddress *ip);
		void mergeType();
		void attachInterfaces();
		void addIPToIPAddresses(Interface *in, IPAddress ip);
		void addIPToNextHop(Interface *in, IPAddress ip);
		void addRef();
		Interface *lookupInterface(Device *d, Interface *in);
		void addNextHop();
		void add_subnet(IPAddress ip, int mask);
		void layer3(int hop);
		int checkInterfacesOnSameSubnet(Interface *in1, Interface *in2);
		gboolean checkIPAndIntOnSameSubnet(IPAddress *ip1, Interface *in);
		Subnet *find_subnet(IPAddress *ip);

		void merge_networks(Network *n1, Network *n2);
		void layer2(Device *dd);
		void layer2hack();
		void layer2go();
		void connectLayer2(Interface *in);
		void severNetwork(Network *nw);
		void recLayer2(Interface *in);
		void buildSwitchList(Interface *in2);
		Interface *isMACOnInterface(MACAddress mac);
		Machine *isMACAMachine(MACAddress mac);
		void copyOverMachines();
		gboolean newLocationBest(Interface *oldin, Interface *newin);
		int pickMachine(Machine *m1, Machine *m2);

		void deleteTaggedDevices();
		void deleteSingleDevice(Device *d);

		Main *main;
		Parser *parser;
		Device *tmpdev;
		Device *origdev;
		int merge_type;
		List switch_list;
};

class Join
{
	public:
		Interface *in1;
		Interface *in2;
		int tag;
};

#endif

