/* data.cc
 * Copyright (C) 2001 The University of Newcastle upon Tyne
 *
 * Written by Paul Coates (Paul.Coates@ncl.ac.uk)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 */

#include <gnome-xml/tree.h>
#include <gnome-xml/parser.h>
#include <gtk/gtk.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include "data.h"
#include "header.h"
#include "interface.h"
#include "network.h"
#include "machine.h"
#include "subnet.h"
#include "device.h"

Data::Data()
{
	width = 0;
	height = 0;
	minx = miny = 0;
	show_leaves = '1';
	subnets.empty();
	networks.empty();
	machines.empty();
	devices.empty();
	strcpy(startup_view, "");
}

Data::~Data()
{
}

void Data::savexml(char *fname)
{
	xmlDocPtr doc;
	xmlNodePtr cur, cur2;
	Subnet *s;
	Device *d;
	Network *n;
	Interface *in, *in2;
	Link *ln, *ln2, *ln3;
	char buf[256];

	doc = xmlNewDoc((xmlChar*)"1.0");
	doc->root = xmlNewDocNode(doc, NULL, (xmlChar*)"Nomad_Topology", NULL);

	if (strcmp(startup_view, "")!=0)
		xmlSetProp(doc->root, (xmlChar*)"startup_view", (xmlChar*)startup_view);

	for (ln=subnets.get_head(); ln!=NULL; ln=ln->next)
	{
		s = (Subnet*) ln->item;
		s->savexml(doc->root);
	}
	for (ln=devices.get_head(); ln!=NULL; ln=ln->next)
	{
		d = (Device*) ln->item;
		d->savexml(doc->root);
	}

	// save only those networks that are attached to stuff
	for (ln=networks.get_head(); ln!=NULL; ln=ln->next)
	{
		n = (Network*) ln->item;
		if (n->interfaces.size() > 1)
			n->savexml(doc->root);
	}

	xmlSaveFile(fname, doc);
	xmlFreeDoc(doc);
}

void Data::loadxml(char *fname)
{
	xmlChar *buf;
	xmlDocPtr doc;
	xmlNodePtr xmlnode;
	Device *d;
	Subnet *s;
	Network *n;
	doc = xmlParseFile(fname);

	if (!doc || !doc->root || !doc->root->name ||
			strcmp((char*)doc->root->name, "Nomad_Topology")!=0)
	{
		xmlFreeDoc(doc);
		return;
	}

	buf = (xmlChar*)xmlGetProp(doc->root, (xmlChar*)"startup_view");
	if (buf!=NULL) { strcpy(startup_view, (char*)buf); free(buf); }

	xmlnode = doc->root->childs;
	while (xmlnode != NULL)
	{
		if (xmlnode->name!=NULL)
		{
			if (strcmp((char*)xmlnode->name, "Subnet")==0)
			{
				s = new Subnet();
				s->loadxml(xmlnode);
				subnets.tag(s);
				while(gtk_events_pending()) gtk_main_iteration();
			}else
			if (strcmp((char*)xmlnode->name, "Device")==0)
			{
				d = new Device();
				d->loadxml(xmlnode, &networks, &machines);
				devices.tag(d);
				while(gtk_events_pending()) gtk_main_iteration();
			}else
			if (strcmp((char*)xmlnode->name, "Network")==0)
			{
				n = new Network();
				n->loadxml(xmlnode, &devices, &networks);
//				printf("LOAD network %d subnets\n", n->subnets.size());
				networks.tag(n);
				while(gtk_events_pending()) gtk_main_iteration();
			}
		}
		xmlnode = xmlnode->next;
	}

	xmlFreeDoc(doc);
}

void Data::print()
{
	Link *ln, *ln2, *ln3;
	Device *d;
	Interface *in;
	IPAddress *ip;

	for (ln=devices.get_head(); ln!=NULL; ln=ln->next)
	{
		d = (Device*) ln->item;
		printf("DEVICE %s\n", d->name);
		for (ln2=d->interfaces.get_head(); ln2!=NULL; ln2=ln2->next)
		{
			in = (Interface*) ln2->item;
			printf("  Interface %d %s\n", in->index, in->descr);
			for (ln3=in->ip_addresses.get_head(); ln3!=NULL; ln3=ln3->next)
			{
				ip = (IPAddress*) ln3->item;
				printf("    IP Address %s\n", ip->getstring());
			}
			for (ln3=in->nexthops.get_head(); ln3!=NULL; ln3=ln3->next)
			{
				ip = (IPAddress*) ln3->item;
				printf("    Next Hop   %s\n", ip->getstring());
			}
		}
	}
}

// convert a machine into a device reference so we don't include
// device interface addresses as machines
void Data::machineToDev(Machine *m)
{
/*
	Link *ln;
	


	machines.del(m);
	if (m->network) m->network->machines.del(m);
	for (ln = machines.get_head(); ln!=NULL; ln=ln->next)
	{
		mach = (Machine*) ln->item;
		if (m == mach) machines.del(n2);
		m->selection = FALSE;
		if (m->shown==TRUE) m->selection = val;
	}
*/	
}

void Data::deleteMachinesMatchingIP(IPAddress *ip)
{
/*
	Link *ln;
	Machine *m;

	for (ln = machines.get_head(); ln!=NULL; ln=ln->next)
	{
		m = (Machine*) ln->item;
		if (m->ip.compare(*ip)==0)
		if (m->interface) m->interface->machines.del(m);
		ln->item=NULL;
		delete m;
	}
	machines.tidy();
*/
}

// search all devices looking for an ip address
Device *Data::find_ip_on_device(IPAddress tip)
{
	IPAddress *ip;
	Link *ln, *ln2, *ln3;
	Device *d;
	Interface *in;

	for (ln=devices.get_head(); ln!=NULL; ln=ln->next)
	{
		d = (Device*) ln->item;
		if (d->pingname.compare(tip)==0) return d;
		for (ln2=d->interfaces.get_head(); ln2!=NULL; ln2=ln2->next)
		{
			in = (Interface*) ln2->item;
			for (ln3=in->ip_addresses.get_head(); ln3!=NULL; ln3=ln3->next)
			{
				ip = (IPAddress*) ln3->item;
				if (ip->compare(tip)==0) return d;
			}
		}
	}
	return NULL;
}

void Data::select(gboolean val)
{
	Link *ln;
	Device *r;
	Network *n;
	Machine *m;

	for (ln = devices.get_head(); ln!=NULL; ln=ln->next)
	{
		r = (Device*) ln->item;
		r->selection = FALSE;
		if (r->shown==TRUE)
		{
			r->selection = val;
			// DUAL select(char val)
		}
	}

	for (ln = machines.get_head(); ln!=NULL; ln=ln->next)
	{
		m = (Machine*) ln->item;
		m->selection = FALSE;
		if (m->shown==TRUE) m->selection = val;
	}

	for (ln = networks.get_head(); ln!=NULL; ln=ln->next)
	{
		n = (Network*) ln->item;
		n->selection = FALSE;
		if (n->shown==TRUE) n->selection = val;
	}
}

/*
void Data::hide_leaf_networks()
{
	Link *ln;
	Network *n;

	for (ln=networks.get_head(); ln!=NULL; ln=ln->next)
	{
		n = (Network*) ln->item;
		if (n->interfaces.size() < 2)
			n->shown = FALSE;
	}
}
*/

void Data::size()
{
	Link *ln;
	Device *d;
	Network *n;
	Machine *m;
	long maxx, maxy;
	int match;

	if (devices.size()==0) return;

	ln=devices.get_head();
	d = (Device*) ln->item;
	minx = miny = 100000;
	maxx = maxy = -100000;
	match = 0;

	for (ln=devices.get_head(); ln!=NULL; ln=ln->next)
	{
		d = (Device*) ln->item;
		if (minx > d->x-(d->w/2)) minx = d->x-(d->w/2);
		if (maxx < d->x+(d->w/2)) maxx = d->x+(d->w/2);
		if (miny > d->y-(d->h/2)) miny = d->y-(d->h/2);
		if (maxy < d->y+(d->h/2)) maxy = d->y+(d->h/2);
		if (d->shown==TRUE) match = 1;
	}

	for (ln=machines.get_head(); ln!=NULL; ln=ln->next)
	{
		m = (Machine*) ln->item;
		if (minx > m->x-(m->w/2)) minx = m->x-(m->w/2);
		if (maxx < m->x+(m->w/2)) maxx = m->x+(m->w/2);
		if (miny > m->y-(m->h/2)) miny = m->y-(m->h/2);
		if (maxy < m->y+(m->h/2)) maxy = m->y+(m->h/2);
		if (m->shown==TRUE) match = 1;
	}

	for (ln=networks.get_head(); ln!=NULL; ln=ln->next)
	{
		n = (Network*) ln->item;
		if (minx > n->x-(n->w/2)) minx = n->x-(n->w/2);
		if (maxx < n->x+(n->w/2)) maxx = n->x+(n->w/2);
		if (miny > n->y-(n->h/2)) miny = n->y-(n->h/2);
		if (maxy < n->y+(n->h/2)) maxy = n->y+(n->h/2);
		if (n->shown==TRUE) match = 1;
	}

	if (match==0)
	{
		minx = miny = 0;
		return;
	}

	for (ln=devices.get_head(); ln!=NULL; ln=ln->next)
	{
		d = (Device*) ln->item;
		d->x-=minx; d->y-=miny;
	}
	for (ln=machines.get_head(); ln!=NULL; ln=ln->next)
	{
		m = (Machine*) ln->item;
		m->x-=minx; m->y-=miny;
	}
	for (ln=networks.get_head(); ln!=NULL; ln=ln->next)
	{
		n = (Network*) ln->item;
		n->x-=minx; n->y-=miny;
	}

	if (match==1)
	{
		width = (maxx-minx);
		height = (maxy-miny);
	}
}

void Data::sizeshow()
{
	Link *ln;
	Device *d;
	Network *n;
	Machine *m;
	long maxx, maxy;
	int match;

	if (devices.size()==0) return;

	ln=devices.get_head();
	d = (Device*) ln->item;
	minx = miny = 100000;
	maxx = maxy = -100000;
	match = 0;

	for (ln=devices.get_head(); ln!=NULL; ln=ln->next)
	{
		d = (Device*) ln->item;
		if (d->shown==TRUE)
		{
			if (minx > d->x-(d->w/2)) minx = d->x-(d->w/2);
			if (maxx < d->x+(d->w/2)) maxx = d->x+(d->w/2);
			if (miny > d->y-(d->h/2)) miny = d->y-(d->h/2);
			if (maxy < d->y+(d->h/2)) maxy = d->y+(d->h/2);
			match = 1;
		}
	}

	for (ln=machines.get_head(); ln!=NULL; ln=ln->next)
	{
		m = (Machine*) ln->item;
		if ((m->shown==TRUE) && (m->interface->network->shown==TRUE))
		{
			if (minx > m->x-(m->w/2)) minx = m->x-(m->w/2);
			if (maxx < m->x+(m->w/2)) maxx = m->x+(m->w/2);
			if (miny > m->y-(m->h/2)) miny = m->y-(m->h/2);
			if (maxy < m->y+(m->h/2)) maxy = m->y+(m->h/2);
			match = 1;
		}
	}

	for (ln=networks.get_head(); ln!=NULL; ln=ln->next)
	{
		n = (Network*) ln->item;
		if (n->shown==TRUE)
		{
			if (minx > n->x-(n->w/2)) minx = n->x-(n->w/2);
			if (maxx < n->x+(n->w/2)) maxx = n->x+(n->w/2);
			if (miny > n->y-(n->h/2)) miny = n->y-(n->h/2);
			if (maxy < n->y+(n->h/2)) maxy = n->y+(n->h/2);
			match = 1;
		}
	}

	if (match==0)
	{
		minx = miny = 0;
		return;
	}

	for (ln=devices.get_head(); ln!=NULL; ln=ln->next)
	{
		d = (Device*) ln->item;
		d->x-=minx; d->y-=miny;
	}
	for (ln=machines.get_head(); ln!=NULL; ln=ln->next)
	{
		d = (Device*) ln->item;
		d->x-=minx; d->y-=miny;
	}
	for (ln=networks.get_head(); ln!=NULL; ln=ln->next)
	{
		n = (Network*) ln->item;
		n->x-=minx; n->y-=miny;
	}

	if (match==1)
	{
		width = (maxx-minx);
		height = (maxy-miny);
	}
}

int Data::selected_size()
{
	Link *ln;
	Device *d;
	Network *n;
	Machine *m;
	long mnx, mny, mxx, mxy;
	int cnt = 0;

	if (devices.size()==0) return 0;

	ln=devices.get_head();
	d = (Device*) ln->item;
	mnx = mny = 100000;
	mxx = mxy = -100000;

	for (ln=devices.get_head(); ln!=NULL; ln=ln->next)
	{
		d = (Device*) ln->item;
		if ((d->shown==TRUE) && (d->selection==TRUE))
		{
			if (mnx > d->x-(d->w/2)) mnx = d->x-(d->w/2);
			if (mxx < d->x+(d->w/2)) mxx = d->x+(d->w/2);
			if (mny > d->y-(d->h/2)) mny = d->y-(d->h/2);
			if (mxy < d->y+(d->h/2)) mxy = d->y+(d->h/2);
			cnt = 1;
		}
	}

	for (ln=machines.get_head(); ln!=NULL; ln=ln->next)
	{
		m = (Machine*) ln->item;
		if ((m->shown==TRUE) && (m->selection==TRUE))
		{
			if (mnx > m->x-(m->w/2)) mnx = m->x-(m->w/2);
			if (mxx < m->x+(m->w/2)) mxx = m->x+(m->w/2);
			if (mny > m->y-(m->h/2)) mny = m->y-(m->h/2);
			if (mxy < m->y+(m->h/2)) mxy = m->y+(m->h/2);
			cnt = 1;
		}
	}

	for (ln=networks.get_head(); ln!=NULL; ln=ln->next)
	{
		n = (Network*) ln->item;
		if ((n->shown==TRUE) && (n->selection==TRUE))
		{
			if (mnx > n->x-(n->w/2)) mnx = n->x-(n->w/2);
			if (mxx < n->x+(n->w/2)) mxx = n->x+(n->w/2);
			if (mny > n->y-(n->h/2)) mny = n->y-(n->h/2);
			if (mxy < n->y+(n->h/2)) mxy = n->y+(n->h/2);
			cnt = 1;
		}
	}

	width = (mxx-mnx);
	height = (mxy-mny);
	minx = mnx;
	miny = mny;

	return cnt;
}

void Data::set_leaves()
{
	Link *ln;
	Network *n;

	for (ln=networks.get_head(); ln!=NULL; ln=ln->next)
	{
		n = (Network*) ln->item;
		if ((n->interfaces.size() < 2) && (show_leaves=='0'))
			n->shown = FALSE;
		else
			n->shown = TRUE;
	}
}

void Data::show()
{
	Device *r;
	Network *n;
	Machine *m;
	Link *ln;

	for (ln=devices.get_head(); ln!=NULL; ln=ln->next)
	{
		r = (Device*) ln->item;
		if (r->selection==TRUE) r->shown = TRUE;
	}
}

void Data::hide()
{
	Device *r;
	Network *n;
	Machine *m;
	Link *ln;

	for (ln=devices.get_head(); ln!=NULL; ln=ln->next)
	{
		r = (Device*) ln->item;
		if (r->selection==TRUE) r->shown = FALSE;
	}
	for (ln=machines.get_head(); ln!=NULL; ln=ln->next)
	{
		m = (Machine*) ln->item;
		if (m->selection==TRUE) m->shown = FALSE;
	}
	for (ln=networks.get_head(); ln!=NULL; ln=ln->next)
	{
		n = (Network*) ln->item;
		if (n->selection==TRUE) n->shown = FALSE;
	}
}

void Data::buncha(Device *r, int sel)
{
	Link *ln, *ln2;
	double radius, angle;
	int l, total, ring, machine_count;
	Interface *in;
	Machine *m;

	if (r->shown==FALSE) return;
	if (sel) r->selection = TRUE;

	l = 0;
	for (ln2 = r->interfaces.get_head(); ln2!=NULL; ln2 = ln2->next)
	{
		in = (Interface*) ln2->item;
		if (in->network != NULL)
		{
			total = ring = 0;
			while (total<l+1) { ring++; total += (8*ring); }

			radius = (((80*ring)+40) * 8)/(2*3.14159);
			angle = 2*3.14159/(ring*8);

			if ((in->network->shown==TRUE) && (is_network_a_leaf(in->network)))
			{
				in->network->x = r->x + int(radius * cos(angle*(total-l)));
				in->network->y = r->y + int(radius * sin(angle*(total-l)));
				if (sel) in->network->selection = TRUE;
				l++;
			}
		}
	}
}

void Data::bunch(Visual *cur, int sel)
{
	Link *ln;
	Device *r;

	if ((cur!=NULL) && (cur->maintype==visual_device) && (cur->selection==FALSE))
	{
		select(FALSE);
		buncha((Device*)cur, sel);
	}
	for (ln=devices.get_head(); ln!=NULL; ln=ln->next)
	{
		r = (Device*) ln->item;
		if (r->selection==TRUE) buncha(r, sel);
	}
}

void Data::group_all(DeviceGroupType ch)
{
	Link *ln;
	Device *r;

	for (ln=devices.get_head(); ln!=NULL; ln=ln->next)
	{
		r = (Device*) ln->item;
		r->group = ch;
	}
}

int Data::is_network_a_leaf(Network *n)
{
	int machine_count;
	Machine *m;
	Interface *in;
	Link *ln, *ln2;

	machine_count=0;
	for (ln = n->interfaces.get_head(); ln!=NULL; ln = ln->next)
	{
		in = (Interface*) ln->item;
		for (ln2 = in->machines.get_head(); ln2!=NULL; ln2 = ln2->next)
		{
			m = (Machine*) ln2->item;
			if (m->shown==TRUE) machine_count++;
		}
	}

	return ((n->interfaces.size() == 1) && (machine_count == 0));
}

void Data::network_type_auto_all()
{
	Link *ln;
	Network *n;

	for (ln=networks.get_head(); ln!=NULL; ln=ln->next)
	{
		n = (Network*) ln->item;
		n->subtype = network_auto;
	}
}

void Data::stara(Device *r, int sel)
{
	Link *ln, *ln2;
	double radius, angle;
	int l, machine_count;
	Interface *in;
	Machine *m;

	if (r->shown==FALSE) return;
	if (sel)
	{
		r->selection = TRUE;
		// DUAL stara(Device *r, int sel)
//		if (selectcb != NULL) selectcb(selectparent, r);
//printf("DUAL %lu\n", selectcb);
//printf("DUAL %lu %lu\n", selectcb, selectparent);
	}

	l = 0;
	for (ln2 = r->interfaces.get_head(); ln2!=NULL; ln2 = ln2->next)
	{
		in = (Interface*) ln2->item;
		if ((in->network!=NULL) && (in->network->shown==TRUE) &&
				(is_network_a_leaf(in->network))) l++;
	}

	radius = ((((l<10)?10:l)*96)+64)/(2*3.14159);
	angle = (2*3.14159)/l;

	l = 0;
	for (ln2 = r->interfaces.get_head(); ln2!=NULL; ln2 = ln2->next)
	{
		in = (Interface*) ln2->item;
		if ((in->network!=NULL) && (in->network->shown==TRUE) &&
				(is_network_a_leaf(in->network)))
		{
			in->network->x = r->x + int(radius * cos(angle*l));
			in->network->y = r->y - int(radius * sin(angle*l));
			if (sel) in->network->selection = TRUE;
			l++;
		}
	}
}

void Data::star(Visual *cur, int sel)
{
	Link *ln;
	Device *r;

	if ((cur != NULL) && (cur->maintype==visual_device) && (cur->selection==FALSE))
	{
		select(FALSE);
		stara((Device*)cur, sel);
	}
	for (ln=devices.get_head(); ln!=NULL; ln=ln->next)
	{
		r = (Device*) ln->item;
		if (r->selection==TRUE) stara(r, sel);
	}
}

// 

void Data::star_network(Network *n, int sel)
{
	Link *ln3, *ln2;
	double radius, angle;
	int l, machine_count;
	Interface *in;
	Machine *m;

	if (n->shown==FALSE) return;
	if (sel) n->selection = TRUE;

	l = 0;
	for (ln2 = n->interfaces.get_head(); ln2!=NULL; ln2 = ln2->next)
	{
		in = (Interface*) ln2->item;
		for (ln3 = in->machines.get_head(); ln3!=NULL; ln3 = ln3->next)
		{
			m = (Machine*) ln3->item;
			if (m->shown==TRUE) l++;
		}
	}

	radius = ((((l<10)?10:l)*96)+64)/(2*3.14159);
	angle = (2*3.14159)/l;

	l = 0;
	for (ln2 = n->interfaces.get_head(); ln2!=NULL; ln2 = ln2->next)
	{
		in = (Interface*) ln2->item;
		for (ln3 = in->machines.get_head(); ln3!=NULL; ln3 = ln3->next)
		{
			m = (Machine*) ln3->item;
			if (m->shown==TRUE)
			{
				m->x = n->x + int(radius * cos(angle*l));
				m->y = n->y - int(radius * sin(angle*l));
				if (sel) m->selection = TRUE;
				l++;
			}
		}
	}
}

void Data::bunch_network(Network *n, int sel)
{
	Link *ln, *ln2, *ln3;
	double radius, angle;
	int l, total, ring, machine_count;
	Interface *in;
	Machine *m;

	if (n->shown==FALSE) return;
	if (sel) n->selection = TRUE;

	l = 0;
	for (ln2 = n->interfaces.get_head(); ln2!=NULL; ln2 = ln2->next)
	{
		in = (Interface*) ln2->item;
		for (ln3 = in->machines.get_head(); ln3!=NULL; ln3 = ln3->next)
		{
			m = (Machine*) ln3->item;

			total = ring = 0;
			while (total<l+1) { ring++; total += (8*ring); }

			radius = (((80*ring)+40) * 8)/(2*3.14159);
			angle = 2*3.14159/(ring*8);

			if (m->shown==TRUE)
			{
				m->x = n->x + int(radius * cos(angle*(total-l)));
				m->y = n->y + int(radius * sin(angle*(total-l)));
				if (sel) m->selection = TRUE;
				l++;
			}
		}
	}
}

void Data::group_machines()
{
	Link *ln, *ln2, *ln3;
	Network *n;
	Machine *m;
	Interface *in;
	int count, val, xlen;
	long tx, ty;

	for (ln=networks.get_head(); ln!=NULL; ln=ln->next)
	{
		n = (Network*) ln->item;
		if (n->group==network_group_star) star_network(n, 0); else
		if (n->group==network_group_bunch) bunch_network(n, 0); else
		if (n->group==network_group_bus)
		{
			count = 0;
			for (ln3=n->interfaces.get_head(); ln3!=NULL; ln3=ln3->next)
			{
				in = (Interface*) ln3->item;
				for (ln2=in->machines.get_head(); ln2!=NULL; ln2=ln2->next)
				{
					m = (Machine*) ln2->item;
					if (m->shown==TRUE) count++;
				}
			}
			if (count>0)
			{
				val = 0;
				xlen = (int) sqrt(count)+1;
				for (ln3=n->interfaces.get_head(); ln3!=NULL; ln3=ln3->next)
				{
					in = (Interface*) ln3->item;
					for (ln2=in->machines.get_head(); ln2!=NULL; ln2=ln2->next)
					{
						m = (Machine*) ln2->item;
						if (m->shown==TRUE)
						{
							m->x = n->x + (2 * m->w * (1+(val%xlen)));
							m->y = n->y + (2 * m->h * (1+(val/xlen)));
							val++;
						}
					}
				}
			}
		}
	}
}

void Data::apply_group()
{
	Link *ln;
	Device *r;

	for (ln=devices.get_head(); ln!=NULL; ln=ln->next)
	{
		r = (Device*) ln->item;
		if (r->group==device_group_bunch) buncha(r, 0); else
		if (r->group==device_group_star) stara(r, 0);
	}
}

// show all devices and link networks
void Data::show_all()
{
	Link *ln;
	Device *r;
	Network *n;

	for (ln=devices.get_head(); ln!=NULL; ln=ln->next)
	{
		r = (Device*) ln->item;
		r->shown = TRUE;
	}

	for (ln=networks.get_head(); ln!=NULL; ln=ln->next)
	{
		n = (Network*) ln->item;
		if (n->interfaces.size()>1)
			n->shown = TRUE;
	}
}

//
// work out position of network relative to visible devices and machines
//

void Data::position_links()
{
	Link *ln, *ln2, *ln3;
	Network *n;
	Device *r;
	Machine *m;
	Interface *in;
	long tx, ty;
	int count;

	for (ln=networks.get_head(); ln!=NULL; ln=ln->next)
	{
		n = (Network*) ln->item;
		if ((n->interfaces.size()!=1) && ((n->position==network_position_auto) || (n->subtype==network_p2p)))
		{
			count = 0;
			tx = ty = 0;
			for (ln2=n->interfaces.get_head(); ln2!=NULL; ln2=ln2->next)
			{
				in = (Interface*) ln2->item;
				r = in->device;
				if (r->shown==TRUE)
				{
					tx += r->x;
					ty += r->y;
					count++;
				}
			}
/*
// network position should not be governed by the position of attached machines
			for (ln3=n->interfaces.get_head(); ln3!=NULL; ln3=ln3->next)
			{
				in = (Interface*) ln3->item;
				for (ln2=in->machines.get_head(); ln2!=NULL; ln2=ln2->next)
				{
					m = (Machine*) ln2->item;
					if (m->shown==TRUE)
					{
						tx += m->x;
						ty += m->y;
						count++;
					}
				}
			}
*/
			if (count>0)
			{
				if (n->subtype!=network_vbus) n->x = tx/count;
				if (n->subtype!=network_hbus) n->y = ty/count;
			}
		}
	}
}

gboolean Data::readyToMerge()
{
	Link *ln;
	Device *r;
	Network *n;
	gint count1, count2, count3;

	count1 = count2 = count3 = 0;
	for (ln=devices.get_head(); ln!=NULL; ln=ln->next)
	{
		r = (Device*) ln->item;
		if ((r->selection==TRUE) && (r->maintype==visual_device))
		{
			if (r->subtype==device_router) count1++; else
			if (r->subtype==device_switch) count2++; else count3++;
		}
	}
	return ((count1==1) && (count2==1) && (count3==0))?TRUE:FALSE;
}

