#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <string.h>
#include <stdlib.h>
#include <fcntl.h>
#include <stdio.h>
#include <pwd.h>
#include "misc.h"

int contains(char *str, char *substr)
{
	int a, b, i, j;
	char buf[256];

	a = strlen(str);
	b = strlen(substr);

	for (i=0; i<a-b+1; i++)
	{
		for (j=0; j<b; j++)
		{
			buf[j] = str[j+i];
		}
		buf[b] = '\0';
		if ((strcmp(buf, substr))==0) return 1;
	}

	return 0;
}

int lower_contains(char *str, char *substr)
{
	int a, b, i;
	char buf1[256];
	char buf2[256];

	strcpy(buf1, str);
	strcpy(buf2, substr);

	a = strlen(buf1);
	b = strlen(buf1);

	for (i=0; i<a; i++)
		if ((buf1[i]>='A') && (buf1[i]<='Z')) buf1[i] = buf1[i] - 'A' + 'a';
	for (i=0; i<b; i++)
		if ((buf2[i]>='A') && (buf2[i]<='Z')) buf2[i] = buf2[i] - 'A' + 'a';
	buf1[a] = '\0'; buf2[b] = '\0';

	return contains(buf1, buf2);
}

int starts_with(char *str, char *bigstr)
{
	char buf[256];
	strcpy(buf, bigstr);
	buf[strlen(str)] = '\0';
	return (strcmp(buf, str)==0);
}

/////////////////////////////////////////////////////
// We use our own read/write functions so we don't
// have to worry about endian issues across platforms
/////////////////////////////////////////////////////

void write_int32(FILE *f, int val)
{
	fputc((val & 0xff000000)>>24, f);
	fputc((val & 0xff0000)>>16, f);
	fputc((val & 0xff00)>>8, f);
	fputc((val & 0xff), f);
}

void write_ulong32(FILE *f, unsigned long val)
{
	fputc((val & 0xff000000)>>24, f);
	fputc((val & 0xff0000)>>16, f);
	fputc((val & 0xff00)>>8, f);
	fputc((val & 0xff), f);
}

int read_int32(FILE *f)
{
	int a, b, c, d;
	a = fgetc(f);
	b = fgetc(f);
	c = fgetc(f);
	d = fgetc(f);
	return (((((a*256)+b)*256)+c)*256)+d;
}

unsigned long read_ulong32(FILE *f)
{
	int a, b, c, d;
	a = fgetc(f);
	b = fgetc(f);
	c = fgetc(f);
	d = fgetc(f);
	return (((((a*256)+b)*256)+c)*256)+d;
}

///////////////////////////////////////////////////////////
// we also have 64 bit versions of the read/write
// functions for future expansion
///////////////////////////////////////////////////////////

void write_int64(FILE *f, int val)
{
	fputc(0, f);
	fputc(0, f);
	fputc(0, f);
	fputc(0, f);
	fputc((val & 0xff000000)>>24, f);
	fputc((val & 0xff0000)>>16, f);
	fputc((val & 0xff00)>>8, f);
	fputc((val & 0xff), f);
}

void write_ulong64(FILE *f, unsigned long val)
{
	fputc(0, f);
	fputc(0, f);
	fputc(0, f);
	fputc(0, f);
	fputc((val & 0xff000000)>>24, f);
	fputc((val & 0xff0000)>>16, f);
	fputc((val & 0xff00)>>8, f);
	fputc((val & 0xff), f);
}

int read_int64(FILE *f)
{
	int a, b, c, d;
	// we throw away the first 4 bytes
//	fseek(f, 4L, SEEK_CUR);
	a = fgetc(f);
	a = fgetc(f);
	a = fgetc(f);
	a = fgetc(f);
	a = fgetc(f);
	b = fgetc(f);
	c = fgetc(f);
	d = fgetc(f);
	return (((((a*256)+b)*256)+c)*256)+d;
}

unsigned long read_ulong64(FILE *f)
{
	int a, b, c, d;
	// we throw away the first 4 bytes
//	fseek(f, 4L, SEEK_CUR);
	a = fgetc(f);
	a = fgetc(f);
	a = fgetc(f);
	a = fgetc(f);
	a = fgetc(f);
	b = fgetc(f);
	c = fgetc(f);
	d = fgetc(f);
	return (((((a*256)+b)*256)+c)*256)+d;
}

int send_email(char *address, char *subject, char *message)
{
	FILE *f;

	f=popen(MAILCMD, "w");
	if (!f) return 0;

	if (f)
	{
		fprintf(f, "HELO\n");
		fprintf(f, "MAIL FROM:<%s>\n", SENDER);
		fprintf(f, "RCPT TO:<%s>\n", address);
		fprintf(f, "DATA\n");
		fprintf(f, "From: %s\n",SENDER);
		fprintf(f, "Subject: %s\n", subject);
		fprintf(f, "Sender: %s\n",SENDER);
		fprintf(f, "To: %s\n", address);
		fprintf(f, "\n\n");
		fprintf(f, "%s\n", message);
		fprintf(f, "\n\n.\n");
		fprintf(f, "QUIT\n");
		pclose(f);
	}
	return 1;
}

// Searches a number of possible locations for the file
// returns full path to file or NULL if not found

char *user_file(char *fname)
{
	struct stat statbuf;
	struct passwd *pwd;
	char *name[7];
	char *file = NULL;
	char buf[256];
	FILE *f;
	int i;

	pwd = getpwuid(getuid());
	sprintf(buf, "%s/%s/%s", pwd->pw_dir, DATA_DIR, fname);
	name[0] = strdup(buf);
	name[1] = strdup(fname);
	sprintf(buf, "../%s", fname);
	name[2] = strdup(buf);
	sprintf(buf, "ui/%s", fname);
	name[3] = strdup(buf);
	sprintf(buf, "../ui/%s", fname);
	name[4] = strdup(buf);
	sprintf(buf, "%s/nomad/src/ui/%s", pwd->pw_dir, fname);
	name[5] = strdup(buf);
	sprintf(buf, "%s/%s", SHARE_DIR, fname);
	name[6] = strdup(buf);

	for (i=0; i<7; i++)
	{
		if (stat(name[i], &statbuf)==0)
		{
			if (file) free(file);
			file = strdup(name[i]);
			i=6;
		}
	}

	for (i=0; i<7; i++)
	{
		free(name[i]);
	}

	return file;
}

gint32 timestr_to_int(gchar *str)
{
	gchar **sarray;
	gint32 num, u, tmp;

	if (str==NULL) return 0;

	sarray = g_strsplit(str, " ", 2);
	if (sarray[0]==NULL) return 0;
	if (sarray[1]==NULL) return 0;
	sscanf(sarray[0], "%d", &num);
	if (strcmp(sarray[1], "Seconds")==0) u=1; else
	if (strcmp(sarray[1], "Minutes")==0) u=60; else
	if (strcmp(sarray[1], "Hours")==0) u=3600; else
	if (strcmp(sarray[1], "Days")==0) u=86400; else
	if (strcmp(sarray[1], "Weeks")==0) u=604800; else
	if (strcmp(sarray[1], "Years")==0) u=31449600; else u=0;

	tmp = (0xffffffff / u)-1;
	if (num<0) return 0;
	if (num>tmp) return 0;

	return (num*u);
}

gchar *int_to_timestr(gint32 num)
{
	static gchar str2[16];
	gchar str1[16];
	gint32 u;

	if ((num%31449600)==0) { sprintf(str1, "Years"); u=31449600; } else
	if ((num%604800)==0) { sprintf(str1, "Weeks"); u=604800; } else
	if ((num%86400)==0) { sprintf(str1, "Days"); u=86400; } else
	if ((num%3600)==0) { sprintf(str1, "Hours"); u=3600; } else
	if ((num%60)==0) { sprintf(str1, "Minutes"); u=60; } else
	{ sprintf(str1, "Seconds"); u=1; }

	sprintf(str2, "%d %s", (num/u), str1);
	return (str2);
}

int linesInFile(char *fname)
{
	int fd, linecount, len;
	u_char *C, buf[1024];

	if ((fd = open(fname, O_RDONLY, 0)) < 0) return 0;
	linecount=1;
	while ((len = read(fd, buf, 1024)) > 0)
	{
		for (C = buf; len--; ++C)
			if (*C == '\n')
				++linecount;
	}
	return linecount;
}

gboolean validFilename(gchar *str)
{
	gint i, len;
	gboolean state;

	len = strlen(str);
	for (i=0; i<len; i++)
	{
		state = FALSE;
		if (((str[i]>='a') && (str[i]<='z')) ||
				((str[i]>='A') && (str[i]<='Z')) ||
				((str[i]>='0') && (str[i]<='9')) ||
				(str[i]=='_') || (str[i]=='-'))
			state = TRUE;
		if (state==FALSE) return FALSE;
	}
	return TRUE;
}

/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////

DeviceType stringToDeviceType(gchar *str)
{
	DeviceType t;
	if (strcmp(str, "switch")==0) t = device_switch; else
	if (strcmp(str, "router")==0) t = device_router; else
	if (strcmp(str, "bridge")==0) t = device_bridge; else
	if (strcmp(str, "hub")==0) t = device_hub; else
	if (strcmp(str, "merged")==0) t = device_merged; else
	if (strcmp(str, "placeholder")==0) t = device_placeholder; else
		t = device_unknown;
	return t;
}

gchar *deviceTypeToString(DeviceType subtype)
{
	static gchar buf[16];
	if (subtype==device_switch) strcpy(buf, "switch"); else
	if (subtype==device_router) strcpy(buf, "router"); else
	if (subtype==device_bridge) strcpy(buf, "bridge"); else
	if (subtype==device_hub)		strcpy(buf, "hub"); else
	if (subtype==device_merged) strcpy(buf, "merged"); else
	if (subtype==device_placeholder) strcpy(buf, "placeholder"); else
		strcpy(buf, "unknown");
	return buf;
}

DeviceGroupType stringToDeviceGroupType(gchar *str)
{
	DeviceGroupType t;
	if (strcmp(str, "bunch")==0) t = device_group_bunch; else
	if (strcmp(str, "star")==0) t = device_group_star; else
		t = device_group_none;
	return t;
}

gchar *deviceGroupTypeToString(DeviceGroupType subtype)
{
	static gchar buf[16];
	if (subtype==device_group_bunch) strcpy(buf, "bunch"); else
	if (subtype==device_group_star) strcpy(buf, "star"); else
		strcpy(buf, "none");
	return buf;
}

NetworkType stringToNetworkType(gchar *str)
{
	NetworkType subtype;
	if (strcmp(str, "vbus")==0) subtype = network_vbus; else
	if (strcmp(str, "hbus")==0) subtype = network_hbus; else
	if (strcmp(str, "p2p")==0) subtype = network_p2p; else
	if (strcmp(str, "ring")==0) subtype = network_ring; else
	if (strcmp(str, "cloud")==0) subtype = network_cloud; else
		subtype = network_auto;
	return subtype;
}

gchar *networkTypeToString(NetworkType subtype)
{
	static gchar buf[16];
	if (subtype==network_vbus) strcpy(buf, "vbus"); else
	if (subtype==network_hbus) strcpy(buf, "hbus"); else
	if (subtype==network_p2p) strcpy(buf, "p2p"); else
	if (subtype==network_ring) strcpy(buf, "ring"); else
	if (subtype==network_cloud) strcpy(buf, "cloud"); else
		strcpy(buf, "automatic");
	return buf;
}

NetworkPositionType stringToNetworkPositionType(gchar *str)
{
	NetworkPositionType subtype;
	if (strcmp(str, "manual")==0) subtype = network_position_manual; else
		subtype = network_position_auto;
	return subtype;
}

gchar *networkPositionTypeToString(NetworkPositionType position)
{
	static gchar buf[16];
	if (position==network_position_manual) strcpy(buf, "manual"); else
		strcpy(buf, "automatic");
	return buf;
}

NetworkGroupType stringToNetworkGroupType(gchar *str)
{
	NetworkGroupType subtype;
	if (strcmp(str, "bus")==0) subtype = network_group_bus; else
	if (strcmp(str, "star")==0) subtype = network_group_star; else
	if (strcmp(str, "bunch")==0) subtype = network_group_bunch; else
		subtype = network_group_none;
	return subtype;
}

gchar *networkGroupTypeToString(NetworkGroupType group)
{
	static gchar buf[16];
	if (group==network_group_bus) strcpy(buf, "bus"); else
	if (group==network_group_star) strcpy(buf, "star"); else
	if (group==network_group_bunch) strcpy(buf, "bunch"); else
		strcpy(buf, "none");
	return buf;
}

